# -*- coding:utf-8 -*-
from qgis.core import (Qgis, QgsCoordinateReferenceSystem, QgsProject, QgsVectorFileWriter, QgsVectorLayer)
from qgis.gui import (QgsBusyIndicatorDialog, QgsCodeEditorSQL, QgsCollapsibleGroupBox, QgsDateTimeEdit,
                      QgsMessageBar, QgsProjectionSelectionDialog)
   
from qgis.PyQt.QtCore import (QAbstractTableModel, QDataStream, QDate, QDir, QEvent, QFileInfo, QIODevice,
                              QItemSelectionModel, QLocale, QModelIndex, QObject, QPoint, QSettings, QSize,
                              QRect, QRegExp,  QSortFilterProxyModel, QStringListModel, Qt, QTranslator, QUrl, pyqtSignal) 
from qgis.PyQt.QtGui import (QBrush, QColor, QFont, QFontMetrics, QIcon, QImage, QKeySequence, QLinearGradient,
                             QMovie, QPainter, QPixmap, QRegExpValidator, QStandardItem, QStandardItemModel, QTextDocument, QTextCursor, QTextOption)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.PyQt.Qsci import (QsciAPIs, QsciLexerXML, QsciScintilla, QsciScintillaBase, QsciStyle)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QButtonGroup, QCheckBox, QCompleter,
                         QComboBox, QCommandLinkButton, QDialog, QDockWidget, QDialogButtonBox, QDoubleSpinBox, QFileDialog, QFileSystemModel, QFrame,
                         QGridLayout, QGroupBox, QHBoxLayout, QInputDialog, QLabel, QLineEdit, QListView, QListWidget, QListWidgetItem, QMdiArea,
                         QMdiSubWindow, QMenu, QMessageBox, QMdiSubWindow, QProgressBar,QPushButton, QRadioButton, QSizePolicy, QSlider,
                         QSpinBox, QSplitter, QTabBar, QTabWidget, QTableView, QTableWidget, QTableWidgetItem, QTextBrowser,
                         QTextEdit, QTreeView, QTreeWidget, QTreeWidgetItem, QToolBar, QToolButton, QVBoxLayout, QWidget)

from ast import literal_eval
from configparser import ConfigParser
from datetime import datetime, date
from decimal import Decimal
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import numpy as np
from os.path import (dirname, exists, join)
import random
import sys
if sys.version_info < (3,): memoryview = buffer
from textwrap import fill
from urllib.parse import (quote, quote_plus)
from webbrowser import open as webbrowseropen


def myMdiSubWindow(parent, name, title, width, height):
        
        self = QMdiSubWindow(parent)
        self.setObjectName(name)
        self.setWindowTitle(title)
        self.setWindowFlags(Qt.WindowMinimizeButtonHint)
        self.setFixedSize(width, height)
        self.setGeometry(10, 10, width, height)
        self.setStyleSheet("QMdiSubWindow {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")
        self.oldPos = self.pos()

        return self

def ctrlMap(self, conteneur, complete, getPointsForExtent, simply, iX, iY, iW, iH):
         from qsphere.qsphere_tools import getThemeIcon
         
         self.groupMap = QGroupBox(conteneur)
         self.actionsMapCanvas = QButtonGroup(conteneur) 
         self.actionsMapCanvas.setExclusive(True)
         

         dicoCtrls = {"buttonZoomIn" : ("mapview/zoomin.png", "Zoom in the map window", 10, 2),
                      "buttonZoomOut" : ("mapview/zoomout.png", "Zoom out the map window", 40, 2),
                      "buttonZoomAll" : ("mapview/zoomall.png", "Zoom all the map window", 70, 2),
                      "buttonZoomPrevious" : ("mapview/zoomprev.png", "Previous zoom", 100, 2),
                      "buttonZoomNext" : ("mapview/zoomnext.png", "Next zoom", 130, 2),
                      "buttonPan" : ("mapview/pan.png", "Move the map window", 160, 2),
                      "buttonPropertiesLayer" : ("mapview/properties.png", "Open properties widget for the layer", 220, 2),
                      "moveDown" : ("movedown.png", "Move down", 190, 2),
                      "moveUp" : ("moveup.png", "Move up", 220, 2),
                      "refreshExtent" : ("mapview/loadextent.png", "Refresh map with current extent", 250, 2),
                      "buttonSendToExtent" : ("mapview/copyextent.png", "Send to", 280, 2),
                      "addAfter_6_tableemprises" : ("mapview/addextent.png", "Add an extent", 310, 2)
                      }
         pos = 10       
         for key in dicoCtrls :
                 cond = False
                 
                 if key in ("buttonZoomIn", "buttonZoomOut", "buttonZoomAll", "buttonZoomPrevious", "buttonZoomNext", "buttonPan") and not simply : cond = True
                 elif key == "buttonPropertiesLayer" and complete : cond = True
                 elif key in ("moveDown", "moveUp", "refreshExtent", "buttonSendToExtent", "addAfter_6_tableemprises") and getPointsForExtent : cond = True

                 if cond :        
                         icon = getThemeIcon(dicoCtrls[key][0])
                         toolTip = QApplication.translate("QSphere", dicoCtrls[key][1])
                         button = myPushButton(self.groupMap)
                         button.initPushButton(25, 25, pos, dicoCtrls[key][3], key, "", toolTip, True, icon, 25, 25, True)


                         if key in ("buttonZoomIn", "buttonZoomOut","buttonPan", "buttonToolTip", "buttonHotlink"):

                              styleSheetPushButton = "QPushButton::checked {icon-size: 25px, 25px; border-width: 5px; border-radius: 10px; border-color: black; background-color: #FFFFCC;}"
                              styleSheetPushButton+= "QPushButton {icon-size: 25px, 25px; border-width: 0px; border-radius: 10px; }"
                              button.setStyleSheet(styleSheetPushButton)
                              button.setCheckable(True)

                              if key == "buttonZoomIn" : button.setChecked(True)
                             
                              index = ("buttonZoomIn", "buttonZoomOut","buttonPan", "buttonToolTip", "buttonHotlink").index(key)
                              self.actionsMapCanvas.addButton(button, index)

                         
                         setattr(self, key, button)
                         pos+= 30

         sizeWidth = (len(self.groupMap.children())-1)*35
         self.groupMap.setGeometry(10, 30, sizeWidth+30, 35) 

         return sizeWidth



class dlgBigProcessSql(QDialog):
       def  __init__(self, parent, cursor):
         super(dlgBigProcessSql, self).__init__()
         self.setWindowTitle("SQL")
         self.minWidth, self.minHeight = 500, 450
         self.setMinimumSize(self.minWidth,self.minHeight)
         self.setMaximumSize(QSize(self.minWidth,self.minHeight))
         self.setWindowFlags(Qt.WindowStaysOnTopHint)

         from qsphere.qsphere_tools import windowIcon
         windowIcon(self, "sgbdr/sql.png")

         self.parent = parent
         self.iface = parent.iface
         self.cursor = cursor
         self.startTheProcess = False
         
         layout = QVBoxLayout(self)

         self.groupBox = QGroupBox(self)
         self.groupBox.setGeometry(5, 5, 450, 400)

         self.lblNameTable = QLabel(self.groupBox )
         self.lblNameTable.setGeometry(25, 20, 90, 25)
         self.lblNameTable.setText("%s : " % QApplication.translate("QSphere", "Name table"))
         self.lblNameTable.setAlignment(Qt.AlignRight)

         self.nameTable = myWidgetField(self.groupBox)
         self.nameTable.initType(4)
         self.nameTable.setGeometry(125, 15, 300, 25)
         
         self.codeSql = QgsCodeEditorSQL(self.groupBox)
         self.codeSql.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
         self.codeSql.setMarginVisible(True)
         self.codeSql.setWrapMode(QsciScintilla.WrapWord)
         self.codeSql.setGeometry(5, 60, 460, 300)
         self.codeSql.setText("")
         self.codeSql.lexer().setAPIs(self.parent.editorSql.lexer().apis())
         self.codeSql.setReadOnly(True)
         
         self.progressBar = myProgressBar(self)
         self.progressBar.initTextEdit(25, 45, 450, 80, "progressBar", True, 0, True, Qt.AlignCenter, 0)
         self.progressBar.setStyleSheet("QProgressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
                                        "QProgressBar::chunk {background-color: #6C96C6; width: 20px;}")

         self.movieCtrl = movieCtrl(self, "sablier.gif")
         self.counter = self.movieCtrl.movie.frameCount()
         self.movieCtrl.setGeometry(self.width()/2-32, self.height()/2-32, 64, 64)

         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Close"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Start"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.startProcess)
         self.buttonBox.rejected.connect(self.reject)
         
         layout.addWidget(self.groupBox)
         layout.addWidget(self.progressBar) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)

       def updateSql(self, cursor, sql, msg):
           if sql : self.codeSql.setText("%s\n%s" % (self.codeSql.text(), sql))
           if msg : self.codeSql.setText("%s\n%s" % (self.codeSql.text(), msg))
           self.codeSql.setCursorPosition(self.codeSql.lines()-1, 0)
           self.codeSql.repaint()
            
           if sql :
                   self.movieCtrl.startMovie()
                   from qsphere.qsphere_tools import multiGetRequest
                   isValidSQL = multiGetRequest(self.movieCtrl.movie, self.movieCtrl, [(sql, cursor)], 10000000000, 2)
                   self.movieCtrl.stopMovie()

       def endProcess(self): self.startTheProcess = False; self.acceptButton.setEnabled(True); 
       
       def startProcess(self) : 
           if self.nameTable.text() == "" : QMessageBox.information(None,"dev", "Must name table !"); return ;
           self.startTheProcess = True
           if self.parent : self.acceptButton.setEnabled(False); self.parent.startProcess(self.cursor); 


class filesNavigatorWidget(QDockWidget):
    def __init__(self, parent):
        super(filesNavigatorWidget, self).__init__()
        self.parent = parent
        self.duration_warning = self.parent.duration_warning
        self.barInfo = self.parent.barInfo
        self.localLangage = self.parent.localLangage
        self.setupUi()

    def setupUi(self):
        from qsphere.qsphere_tools import (getThemeIcon, windowIcon)

        self.setFloating(False)
        self.setAllowedAreas(Qt.NoDockWidgetArea)
        self.setTitleBarWidget(QWidget())
 
        self.dirModel = QFileSystemModel()
        self.dirModel.setFilter(QDir.NoDotAndDotDot | QDir.AllDirs)
        self.dirModel.setRootPath(QDir.homePath()) 

        self.lastPosX, self.lastPosY = 10, 60
        
        self.setObjectName("filesNavigatorWidget")
        self.setMinimumSize(200, 200)
               
        self.frame = QFrame(self)
                                    
        self.masterHLayout = QHBoxLayout(self.frame)
        self.masterHLayout.setObjectName("masterHLayout")
        self.splitter = QSplitter(self.frame)
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setObjectName("splitter")
            
        self.treeView = QTreeView(self.splitter)
        self.treeView.setObjectName("treeView")
        self.treeView.setGeometry(10, 10, 380, 780)
        self.treeView.setAlternatingRowColors(True)
        self.treeView.setSelectionBehavior(QAbstractItemView.SelectItems)
        self.treeView.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.treeView.setAnimated(False)
        self.treeView.setAllColumnsShowFocus(True)
        self.treeView.setModel(self.dirModel)
        for i in range(4, 0, -1): self.treeView.hideColumn(i)

        self.treeView.clicked[QModelIndex].connect(self.updateListView)
           
        self.fileModel = QFileSystemModel()
        self.fileModel.setFilter(QDir.Files) 
        self.fileModel.setRootPath(QDir.homePath())
        self.fileModel.setNameFilters(("*.xml", "*.qmd", "*.qsp", "*.qss"))
        self.fileModel.setNameFilterDisables(False)

        self.listView = QListView(self.splitter)
        self.listView.setObjectName("listView")
        self.listView.setGeometry(410, 10,380, 780)
        self.listView.setStyleSheet("QListView {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}"
                                    "subcontrol-position: top right; width:50px; border-image: url('outils.png');}" "border-width: 1px; width:25px;}")
        self.listView.setSelectionMode(QAbstractItemView.SingleSelection)
        self.listView.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.listView.setDragEnabled(True)

        self.masterHLayout.addWidget(self.splitter)

        self.listView.doubleClicked.connect(self.loadFile)

        self.visibilityChanged.connect(self.updateIconToolBar)

          
    def resizeEvent(self, event):
        self.frame.setGeometry(5, 5, self.width()-10, self.height()-10)
        self.treeView.setGeometry(5, 5, self.treeView.width(), self.height()-40)   
        self.listView.setGeometry(self.treeView.width()+30, 10, self.width()-self.treeView.width()-60, self.height()-40)
    
    def updateIconToolBar(self):
        from qsphere.qsphere_tools import getThemeIcon   
        state = not self.isVisible()
        self.parent.butToolFiles.setIcon(QIcon(getThemeIcon("filesnavigator_%s.png".lower() % state)))
        msg = "View files navigator" if state else "Mask files navigator"
        self.parent.butToolFiles.setToolTip(QApplication.translate("QSphere",msg))

    def updateListView(self, index):
        self.path = self.dirModel.fileInfo(index).absoluteFilePath()
        inputFile = QFileInfo(self.path)
        if inputFile.isDir():
                self.fileModel.setRootPath("%s/" % self.path)
                self.listView.setRootIndex(self.fileModel.setRootPath(self.path))
                self.listView.setModel(self.fileModel)
    
    def loadFile(self, index):
        from qsphere.qsphere_tools import (sendMsg, testXmlData)
        fileName = "%s/%s" % (self.path, index.data(Qt.DisplayRole))
        inputFile = QFileInfo(fileName)
        if inputFile.isFile():
           extension = inputFile.suffix().lower()     
           if extension == "xml" :
                test, data = testXmlData(self, fileName, False) if extension == "xml" else True, None
                if test[0] : 
                        result = self.parent.loadMetadataFile(fileName, False, -1)
                        if self.parent.result :
                           self.parent.InitDir = dirname(fileName)
                           self.parent.ajustListInfos(fileName, self.parent.mddFiles)
                else :
                    title = QApplication.translate("QSphere","Information")
                    msg = QApplication.translate("QSphere","Error on unsupported content :") 
                    sendMsg(self, title , "%s :\n%s" % (msg, fileName), Qgis.Warning, self.duration_warning)

           elif extension in ("qmd", "qsp") :     
                    result = self.parent.loadMetadataFile(fileName, False, -1)
                    if self.parent.result :
                       self.parent.InitDir = dirname(fileName)
                       self.parent.ajustListInfos(fileName, self.parent.mddFiles)
                           
           elif extension == "qss" :
                  text = QApplication.translate("QSphere","Load a session file")
                  result = self.parent.loadTheSession(fileName, self.sender().icon(), text)
                  if result : self.parent.ajustListInfos(fileName, self.parent.listSessions)
      
        

class expandableGroupBox(QGroupBox):
     def __init__(self, *args): super(expandableGroupBox, self).__init__(*args)

     def initGroupBox(self, parent, name, posX, posY, minimumWidth, minimumHeight, maximumWidth, maximumHeight, styleSheet):
            from qsphere.qsphere_tools import getThemeIcon
            self.parent = parent
            self.posX, self.posY, self.minimumWidth, self.minimumHeight, self.maximumWidth, self.maximumHeight = posX, posY, minimumWidth, minimumHeight, maximumWidth, maximumHeight
            self.setGeometry(self.posX, self.posY, self.minimumWidth, self.minimumHeight)
            self.setObjectName(name)
            self.state = 1

            if styleSheet :
               self.setStyleSheet("QGroupBox {border: 2px solid grey; border-radius: 5px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #ffffcc, stop: 1 #d2e8ff); margin: 0.5px;}")

            self.resizeButton = myPushButton(self)
            self.resizeButton.initPushButton(self.minimumWidth, self.minimumHeight, self.maximumWidth-self.minimumWidth, 0, "resizeButton", "", QApplication.translate("QSphere","Minimize size"), True, getThemeIcon("redimmin.png"), self.minimumWidth, self.minimumHeight, True)     
            
            self.resizeButton.clicked.connect(self.resizeGroupBox)

     def updateSizes(self, posX, posY): self.posX = posX; self.posY = posY

     def resizeGroupBox(self):
         from qsphere.qsphere_tools import getThemeIcon
         if self.state == 1 :
             self.setGeometry(self.parent.width()-self.minimumWidth, self.posY, self.minimumWidth, self.minimumHeight)
             self.resizeButton.setGeometry(2, 0,self.minimumWidth, self.minimumHeight) 
             self.resizeButton.setToolTip(QApplication.translate("QSphere","Maximize size"))
             self.state = 0      
         else :
             self.setGeometry(self.posX, self.posY, self.maximumWidth, self.maximumHeight)
             self.resizeButton.setGeometry(self.maximumWidth-self.minimumWidth+2, 0,self.minimumWidth, self.minimumHeight) 
             self.resizeButton.setToolTip(QApplication.translate("QSphere","Minimize size"))
             self.state = 1
         icon = getThemeIcon("redimmin.png") if self.state == 1 else getThemeIcon("redimmax.png")   
         self.resizeButton.setIcon(QIcon(icon))
         

class myPushButtonMappable(QPushButton):
     def __init__(self, *args): super(myPushButtonMappable, self).__init__(*args)
            
     def initPushButton(self, parent, sizeWidth, sizeHeight, posX, posY, name, text, toolTip, geom, icon, iconSizeWidth, iconSizeHeight, styleSheet):
            self.parent = parent
            self.setMinimumSize(sizeWidth, sizeHeight)
            self.setMaximumSize(sizeWidth, sizeHeight)
            if geom : self.setGeometry(posX, posY, sizeWidth, sizeHeight)
            if icon != "" :
               self.setIcon(QIcon(icon))
               self.setIconSize(QSize(iconSizeWidth, iconSizeHeight))
            self.setToolTip(toolTip)
            if styleSheet : self.setStyleSheet("QPushButton {border: none;}")
            self.setObjectName(name)
            self.isMappable, self.MainPlugin = False, None
            if text != "" : self.setText(text)

     def mousePressEvent(self, event):
            if self.isMappable :
               if event.button() == Qt.LeftButton:
                  pos, index = event.pos().x(), -1
                  if 65 <= pos <= 90     : index = 0 #("fr")
                  elif 91 <= pos <= 110  : index = 1 #("en")
                  elif 111 <= pos <= 130 : index = 2 #("it")
                  elif 131 <= pos <= 155 : index = 3 #("de")
                  elif 156 <= pos <= 175 : index = 4 #("es")
                  elif 176 <= pos <= 200 : index = 5 #("fi")

                  if index != -1 :
                     if self.qspherePlugin != None :    
                            self.qspherePlugin.indexLang = index
                            self.parent.indexLang = index
                            self.parent.changeLang()
                     else :
                            from qsphere.qsphere_tools import sendMsg      
                            title = QApplication.translate("QSphere", "Warning" ) 
                            msg = QApplication.translate("QSphere","Can only change the language by MainPlugin !")
                            sendMsg(self.parent, title, "<font color='#ff0000'><b>%s</b></font>" % (msg), Qgis.Warning, self.parent().duration_warning)


class printerSphere(object):
        def __init__(self, parent, obj, title, orientation):
                from qsphere.qsphere_tools import windowIcon
                    
                printer = QPrinter() 
                printer.setPageSize(QPrinter.A4)
                printer.setOrientation(orientation)
                printer.setPageMargins(5, 10, 5, 10, QPrinter.Millimeter)
                printer.setOrientation(orientation)
                printer.setOutputFormat(QPrinter.NativeFormat)
                self = QPrintPreviewDialog(printer)
                windowIcon(self, "print.png")
                self.setWindowTitle(title)
                self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowStaysOnTopHint | Qt.WindowCloseButtonHint)
                self.paintRequested.connect(obj.print_)
                self.exec_()
        

class windowPlotStatsAndGraphs(QDialog):
    def __init__(self, parent, typeStats, data, labels, obj):
        super(windowPlotStatsAndGraphs, self).__init__()
        self.parent = parent
        self.typeStats = typeStats
        self.data = data
        self.labels = labels
        self.obj = obj

        self.minWidth, self.minHeight = 1000, 700
        self.resize(self.minWidth, self.minHeight)

        from qsphere.qsphere_tools import windowIcon
        from matplotlib.figure import Figure as pltFigure

        windowIcon(self, "histogramchart.png") if self.typeStats == "bar" else  windowIcon(self, "radarchart.png")
        
        self.setWindowTitle("%s : %s" % (QApplication.translate("QSphere","Graph statistics"), self.parent.windowTitle()))

        self.figure = pltFigure()
        self.canvas = FigureCanvas(self.figure)
        self.toolbar = NavigationToolbar(self.canvas, self)

        layout = QVBoxLayout()
        layout.addWidget(self.toolbar)
        layout.addWidget(self.canvas)
        self.setLayout(layout)

        self.plot()

    def plot(self):

        self.figure.clear()            
        if self.typeStats == "bar" :   
           n = len(self.data)     
           ind = np.arange(n)
           ax = self.figure.add_subplot(111)
           ax.set_ylabel("percent")
           ax.set_title("QSphere : %s" % QApplication.translate("QSphere", "Percent by tab"))
           ax.bar(ind, self.data, width=0.35, color='b')
           xticksNames = ax.set_xticklabels(self.labels)
           percents = tuple([float((i/n)) for i in range(n)])
           ax.set_yticklabels(percents, minor=0.25)
          
        elif self.typeStats == "pie" :
           explode = tuple([0 for elt in self.data])
           ax = self.figure.add_subplot(111)
           ax.pie(self.data, explode=explode, labels=self.labels, autopct='%1.1f%%', shadow=True, startangle=90)
           ax.axis('equal')  
                
        self.canvas.draw()



class movieCtrl(QLabel):

      def __init__(self, parent, movie, fadding=False):
              super(movieCtrl, self).__init__(parent)
              from qsphere.qsphere_tools import getThemeIcon
              self.parent = parent
              self.fadding = fadding
              self.movie = QMovie(getThemeIcon(movie))
              self.setMovie(self.movie)
              self.setLayout(QHBoxLayout())
              self.layout().addWidget(QLabel(''))
              self.setVisible(False)

      def startMovie(self):
              self.setVisible(True)
              self.movie.start()
              self.repaint()
              if self.fadding : self.parent.setWindowOpacity(0.5)

      def stopMovie(self):
              self.movie.stop()
              self.setVisible(False)
              if self.fadding : self.parent.setWindowOpacity(1.0)

              

class geoLocalisatorDialog(QDialog):
                
      def  __init__(self, parent, emprise, widget, row, synchronize, formWidgetDialog, badNetwork):
                super(geoLocalisatorDialog, self).__init__()
  
                from qsphere.qsphere_tools import (fileRessourceExist, getThemeIcon, getListWmsServer, windowIcon)
                from os.path import basename  
                
                
                self.minWidth, self.minHeight = 800, 800
                self.parent = parent
                self.formWidgetDialog = formWidgetDialog
                self.duration_warning = self.parent.duration_warning
                self.emprise = emprise
                self.widget = widget
                self.row = row

                self.setWindowTitle("%s ... %s : %s / %s" % (QApplication.translate("QSphere","Tiles map services"), QApplication.translate("QSphere","Item").lower(), self.row+1, self.widget.rowCount()))
                
                self.synchronize = synchronize
                self.badNetwork = badNetwork
                self.iface = self.parent.iface
                self.rasterLayer = None
                self.setMinimumSize(self.minWidth,self.minHeight)
                self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
                layout = QVBoxLayout(self)

                windowIcon(self, "voir.png")
                
                if self.badNetwork :
                    from qgis.core import (QgsMapLayer, QgsProject, QgsRasterLayer, QgsRectangle)  
                    from qgis.gui import (QgsMapCanvas, QgsMapToolIdentify, QgsMapToolPan, QgsMapToolZoom)
                    from qgis.core import (Qgis, QgsCoordinateReferenceSystem, QgsCoordinateTransform)

                    self.mapViewer = QgsMapCanvas(self)
                    self.mapViewer.setAccessibleDescription("")
                    self.mapViewer.setGeometry(0, 0, self.width(), self.height())

                    self.transformer = QgsCoordinateTransform() 
                    self.transformer.setSourceCrs(QgsCoordinateReferenceSystem(4326))
                    self.transformer.setDestinationCrs(QgsCoordinateReferenceSystem(3857))
                    self.mapViewer.setDestinationCrs(self.transformer.destinationCrs())

                    #0 : upperLeftY, 1 : lowerRightY, 2 : UpperLeftX, 3 : lowerRightX
                    box = QgsRectangle(self.emprise[2], self.emprise[1], self.emprise[3], self.emprise[0])
                    box = self.getTransform(box, 0)

                    self.styleSheetTextEditCoords = "QLineEdit { font-size: 12px;  text-align: center; color : white; background-color: rgba(160, 160, 160, 125); border: 2px solid cyan;  border-radius: 4px;  padding: 2px;}"

                    lineInfosCoord = {"upperLeftX" : 2, "lowerRightY" : 1, "lowerRightX" : 3, "upperLeftY" : 0}
                    for key in lineInfosCoord :
                            obj = QLineEdit(self.mapViewer)
                            obj.setAlignment(Qt.AlignCenter)
                            obj.setStyleSheet(self.styleSheetTextEditCoords)
                            obj.setText("%s" % self.emprise[lineInfosCoord[key]])
                            obj.setEnabled(False)
                            setattr(self, key, obj)
  
                    self.widgetMapView = myMdiSubWindow(self.mapViewer, "widgetMapView", "%s :" % QApplication.translate("QSphere","Actions ..."), 280, 80 )
                    sizeWidth = ctrlMap(self, self.widgetMapView, False, True, False, 5, 5, 340, 35)
                    self.widgetMapView.setFixedSize(sizeWidth+30, self.widgetMapView.height())
                    self.widgetMapView.setGeometry(self.widgetMapView.x(), 35, sizeWidth+30, self.widgetMapView.height())
                    
                else :
                    from qgis.PyQt.QtNetwork import (QNetworkAccessManager, QNetworkReply, QNetworkRequest, QNetworkCookieJar)
                    from PyQt5.QtWebKitWidgets import (QWebPage, QWebView)
                    from PyQt5.QtWebKit import QWebSettings
                    
                    self.mapViewer = QWebView(self) 
                    self.mapViewer.settings().setAttribute(QWebSettings.JavaEnabled, True)
                    self.mapViewer.settings().setAttribute(QWebSettings.PluginsEnabled, True)
                    self.mapViewer.settings().setAttribute(QWebSettings.AutoLoadImages, True)
                    self.mapViewer.settings().setAttribute(QWebSettings.LocalContentCanAccessRemoteUrls, True)
                    self.mapViewer.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
                    self.mapViewer.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
                    self.mapViewer.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
                    self.mapViewer.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
                    self.mapViewer.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
                    self.mapViewer.page().setForwardUnsupportedContent(True)
                    self.manager = QNetworkAccessManager(self.mapViewer)
                    self.jar = QNetworkCookieJar(self.manager)
                    self.manager.setCookieJar(self.jar)
                   
                    self.widgetMapView = myMdiSubWindow( self.mapViewer, "widgetConsole", "%s :" % QApplication.translate("QSphere","Actions ..."), 200, 80 )
                    sizeWidth = ctrlMap(self, self.widgetMapView, False, True, True, 45, 5, 180, 35)
        
        
                    self.loadGeolocalisator()


                self.barInfo = QgsMessageBar(self)
                self.barInfo.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)  

                if self.badNetwork :
                    self.widgetParamMapView = myMdiSubWindow(self.mapViewer, "widgetParamMapView", "%s ...:" % QApplication.translate("QSphere","Params"), 360, 120)
                    self.widgetParamMapView.setFixedSize(self.widgetParamMapView.width(), self.widgetParamMapView.height())
                    self.widgetParamMapView.setGeometry(self.width()-self.widgetParamMapView.width()-10, 35, self.widgetParamMapView.width(), self.widgetParamMapView.height()) 

                    self.listeWmsServer = getListWmsServer()
                    self.nameWmsServer = QComboBox(self.widgetParamMapView)
                    self.nameWmsServer.addItems(self.listeWmsServer)
                
                    test, value = fileRessourceExist(self, "xml/geoloc/osm.xml")

                    if value != "" : value = basename(value)
                    index = self.listeWmsServer.index(value) if value in (self.listeWmsServer) else 0
                    self.nameWmsServer.setCurrentIndex(index)


                    self.nameWmsServer.currentIndexChanged.connect(self.getWmsServer)
                    self.nameWmsServer.setGeometry(15, 35, 300, 25)

                    self.sliderOpacity = mySlider(self.widgetParamMapView)
                    self.sliderOpacity.initSlider(330, 30, 15, 80, "sliderOpacity", "", True, False, Qt.Horizontal, 2, 10, 10)

                    icon = getThemeIcon("reload.png")   
                    toolTip = "%s" % QApplication.translate("QSphere", "Reload page")
                    self.refreshWmsServer = myPushButton(self.widgetParamMapView)
                    self.refreshWmsServer.initPushButton(25, 25, 320, 35, "refreshWmsServer", "", toolTip, True, icon, 25, 25, True)

                    self.movieCtrl = movieCtrl(self, "attente.gif")

                    self.toolPan = QgsMapToolPan(self.mapViewer)
                    self.toolZoomIn = QgsMapToolZoom(self.mapViewer, False) 
                    self.toolZoomOut = QgsMapToolZoom(self.mapViewer, True)

                    self.buttonPan.clicked.connect(self.pan)
                    self.buttonZoomIn.clicked.connect(self.zoomIn)
                    self.buttonZoomOut.clicked.connect(self.zoomOut)
                    self.buttonZoomAll.clicked.connect(self.zoomAll)
                    self.buttonZoomPrevious.clicked.connect(self.zoomPrevious)
                    self.buttonZoomNext.clicked.connect(self.zoomNext)

                    self.mapViewer.extentsChanged.connect(self.updateBoxValues)
                    self.mapViewer.mapCanvasRefreshed.connect(self.movieCtrl.stopMovie)
                    self.sliderOpacity.valueChanged.connect(self.fixeOpacity)
                    self.refreshWmsServer.clicked.connect(self.getWmsServer)

                self.moveDown.clicked.connect(self.moveSelDown)
                self.moveUp.clicked.connect(self.moveSelUp)
                self.refreshExtent.clicked.connect(self.refreshMapWithCurrentExtent)
                self.buttonSendToExtent.clicked.connect(self.catchExtent)
                self.addAfter_6_tableemprises.clicked.connect(self.addExtent)
                
                if self.badNetwork :
                    self.movieCtrl.startMovie()
                    self.getWmsServer()


      def getWmsServer(self):
              from qgis.core import (QgsProject, QgsRasterLayer, QgsRectangle)
              from qsphere.qsphere_tools import (badNetwork, sendMsg)
              from os.path import (basename, dirname, exists, join)
              from osgeo import gdal
 
              self.movieCtrl.startMovie()

              if self.rasterLayer != None : QgsProject.instance().removeMapLayers([self.rasterLayer.id()])
              
              wmsServerFile = join(dirname(__file__),"xml/geoloc/%s" % self.nameWmsServer.currentText())
              
              if exists(wmsServerFile):

                              with open(wmsServerFile, "r") as zFile : xml = zFile.read()
                              vfn = "/vsimem/%s" % self.nameWmsServer.currentText()
                              gdal.FileFromMemBuffer(vfn, xml)
                              self.rasterLayer = QgsRasterLayer(vfn, "osm")

                              if not self.rasterLayer.isValid():
     
                                    arcGisServer = "http://server.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer/WMTS"
                                    self.rasterLayer = QgsRasterLayer(arcGisServer, "World Imagery")
  
                                    if not self.rasterLayer.isValid():
                                            
                                            title = QApplication.translate("QSphere", "Warning" ) 
                                            msg = QApplication.translate("QSphere","Layer failed to load")
                                            sendMsg(self, title, "<font color='#ff0000'><b>%s !</b></font>" % (msg), Qgis.Warning, self.duration_warning)
                                            self.rasterLayer = None

                                            return
                                            
                                    else : startPosition = 15 ; epsg = 3857 ; 

                              else :
                                        startPosition = xml.find("<Projection>")
                                        if startPosition > 0 :
                                                endPosition = xml.find("</Projection>")
                                                epsg = xml[startPosition+12:endPosition].upper().replace("EPSG:", "")
                                        else : epsg = 3857;  
              else : return 

              box2 = float(self.upperLeftX.text()) if self.upperLeftX.text()!= "" else self.emprise[2]
              box1 = float(self.lowerRightY.text()) if self.lowerRightY.text()!= "" else self.emprise[1]
              box3 = float(self.lowerRightX.text()) if self.lowerRightX.text()!= "" else self.emprise[3]
              box0 = float(self.upperLeftY.text()) if self.upperLeftY.text()!= "" else self.emprise[0]

              box = QgsRectangle(box2, box1, box3, box0)
              box = self.getTransform(box, 0)

              self.transformer.setDestinationCrs(QgsCoordinateReferenceSystem(int(epsg)))
              self.mapViewer.setDestinationCrs(self.transformer.destinationCrs())

              QgsProject.instance().addMapLayer(self.rasterLayer, False)
              self.mapViewer.setLayers([self.rasterLayer])
                
              if startPosition > 0 : self.mapViewer.setExtent(box)
              self.mapViewer.refresh()
              self.rasterLayer.dataProvider().reloadData()
              self.rasterLayer.triggerRepaint()


      def fixeOpacity(self):
              opacity = float(self.sliderOpacity.value()/10)
              self.setWindowOpacity(opacity)
              

      def getTransform(self, box, sens):
              from qgis.core import QgsCoordinateTransform
              if self.transformer.sourceCrs() == self.transformer.destinationCrs() : return box 
              if sens == 0 : targetBox = self.transformer.transformBoundingBox(box, QgsCoordinateTransform.ForwardTransform)
              else : targetBox = self.transformer.transformBoundingBox(box, QgsCoordinateTransform.ReverseTransform)
              return targetBox

      def refreshData(self):
          try : self.parent.refreshData()
          except : pass
          if self.sender()!= self.addAfter_6_tableemprises : self.refreshMapWithCurrentExtent() 
      
      def reloadGeolocalisator(self):
          from PyQt5.QtWebKitWidgets import QWebPage
          self.emprise = [self.widget.cellWidget(self.row, i).value() for i in range(0,4)]
          self.mapViewer.pageAction(QWebPage.Reload)
          self.loadGeolocalisator()

      def loadGeolocalisator(self):
          from codecs import open as codecsopen
          from os.path import (exists, join)
          from urllib.parse import unquote
          file = "%sgeolocalisator.html" % (join(dirname(__file__),"ressources/html/"))
          if exists(unquote(file)) :
                html = ""
                if self.emprise!=[]: emprise = "<script>var bounds = new OpenLayers.Bounds(%s, %s, %s, %s);</script>" % (self.emprise[2], self.emprise[1], self.emprise[3], self.emprise[0])
                else : emprise = "<script>var bounds = new OpenLayers.Bounds(-9.6199999999999992, 41.18, 10.300000000000001, 51.539999999999999);</script>"

                f = codecsopen(file) 
                try :
                    for line in f: html+= line
                except : pass
                f.close()
                            
                if html.find("%SCRIPT_EMPRISE%")!= -1: html = html.replace("%SCRIPT_EMPRISE%", emprise)
                      
                baseUrl = QUrl.fromLocalFile(file) 
                self.mapViewer.setHtml(html, baseUrl)
 
      def refreshMapWithCurrentExtent(self):
          from qgis.core import QgsRectangle
          self.emprise = [self.widget.cellWidget(self.row, i).value() for i in range(0,4)]
          if self.badNetwork : 
            #0 : upperLeftY, 1 : lowerRightY, 2 : UpperLeftX, 3 : lowerRightX
            box = QgsRectangle(self.emprise[2], self.emprise[1], self.emprise[3], self.emprise[0])
            box = self.getTransform(box, 0)
            self.mapViewer.setExtent(box)
            self.rasterLayer.dataProvider().reloadData()
            self.rasterLayer.triggerRepaint()
          else : self.reloadGeolocalisator()
       
      def majIconButton(self, cond):
         from qsphere.qsphere_tools import changeButtonIcon
         icon = "voir.png" if not cond else "voiractivate.png"
         button = self.widget.cellWidget(self.row, 4)
         self.setWindowTitle("%s ... %s : %s / %s" % (QApplication.translate("QSphere","Tiles map services"), QApplication.translate("QSphere","Item").lower(), self.row+1, self.widget.rowCount()))
         changeButtonIcon(self, button, icon, 24, 24)     

      def closeEvent(self,event):
         if self.synchronize : self.majIconButton(False)
         from qgis.core import QgsProject
         try : QgsProject.instance().removeMapLayers([self.rasterLayer.id()])
         except : pass
         event.accept()      

      def moveSelUp(self):
        self.majIconButton(False)
        if self.row > 0 : self.row+=  - 1
        else :  self.row = self.widget.rowCount()-1       
        self.widget.selectRow(self.row)
        self.refreshData() if self.badNetwork else self.reloadGeolocalisator()
        if self.formWidgetDialog : self.parent.refreshData()
        self.majIconButton(True)
                  
      def moveSelDown(self):
        self.majIconButton(False)
        if self.row <  (self.widget.rowCount()-1) : self.row+= 1
        else : self.row = 0
        self.widget.selectRow(self.row)
        self.refreshData() if self.badNetwork else self.reloadGeolocalisator()
        if self.formWidgetDialog : self.parent.refreshData()
        self.majIconButton(True)
       
      def zoomIn(self): self.mapViewer.setMapTool(self.toolZoomIn)
      def zoomOut(self):  self.mapViewer.setMapTool(self.toolZoomOut)
      def pan(self): self.mapViewer.setMapTool(self.toolPan)

      def zoomAll(self): self.mapViewer.zoomToFullExtent()
      def zoomPrevious(self):self.mapViewer.zoomToPreviousExtent()
      def zoomNext(self): self.mapViewer.zoomToNextExtent()

      def addExtent(self):
          self.parent.addLine() 
          self.catchExtent()
          self.moveSelDown()

      def catchExtent(self):
          from qsphere.qsphere_tools import fixeValuesForExtent
          
          if self.badNetwork :
                    box = self.mapViewer.extent()
                    box = self.getTransform(box, 1)
                    self.emprise = (box.yMaximum(), box.yMinimum(), box.xMinimum(), box.xMaximum())
          else :
                    document = self.mapViewer.page().mainFrame().documentElement()
                    latnorth = document.evaluateJavaScript('document.getElementById("latnorth").value;')
                    latsouth = document.evaluateJavaScript('document.getElementById("latsouth").value;')
                    longwest = document.evaluateJavaScript('document.getElementById("longwest").value;')
                    longeast = document.evaluateJavaScript('document.getElementById("longeast").value;')
                    self.emprise = [float(latnorth), float(latsouth), float(longwest), float(longeast)]

          fixeValuesForExtent(self, self.widget, self.widget.currentRow(), self.emprise, True, True)
          
          if self.badNetwork : self.refreshData()
          if self.formWidgetDialog : self.parent.refreshData()

      def updateBoxValues(self):
          self.movieCtrl.startMovie()    
          box = self.getTransform(self.mapViewer.extent(),1)
          self.upperLeftX.setText("%s" % round(box.xMaximum(), 8)) 
          self.lowerRightX.setText("%s" % round(box.xMinimum(), 8))
          self.upperLeftY.setText("%s" % round(box.yMaximum(), 8))
          self.lowerRightY.setText("%s" % round(box.yMinimum(),8)) 

      def resizeEvent(self, ev):
            self.barInfo.setGeometry(0, 0, self.width(), 90)  
            self.mapViewer.setGeometry(0, 0, self.width(), self.height())
            if self.badNetwork :
                self.movieCtrl.setGeometry(int(self.width()/2)-64, int(self.height()/2)-64, 64, 64)    
                self.upperLeftX.setGeometry(self.width()-130, int(self.height()/2)-10, 120, 25) 
                self.lowerRightX.setGeometry(5, int(self.height()/2)-10, 120, 25) 
                self.upperLeftY.setGeometry(int(self.width()/2)-60, 5, 120, 25) 
                self.lowerRightY.setGeometry(int(self.width()/2)-60, self.height()-30, 120, 25)
                

class mdiArea(QMdiArea):
        def __init__(self, parent=None):
            super(mdiArea, self).__init__(parent)
            self.setTabsClosable(True)
            self.setTabsMovable(True)            
            self.setAcceptDrops(True)
            self.parent = parent
            self.duration_warning = self.parent.duration_warning

        def dragEnterEvent(self, event): event.accept()
        def dragLeaveEvent(self, event): event.accept()
        def dragMoveEvent(self, event): event.accept()                
        def dropEvent(self, event):
                source = event.mimeData().text().replace("file:///", "")
                source = source.split("\n")
                for elt in source :
                        if elt != "" :
                                sourceFile = QFileInfo(elt)
                                if sourceFile.isFile():
                                        if sourceFile.suffix().lower() in ("qsp", "qmd", "xml") : mdiWindow, child = self.parent.loadMetadataFile(sourceFile.absoluteFilePath(), False, -1)
                                        else : self.badIso(0, sourceFile.fileName())
                                else :  self.badIso(1, sourceFile.fileName())

        def badIso(self, useCase, fileName):
                from qsphere.qsphere_tools import sendMsg
                msg = "%s<br>%s" % (QApplication.translate("QSphere","Error on unsupported content :"), fileName) if useCase == 0 else "%s<br>%s !" % (fileName, QApplication.translate("QSphere","Not valid"))
                sendMsg(self.parent, "information" , msg, Qgis.Warning, self.duration_warning)
                
class roundDockWidget(QDockWidget):
    def __init__(self, parent):
        super(roundDockWidget, self).__init__()
        self.parent = parent
        self.setupUi()

    def setupUi(self):
        from qsphere.qsphere_tools import (changeButtonIcon, getThemeIcon, windowIcon)

        self.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.sizeWidthEllipse, self.sizeHeightEllipse = 800, 400
        self.iconSize = 48
        
        windowIcon(self, "globe_sphery.png")
        self.setWindowTitle("Controler")

        self.tabWidget = QTabWidget(self)
        self.tabWidget.setObjectName("tabWidget")
        self.tabWidget.setTabShape(QTabWidget.Rounded)
        self.tabWidget.setTabPosition(QTabWidget.West)
        self.tabWidget.setDocumentMode(True)
        self.tabWidget.setUsesScrollButtons(True)
        self.tabWidget.setIconSize(QSize(self.iconSize, self.iconSize))

        toolbars = {"tabFile" : ("toolBarFiles", "tabfiles.png", self.parent.listActionsTabFile),
                    "tabEdition" : ("toolBarEdition", "tabedit.png",self.parent.listActionsTabEdition),
                    "tabWebServices" : ("toolBarWebServices", "tabwebservices.png", self.parent.listActionsTabWebServices),
                    "toolBarWindows" : ("toolBarWindows", "tabwindows.png", self.parent.listActionsTabWindows)
                    }

        self.fixedHeightToolBar = 120
        sizeTabWidget = 0
        toolBars = []
        self.coeff = 1.6
        for key in toolbars :
            objTabBar = QWidget(self.tabWidget)
            objTabBar.setObjectName(key)
            setattr(self, key, objTabBar)

            objToolBar = self.parent.makeMenuCentralWidget(objTabBar, toolbars[key][1], "",  toolbars[key][2], False)
            setattr(self, toolbars[key][0], objToolBar)
            objToolBar.setObjectName(toolbars[key][0])
            objToolBar.setStyleSheet("QToolBar { font-size: 12px;  text-align: center; color : white; background-color: rgba(160, 160, 160, 125); border: 2px solid black;  border-radius: 4px;  padding: 2px;}")

            toolBars.append(objToolBar)

            self.tabWidget.addTab(objTabBar, QIcon(getThemeIcon(toolbars[key][1])), "")
            sizeTabWidget = max(objToolBar.width(), sizeTabWidget)

            objToolBar.setGeometry(10, 25, sizeTabWidget*self.coeff+40, self.fixedHeightToolBar)

       
        self.tabWidget.sizeTabWidget = sizeTabWidget
        self.sizeWidthEllipse = sizeTabWidget*self.coeff
        self.tabWidget.setGeometry(10, 50, sizeTabWidget*self.coeff, self.fixedHeightToolBar)

        for toolBar in toolBars :
            toolBar.setMinimumSize(sizeTabWidget*1.4, 50)
            toolBar.setMaximumSize(sizeTabWidget*1.4, 50)

        self.labelImage = QLabel(self)
        icon = QImage(getThemeIcon("globe_sphery.png"))
        self.labelImage.setPixmap(QPixmap.fromImage(icon))
        self.labelImage.setGeometry(5, 0, 70, 76)

        self.reduceIcon = QIcon(getThemeIcon("hreduce.png"))
        self.expandIcon = QIcon(getThemeIcon("hexpand.png"))
        self.stateExpand = True
        self.expandReduceDock = myPushButton(self)
        self.expandReduceDock.initPushButton(24, 24, 45, 50, "expandReduceDock", "", QApplication.translate("QSphere", "Expand"), True, self.reduceIcon, 24, 24, True)

        self.expandReduceDock.clicked.connect(self.fixeSize)

        self.setMinimumSize(50, self.height())
        self.isClosable = False

        self.tabWidget.setCurrentIndex(0)        
        self.updateTableWidgetConnector()


    def updateTableWidgetConnector(self):
        up = True        
        for children in self.tabWidget.children():
              for child in children.children():
                      if type(child) == QToolButton :
                         child.clicked.connect(self.getTab)
                         child.setAccessibleDescription("Up") if up else child.setAccessibleDescription("Down")
                         if up : up = False

    def getTab(self):
            index = self.tabWidget.currentIndex()
            index+= 1 if self.sender().accessibleDescription() ==  "Down" else -1
            if index > self.tabWidget.count()-1 : index = self.tabWidget.count()-1
            if index < 0 : index = 0
            try : self.tabWidget.setCurrentIndex(index)
            except : pass
    
            
    def closeEvent(self,event):
        event.accept() if self.isClosable else event.ignore() 

    def restoreFloating(self):
        if not self.isFloating() : self.setFloating(True)

    def fixeSize(self):
        self.stateExpand = not (self.stateExpand)
        if not self.stateExpand :
            self.expandReduceDock.setIcon(self.expandIcon)
            self.setGeometry(self.x(), self.y(), self.minimumSize().width()+10, self.height())
        else :
            self.expandReduceDock.setIcon(self.reduceIcon)
            self.setGeometry(self.x(), self.y(), self.sizeWidthEllipse, self.height())
       
    def sizeHint(self): return QSize(self.sizeWidthEllipse, self.sizeHeightEllipse)

    def paintEvent(self, event):
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing)
        qp.setPen(Qt.NoPen)
        qp.setBrush(QColor(25, 110, 200, 125)) 
        pct2 = 0
        for pct1 in range(360): 
              qp.setPen(QColor(pct2, pct2, pct2, 127)) 
              qp.setBrush(QColor(25, 110, 200, 127)) 
              qp.drawPie(20, 10, 40, 40,(16*pct1),16 )
              pct2+= 1 if (pct1 < 150) else -1
        qp.drawEllipse(20, 10, 40, 40) 
        qp.end()

    def mousePressEvent(self, event): 
        try :
            if event.button() == Qt.LeftButton: self.dragPosition = event.globalPos() - self.frameGeometry().topLeft() 
            QApplication.postEvent(self, QEvent(174)) 
            event.accept()
        except: pass
        
    def mouseMoveEvent(self, event):
        try :
             if event.buttons() == Qt.LeftButton: self.move(event.globalPos() - self.dragPosition) 
             event.accept()
        except : pass


class dlgNode(QMessageBox):
    def __init__(self, parent, title, msg, icon, name, qry, enabled, *args):
            super(dlgNode, self).__init__()
            from qsphere.qsphere_tools import (getThemeIcon, windowIcon)

            self.parent = parent
            
            self.setWindowTitle(title)
            self.setWindowFlags(Qt.WindowCloseButtonHint)
            self.setIcon(QMessageBox.NoIcon)
            windowIcon(self, "layers-database.png")
            
            groupBoxNodeInfos = QGroupBox(self)
            groupBoxNodeInfos.setFixedSize(620, 510)
            groupBoxNodeInfos.setObjectName("groupBoxNodeInfos")

            self.name = sqlTitle(groupBoxNodeInfos)
            self.name.setGeometry(55, 5, 550, 25)
            self.name.setText(name)
            self.name.textChanged.connect(self.name.checkRegExp)
            
            labelImage = QLabel(groupBoxNodeInfos)
            icon = getThemeIcon(icon)
            carIcon = QImage(icon)
            labelImage.setPixmap(QPixmap.fromImage(carIcon).scaled(32, 32))
            labelImage.setFixedSize(32, 32)
            labelImage.setGeometry(5, 5, 32, 32)
            
            self.editorSql = QgsCodeEditorSQL(groupBoxNodeInfos)
            self.editorSql.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
            self.editorSql.setMarginVisible(True)
            self.editorSql.setWrapMode(QsciScintilla.WrapWord)
            self.editorSql.setGeometry(5, 40, 600, 400)
            self.editorSql.setText(qry)
            self.editorSql.lexer().setAPIs(self.parent.editorSql.lexer().apis())

           
            self.enabledCheckBox = QCheckBox(groupBoxNodeInfos)
            self.enabledCheckBox.setText(QApplication.translate("QSphere", "Enabled"))
            self.enabledCheckBox.setGeometry(55, 450, 180, 20)
            self.enabledCheckBox.setChecked(self.str2bool("%s" % enabled))

            self.findprevious = myPushButtonFunny(groupBoxNodeInfos)
            self.findprevious.initPushButton(48, 48, 230, 450, "findprevious", "", QApplication.translate("QSphere", "Find previous"), True, getThemeIcon("leftarrow.png"), 48, 48, True)
            self.findprevious.setShortcut(QKeySequence("F2"))

            icon = getThemeIcon("sgbdr/sqlplay.png")
            toolTip = QApplication.translate("QSphere","Execute SQL")
            self.testSql = myPushButtonFunny(groupBoxNodeInfos) 
            self.testSql.initPushButton(48, 48, 280, 450, "testSql", "", toolTip, True, icon, 48, 48, True)
            self.testSql.setShortcut(QKeySequence("F5"))

            self.findnext = myPushButtonFunny(groupBoxNodeInfos)
            self.findnext.initPushButton(48, 48, 330, 450, "findnext", "", QApplication.translate("QSphere", "Find next"), True, getThemeIcon("rightarrow.png"), 48, 48, True)
            self.findnext.setShortcut(QKeySequence("F3"))

            self.changeMode = QButtonGroup(groupBoxNodeInfos)
            self.changeMode.setExclusive(True)

            Obj = myPushButtonFunny(groupBoxNodeInfos) 
            Obj.initPushButton(48, 48, 450, 450, "baseView", "", QApplication.translate("QSphere","Databases"), True, self.parent.pucebase, 48, 48, False)
            Obj.setCheckable(True)
            self.changeMode.addButton(Obj, 0)


            self.expandAllBaseNodes = myPushButton(groupBoxNodeInfos)
            self.expandAllBaseNodes.initPushButton(24, 24, 500, 450, "expandAllBaseNodes", "", QApplication.translate("QSphere", "Expand all"), True, getThemeIcon("sgbdr/expandall.png"), 24, 24, True)

            self.closeAllBaseNodes = myPushButton(groupBoxNodeInfos)
            self.closeAllBaseNodes.initPushButton(24, 24, 500, 475, "closeAllBaseNodes", "", QApplication.translate("QSphere", "Reduce all"), True, getThemeIcon("sgbdr/reduceall.png"), 24, 24, True)            
            
            
            Obj = myPushButtonFunny(groupBoxNodeInfos) 
            Obj.initPushButton(48, 48, 530, 450, "sqlView", "", QApplication.translate("QSphere","SQL"), True, getThemeIcon("sgbdr/sql.png"), 48, 48, False)
            Obj.setCheckable(True)
            self.changeMode.addButton(Obj, 1)
            Obj.setChecked(True)


            self.expandAllSqlNodes = myPushButton(groupBoxNodeInfos)
            self.expandAllSqlNodes.initPushButton(24, 24, 580, 450, "expandAllSqlNodes", "", QApplication.translate("QSphere", "Expand all"), True, getThemeIcon("sgbdr/expandall.png"), 24, 24, True)

            self.closeAllSqlNodes = myPushButton(groupBoxNodeInfos)
            self.closeAllSqlNodes.initPushButton(24, 24, 580, 475, "closeAllSqlNodes", "", QApplication.translate("QSphere", "Reduce all"), True, getThemeIcon("sgbdr/reduceall.png"), 24, 24, True)      
            

            acceptButton = QPushButton(self)
            acceptButton.setObjectName("SaveButton")
            acceptButton.setText(QApplication.translate("QSphere", "Apply"))
            acceptButton.setFixedSize(100, 25)

            rejectButton = QPushButton(self)
            rejectButton.setObjectName("closeButton")
            rejectButton.setText(QApplication.translate("QSphere", "Cancel"))
            rejectButton.setFixedSize(100, 25)

            self.changeMode.buttonClicked[int].connect(self.changeViewMode)
            self.expandAllBaseNodes.clicked.connect(self.expandCloseTreeView)
            self.closeAllBaseNodes.clicked.connect(self.expandCloseTreeView)
            self.expandAllSqlNodes.clicked.connect(self.expandCloseTreeView)
            self.closeAllSqlNodes.clicked.connect(self.expandCloseTreeView)
            self.findprevious.clicked.connect(self.goPrevious)
            self.findnext.clicked.connect(self.goNext)
            self.testSql.clicked.connect(self.executeSql)
            acceptButton.clicked.connect(self.applychanges)
            rejectButton.clicked.connect(self.reject)

            self.layout().addWidget(groupBoxNodeInfos,0,1,1,3)
            self.layout().addWidget(acceptButton,4,2,1,1)
            self.layout().addWidget(rejectButton,4,3,1,1)

            self.setStandardButtons(self.NoButton)            

    def applychanges(self):
        self.parent.integrateChanges(self.infos())
        item = self.parent.objSqlViewer.currentItem()
        self.updateCtrl(item.text(0))        

    def executeSql(self): self.parent.executeSql()

    def goPrevious(self):
        if self.parent.objSqlViewer.currentIndex().row()> 0 :
            item = self.parent.objSqlViewer.itemAbove(self.parent.objSqlViewer.currentItem())
            if not(item) : return
            if self.parent.objSqlViewer.indexOfTopLevelItem(item) == -1 :  item = item.parent()
            self.parent.objSqlViewer.setCurrentItem(item)
            self.updateCtrl(item.text(0))
        
    def goNext(self):
        if self.parent.objSqlViewer.currentIndex().row()<self.parent.objSqlViewer.topLevelItemCount()-1 :
            item = self.parent.objSqlViewer.itemBelow(self.parent.objSqlViewer.currentItem())
            if not(item) : return
            while self.parent.objSqlViewer.indexOfTopLevelItem(item) == -1 :
                item = self.parent.objSqlViewer.itemBelow(self.parent.objSqlViewer.currentItem())
                self.parent.objSqlViewer.setCurrentItem(item)
            self.parent.objSqlViewer.setCurrentItem(item)
            self.updateCtrl(item.text(0))

    def expandCloseTreeView(self) :
        treewidget = self.parent.objSqlViewer if self.sender() in (self.expandAllSqlNodes, self.closeAllSqlNodes) else self.parent.objDataBaseViewer
        cond = True if self.sender() in (self.expandAllBaseNodes, self.expandAllSqlNodes) else False
        treewidget.expandAll() if cond else treewidget.collapseAll()

    def updateCtrl(self, title):
        self.parent.oldtitle = title
        infosNode = self.parent.dicoSql[title]
        self.name.setText(infosNode['title'])
        self.editorSql.setText(infosNode['source'])
        self.enabledCheckBox.setChecked(self.str2bool(infosNode['enabled']))
        
    def str2bool(self, v): return v.lower() == "true"
    
    def changeViewMode(self, index): self.parent.tabs.setCurrentIndex(index)
        
    def infos(self): return {'title': self.name.text(), 'source': self.editorSql.text(), 'enabled': '%s' % self.enabledCheckBox.isChecked()}


class sqlTitle(QLineEdit):
       def __init__(self, parent):
           super(sqlTitle, self).__init__(parent)
           self.setObjectName(parent.objectName())

       def checkRegExp(self): self.setText(self.text().upper().replace("<", "").replace(">", "").replace("%", ""))          

class baseTableModel(QAbstractTableModel):
    def __init__(self, header=None, data=None, parent=None):
        super(baseTableModel, self).__init__(parent)
        self.header = header if header else []
        self.resdata = data if data else []

    def headerToString(self, sep=u"\t"):
        header = self.header
        return sep.join(header)

    def rowToString(self, row, sep=u"\t"):
        text = u""
        for col in range(self.columnCount()):
            text += u"%s" % self.getData(row, col) + sep
        return text[:-1]

    def getData(self, row, col):
        try : return self.resdata[row][col]
        except : pass

    def columnNames(self): return list(self.header)
    def rowCount(self, parent=None): return len(self.resdata)
    def columnCount(self, parent=None): return len(self.header)
    def data(self, index, role):
        if role != Qt.DisplayRole and role != Qt.FontRole: return None

        val = self.getData(index.row(), index.column())

        if role == Qt.FontRole:  
            if val is not None: return None
            f = QFont()
            f.setItalic(True)
            return f

        if val is None: return "NULL"
        elif isinstance(val, memoryview): return None
        elif isinstance(val, (str, unicode)) and len(val) > 300: return u"%s..." % val[:300]
        return unicode(val)  

    def headerData(self, section, orientation, role):
        if role != Qt.DisplayRole: return None
        if orientation == Qt.Vertical: return section + 1
        else : return self.header[section]
        

class myMsgDlgBox(QMessageBox):
        
        def __init__(self, parent, icon, title, msg, qsphereIcon, ctrl, *args):
            super(myMsgDlgBox, self).__init__()
            from qsphere.qsphere_tools import (getThemeIcon, sortDic, windowIcon)
            self.setWindowTitle(title)
            self.setIcon(icon)
            self.parent = parent
            self.listOfRules = [self.parent.dictListOfRules[key] for key in self.parent.dictListOfRules]
            self.typeData = self.parent.typeData
            self.langs = sortDic(self.parent.languesDico)
            self.listCodecs = self.parent.listCodecs
            
            windowIcon(self, "qsphere.png")

            groupBoxNodeInfos = QGroupBox(self)
            groupBoxNodeInfos.setFixedSize(420, 180)
            groupBoxNodeInfos.setObjectName("groupBoxNodeInfos")

            lblText = QLabel(groupBoxNodeInfos)
            lblText.setText(msg)
            lblText.setAlignment(Qt.AlignCenter)
            lblText.setFixedSize(420, 65)
            lblText.setGeometry(5, 110, 420, 65)

            labelImage = QLabel(groupBoxNodeInfos)
            icon = getThemeIcon(qsphereIcon)
            carIcon = QImage(icon)
            labelImage.setPixmap(QPixmap.fromImage(carIcon).scaled(128, 108))
            labelImage.setFixedSize(128, 108)
            labelImage.setGeometry(145, 5, 128, 108)

            if ctrl :
                    ctrl.setFixedSize(420, 240)    
                    self.layout().addWidget(ctrl,2,1,1,2)
                    self.ctrl = ctrl
                    self.ctrl.itemDoubleClicked.connect(self.getInfos)

            acceptButton = QPushButton(self)
            acceptButton.setObjectName("SaveButton")
            acceptButton.setText(QApplication.translate("QSphere", "Apply"))
            acceptButton.setFixedSize(100, 25)

            rejectButton = QPushButton(self)
            rejectButton.setObjectName("closeButton")
            rejectButton.setText(QApplication.translate("QSphere", "Cancel"))
            rejectButton.setFixedSize(100, 25)

            acceptButton.clicked.connect(self.accept)
            rejectButton.clicked.connect(self.reject)

            self.layout().addWidget(groupBoxNodeInfos,0,1,1,2)
            self.layout().addWidget(acceptButton,4,1,1,1)
            self.layout().addWidget(rejectButton,4,2,1,1)

            self.setStandardButtons(self.NoButton)


        def setInfos(self, event):
            from qsphere.qsphere_tools import dateToString
            currentItem = self.ctrl.currentItem()
            widget = self.ctrl.itemWidget(currentItem, 0)
            oldText = currentItem.text(0)
            text = None
            try : text = widget.text()
            except :
                    try : text = widget.currentText()
                    except :
                            try : text = widget.value()
                            except :
                                    try : text = dateToString(self, widget)
                                    except : pass

            if text :    
                currentItem.setText(0, "%s" % text)    

            itemParent = currentItem.parent()                       
            if itemParent : itemParent.setToolTip(0, itemParent.toolTip(0).replace('"%s"' % oldText, '"%s"' % text))

                
        def getInfos(self, item):

             if item.whatsThis(0) == "property" : 

                     if item.parent().text(0) == "gco:CharacterString" :
                             if item.text(0).find("@")!=-1 : wizard = MyWidgetLineEdit(self); wizard.initType(0); wizard.textChanged.connect(wizard.checkRegExp) ;
                             else :  wizard = QLineEdit(self)
                             wizard.setText(item.text(0))
                     elif item.parent().text(0) == "gco:Date" :
                             wizard = QgsDateTimeEdit(self)
                             eltsDate = item.text(0).split("-")
                             wizard.setStyleSheet("QgsDateTimeEdit {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}"
                                                  "QMenu { font-size:12px; width: 100px; left: 16px; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}"
                                                  "QToolButton {icon-size: 20px, 20px;background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D); height: 20px; width: 140px;}"
                                                  "QAbstractItemView {selection-background-color: rgb(255, 174, 0);}"
                                                  "QToolButton::menu-arrow {}"
                                                  "QToolButton::menu-button {}"
                                                  "QToolButton::menu-indicator{width: 5px;}"
                                                  "QToolButton::menu-indicator:pressed,"
                                                  "QToolButton::menu-indicator:open{top:10px; left: 20px;}"
                                                  "QListView {background-color:white;}"
                                                  "subcontrol-position: top right; width:50px; border-image: url(icons:arrow_up_n.png);}"
                                                  "border-width: 1px; width:25px;}")
                             wizard.setDisplayFormat("yyyy-MM-dd")
                             wizard.setCalendarPopup(True)
                             wizard.setAllowNull(False)
                             wizard.setDate(QDate(int(eltsDate[0]), int(eltsDate[1]), int(eltsDate[2])))

                     elif item.parent().text(0) == "gco:Decimal" :
                             from qsphere.qsphere_tools import propertiesDoubleSpinBox
                             wizard = mySpinBox(self)
                             wizard.setDecimals(10)
                             propertiesDoubleSpinBox(self, wizard, 8, -180, 180, 1) if item.parent().parent().text(0).find("BoundLongitude")!= -1 else propertiesDoubleSpinBox(self, wizard, 8, -90, 90, 1)
                             wizard.setValue(Decimal(item.text(0)))

                     elif item.parent().text(0) == "gco:Boolean" :   
                             wizard = QComboBox(self)
                             wizard.addItems(['false', 'true'])
                             wizard.setCurrentText(item.text(0))

                     elif item.parent().text(0) == "gmd:LanguageCode" :
                             wizard = QComboBox(self)
                             wizard.addItems(self.langs)
                             if not item.text(0) in self.langs : wizard.addItem(item.text(0))
                             wizard.setEditable(True)
                             wizard.setCurrentText(item.text(0))

                     elif item.parent().text(0).find("MD_ScopeCode") != -1 :
                             wizard = QComboBox(self)
                             wizard.addItems(self.typeData)
                             wizard.setCurrentText(item.text(0))

                     elif item.parent().text(0).find("MD_CharacterSetCode") != -1 :
                             wizard = QComboBox(self)
                             wizard.addItems(self.listCodecs) 
                             wizard.setCurrentText(item.text(0))                             
                          
                     elif item.parent().text(0).find("DateTypeCode")!= -1 :
                             wizard = QComboBox(self)
                             wizard.addItems(['creation', 'publication', 'revision'])
                             wizard.setCurrentText(item.text(0))

                     elif item.parent().text(0).find("URL")!=-1 :
                             wizard = MyWidgetLineEdit(self)
                             wizard.initType(4); wizard.textChanged.connect(wizard.checkRegExp)
                             wizard.setText(item.text(0))

                     elif item.parent().text(0).find("RoleCode")!=-1 :
                             wizard = QComboBox(self)
                             wizard.addItems(self.listOfRules)
                             wizard.setCurrentText(item.text(0))               
                             
                     else :
                             wizard = QLineEdit(self)
                             wizard.setText(item.text(0))

                     wizard.focusOutEvent = self.setInfos
                     self.ctrl.setItemWidget(item, 0, wizard)
             
             
          
class mySubWindow(QMdiSubWindow):
     def __init__(self, parent, dialog, icon, *args):
         super(mySubWindow, self).__init__()
         self.setWidget(dialog)
         self.widget = self.children()[3]
         self.parent = parent
         self.setWindowIcon(icon)
         self.setMinimumSize(0, 0) 
         self.resize(800, 800)

     def closeEvent(self, event):
             self.parent.closeCurrentWindow()
             if type(self.widget) == filesNavigatorWidget : self.parent.filesNavigatorWidget = None
     

class myScintilla(QsciScintilla):
     def __init__(self, parent, nameObj, readOnly, boundsGeom):
        super(myScintilla, self).__init__(parent)
        self.parent = parent
        self.setObjectName(nameObj)
        self.setReadOnly(readOnly)
        self.duration_info = self.parent.duration_info
        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed )
        self.barInfo.setGeometry(0, 0, boundsGeom[2]-10, 90) 
        self.setGeometry(boundsGeom[0], boundsGeom[1], boundsGeom[2], boundsGeom[3])
 
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.arrowMarkerNum = 8

        self.findDialog = findDialog(self)
        font = QFont()
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(12)
        self.setFont(font)
        self.setMarginsFont(font)

        fontmetrics = QFontMetrics(font)
        self.setMarginsFont(font)
        self.setMarginWidth(0, fontmetrics.width("00000") + 6)
        self.setMarginLineNumbers(1, True)
        self.setMarginsBackgroundColor(QColor("#cccccc"))

        self.setMarginSensitivity(1, True)
        self.markerDefine(QsciScintilla.RightArrow, self.arrowMarkerNum)
        self.setMarkerBackgroundColor(QColor("#ee1111"), self.arrowMarkerNum)
        self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
        self.setCaretLineVisible(True)
        self.setCaretLineBackgroundColor(QColor("#ffffcc"))
        self.setUtf8(True)

        lexer = QsciLexerXML(self) 
        api = QsciAPIs(lexer)
        api.prepare()
        self.setAutoCompletionSource(QsciScintilla.AcsDocument)
        self.setAutoCompletionFillupsEnabled(True)
        self.setAutoCompletionThreshold(2)
        self.setLexer(lexer)
        
        font = QFont() 
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(12)
        self.setFont(font)
        
        font.setPointSize(8)
        self.setMarginsFont(font)
        large = '9999'
        self.setMarginWidth(0, large) 
        self.setMarginLineNumbers(0, True)
        self.setMarginWidth(1,1) 
        self.setMarginsBackgroundColor(QColor("#00b4FF"))
        self.setMarginsForegroundColor(QColor("#FFFFFF"))        
        
        self.setTabWidth(5) 
        self.setIndentationsUseTabs(True) 
        self.setIndentationWidth(5) 
        self.setTabIndents(True) 
        self.setBackspaceUnindents(True) 
        self.setAutoIndent( True )
        self.setIndentationGuides( True )
        self.setFolding(QsciScintilla.CircledTreeFoldStyle) 
        self.setFoldMarginColors(QColor("#99CC66"), QColor("#AAAAAA"))

        
        self.setWrapMode( QsciScintilla.WrapWord )
        self.setWrapIndentMode(QsciScintilla.WrapIndentSame )
        self.setAnnotationDisplay(QsciScintilla.AnnotationStandard) 


        self.find_text = ""
        self.point = None
        self.blockText = ""
        self.endBalise = ""
        self.nbTab = 0
        self.lineStart = 0
        self.lineEnd = 0
        self.msgDlgBox = None
        self.annotationStyle = QsciStyle()
        self.annotationStyle.setColor(QColor(255,255,255, 255))
        self.annotationStyle.setPaper(QColor(0,0,0, 125))
        font.setPointSize(18)
        self.annotationStyle.setFont(font)
        self.setWrapMode(QsciScintilla.WrapWord)


        self.customContextMenuRequested.connect(self.contextMenuRequested)


     def wheelEvent(self, ev):
           if Qt.ControlModifier & ev.modifiers():
               try :
                   if ev.angleDelta().y() > 0: self.zoomIn()
                   else: self.zoomOut()
               except : pass    
           else:
               try : return super(self, self).wheelEvent(ev)
               except : pass

     def mouseDoubleClickEvent(self, event):
         self.point = QPoint(event.pos().x(), event.pos().y())
         lineFrom = self.lineAt(self.point)
         
         if self.text() != "" :
              lineFrom = self.lineAt(self.point)
              if lineFrom > 0 :
                 try : self.extractLine()
                 except : pass
 
     def getToCopy(self): QApplication.clipboard().setText(self.ToCopy)
     def getToCopyXmlBlock(self): QApplication.clipboard().setText(self.blockText)
     def selectAll(self): self.selectAll(True)
     def expandAll(self): self.clearFolds()
          
     def closeAll(self):
         self.foldAll(True)
         self.foldLine(0)
         
     def clearMyAnnotation(self):
          if self.point != None :
             lineFrom = self.lineAt(self.point)
             self.clearAnnotations(lineFrom)
             
     def clearMyAnnotations(self): self.clearAnnotations(-1)
     

     def validateRecords(self):
             from qsphere.qsphere_tools import (cleanHtml, iterateTreeWidget)
             nodeRoot = cleanHtml(self.nodeXmlText)
             html = ""
             root = self.treeWidget.invisibleRootItem()
             for i in range(root.childCount()):
                 html+= iterateTreeWidget(html, root.child(i), root.child(i), self.nbTab)
                 html+= "</%s>\n" % root.child(i).text(0)
             html+= nodeRoot.replace("<", "</")
             html = "%s%s" % (nodeRoot, html)

             self.replaceRecords(html)

   
     def cloneRecords(self):
          if self.blockText != "" and self.lineEnd > 0 :
                  insertText = "%s\n%s\n" % (self.blockText , self.endBalise)
                  self.insertAt(insertText, self.lineEnd, 0)
   
     def replaceRecords(self, text):
         QApplication.clipboard().setText(text)
         self.setCursorPosition(self.lineEnd, 0)
         self.paste()
         self.removeRecords()
         
                  
     def removeRecords(self):
          if self.blockText != "" and self.lineEnd > 0 :
                  self.setSelection(self.lineStart, 0, self.lineEnd-1, self.lineLength(self.lineStart)-1)
                  self.removeSelectedText()
                  self.lineEnd = -1
                  if self.msgDlgBox : self.msgDlgBox.reject()

     def expandTreeWidget(self):
          if self.sender() == self.treeWidgetExpand : self.treeWidget.expandAll()
          else : self.treeWidget.collapseAll()
             
     def extractLine(self):
         if self.point != None :
            lineFrom = self.lineAt(self.point)
            self.getExtractLines(lineFrom)

     def getExtractLines(self, lineFrom):        
            lineStart = lineFrom+1
            self.lineStart = lineFrom
             
            selection = self.setSelection(lineFrom, 0, lineFrom, self.lineLength(lineFrom)-1)
            text = self.selectedText()
            listInfos = []

            if text.find("</")!=-1: return

            from qsphere.qsphere_tools import (getDicoXml, netToHtml, windowIcon)
            
            baliseD = text
            self.nodeXmlText = netToHtml(baliseD)

            from qsphere.qsphere_tools import countCaractere
            self.nbTab = countCaractere(baliseD, "\t", False, False)
            
            posD = text.find("<")
            if posD != -1 :
               posF = text.find(">", posD)
               if posF != -1 and posF > posD :
                  baliseD = "%s" % (text[posD:posF+1])
                  
                  if baliseD.find(" ")!=-1 :
                     tempo = baliseD.split(" ")
                     baliseD = "%s>" % tempo[0]
                  baliseF = baliseD.replace("<", "</")
                  lineFrom+=1

                  while self.text(lineFrom).find(baliseF) == -1 :
                        extractTxt = self.text(lineFrom).strip()
                        text = "%s\n%s" % (text.strip(), extractTxt)
                        if not extractTxt in listInfos : listInfos.append(extractTxt)
                        lineFrom+= 1

            self.blockText = "%s\n%s" % (text, baliseF) 
            self.lineEnd = lineFrom+1
            self.endBalise = baliseF
          
            self.annotate(lineStart-1, fill(self.blockText,50), self.annotationStyle)
            self.foldLine(lineStart-1)
            self.treeWidget = getDicoXml(self.blockText)

            toolBar = QToolBar(self.treeWidget) 
            toolBar.setObjectName("toolBar")
            toolBar.setGeometry(240, 5, 150, 30)
            toolBar.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
            self.actionsForToolBar(toolBar)

            toolBar.setStyleSheet("QToolBar {icon-size: 24px, 24px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E1E1E1, stop: 0.4 #9B9B9B, stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);  border-width: 2px; border-radius: 10px; border-color: beige;}")
            
            self.msgDlgBox = myMsgDlgBox(self, QMessageBox.NoIcon, QApplication.translate("QSphere","Parser and Editor"),
                                                "<u>%s</u> : <b>%s</b><hr>" % (QApplication.translate("QSphere","Root"), self.nodeXmlText), "qsphere_editxml.png", self.treeWidget)

            windowIcon(self.msgDlgBox , "editxml.png")
            ret = self.msgDlgBox.exec_()
            if ret == 1 : self.validateRecords()
            self.clearAnnotations(-1)
            self.blockText = ""
            self.endBalise = ""
            self.nodeXmlText = ""
            self.lineEnd = 0

     def actionsForToolBar(self, toolbar):
          from qsphere.qsphere_tools import getThemeIcon

          menuIcon = getThemeIcon("nodes_expand.png")
          self.treeWidgetExpand = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Expand all nodes"), self)
          self.treeWidgetExpand.setObjectName("treeWidgetExpand")
          self.treeWidgetExpand.setShortcut(Qt.Key_Plus) 
          toolbar.addAction(self.treeWidgetExpand)
          self.treeWidgetExpand.triggered.connect(self.expandTreeWidget)

          menuIcon = getThemeIcon("nodes_reduce.png")
          self.treeWidgetReduce = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Reduce all nodes"), self)
          self.treeWidgetReduce.setObjectName("treeWidgetReduce")
          self.treeWidgetReduce.setShortcut(Qt.Key_Minus)
          toolbar.addAction(self.treeWidgetReduce)
          self.treeWidgetReduce.triggered.connect(self.expandTreeWidget)

          toolbar.addSeparator()

          menuIcon = getThemeIcon("copier.png")
          self.treeWidgetClone = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Clone"), self)
          self.treeWidgetClone.setObjectName("treeWidgetClone")
          toolbar.addAction(self.treeWidgetClone)
          self.treeWidgetClone.triggered.connect(self.cloneRecords)

          menuIcon = getThemeIcon("deleteall.png")
          self.treeWidgetDelete = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Delete"), self)
          self.treeWidgetDelete.setObjectName("treeWidgetDelete")
          toolbar.addAction(self.treeWidgetDelete)
          self.treeWidgetDelete.triggered.connect(self.removeRecords)

     def contextMenuRequested(self, point):
            if self.sender().objectName()== "mActionXMLEditor" : return
            lineFrom = self.lineAt(point)
            if lineFrom <0 : return
            
            from qsphere.qsphere_tools import getThemeIcon

            self.point = point
            self.context_menu = QMenu()
            
            menuIcon = getThemeIcon("undo.png")
            text = (QApplication.translate("QSphere", "Undo"))
            self.undoAction = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.undoAction)
            self.undoAction.triggered.connect(self.undo)
            self.undoAction.setShortcut(QKeySequence("Ctrl+Z"))

            menuIcon = getThemeIcon("redo.png")
            text = (QApplication.translate("QSphere", "Redo"))
            self.redoAction = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.redoAction)
            self.redoAction.triggered.connect(self.redo)
            self.redoAction.setShortcut(QKeySequence("Ctrl+R"))

            self.context_menu.addSeparator()
               
            menuIcon = getThemeIcon("copier.png")
            text = (QApplication.translate("QSphere", "Copy"))
            self.copy = self.context_menu.addAction(QIcon(menuIcon), text)
            self.ToCopy = unicode(self.selectedText())
            self.copy.triggered.connect(self.getToCopy)
            self.copy.setShortcut(QKeySequence("Ctrl+C"))

            self.copy.setEnabled(True) if unicode(self.selectedText()) else self.copy.setEnabled(False) 

            menuIcon = getThemeIcon("copier.png")
            text = (QApplication.translate("QSphere", "Copy the xml block"))
            self.copyXmlBlock = self.context_menu.addAction(QIcon(menuIcon), text)
            self.copyXmlBlock.triggered.connect(self.getToCopyXmlBlock)

            self.copyXmlBlock.setEnabled(True) if self.blockText!= "" else self.copyXmlBlock.setEnabled(False) 

            menuIcon = getThemeIcon("coller.png")
            text = (QApplication.translate("QSphere", "Paste"))
            self.pasteAction = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.pasteAction)
            self.pasteAction.triggered.connect(self.paste)
            self.pasteAction.setShortcut(QKeySequence("Ctrl+V"))

            self.pasteAction.setEnabled(True) if unicode(QApplication.clipboard().text()) else self.pasteAction.setEnabled(False)


            menuIcon = getThemeIcon("cut.png")
            text = (QApplication.translate("QSphere", "Cut"))
            self.cutAction = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.cutAction)
            self.cutAction.triggered.connect(self.cut)
            self.cutAction.setShortcut(QKeySequence("Ctrl+X"))

            self.cutAction.setEnabled(True) if unicode(self.selectedText()) else self.cutAction.setEnabled(False)

            self.context_menu.addSeparator()            
           
            menuIcon = getThemeIcon("selectall.png")
            text = (QApplication.translate("QSphere", "Select all"))
            self.selectAllAction = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.selectAllAction)
            self.selectAllAction.triggered.connect(self.selectAll)
            self.selectAllAction.setShortcut(QKeySequence("Ctrl+A"))

            
            self.context_menu.addSeparator()

            menuIcon = getThemeIcon("nodes_expand.png")
            text = (QApplication.translate("QSphere", "Expand all nodes"))
            self.expandAllNodes = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.expandAllNodes)
            self.expandAllNodes.triggered.connect(self.expandAll)

            menuIcon = getThemeIcon("nodes_reduce.png")
            text = (QApplication.translate("QSphere", "Close all nodes"))
            self.closeAllNodes = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.closeAllNodes)
            self.closeAllNodes.triggered.connect(self.closeAll)

            self.context_menu.addSeparator()


            menuIcon = getThemeIcon("notes.png")
            text = (QApplication.translate("QSphere", "Get annotation"))
            self.getline = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.getline)
            self.getline.triggered.connect(self.extractLine)

            self.context_menu.addSeparator()

            menuIcon = getThemeIcon("delonenote.png")
            text = (QApplication.translate("QSphere", "Delete this annotation"))
            self.delOneNota = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.delOneNota)
            self.delOneNota.triggered.connect(self.clearMyAnnotation)

            
            menuIcon = getThemeIcon("delallnotes.png")
            text = (QApplication.translate("QSphere", "Delete all annotations"))
            self.delAllNota = QAction(QIcon(menuIcon), text, self)
            self.context_menu.addAction(self.delAllNota)
            self.delAllNota.triggered.connect(self.clearMyAnnotations)


            self.getline.setEnabled(True) if unicode(self.selectedText()) else self.getline.setEnabled(False)

            self.context_menu.exec_(self.mapToGlobal(point))
            

class findDialogToolSearchFiles(QDialog):
    findNext = pyqtSignal(str, int)
    findPrevious = pyqtSignal(str, int)
  
    def __init__(self, parent, dataViewer):
        super(findDialogToolSearchFiles, self).__init__()
        self.setMinimumSize(400, 220)
        self.setMaximumSize(400, 220)

        self.parent = parent
        self.dataViewer = dataViewer

        self.groupBoxSearch = QGroupBox(self)
        self.groupBoxSearch.setGeometry(5, 5, 390, 160)
        
        self.lblLineEdit = QLabel(self)
        self.lblLineEdit.setText("%s : " % QApplication.translate("QSphere", "Search for"))
        self.lblLineEdit.setGeometry(10, 15, 80, 25)
        self.lblLineEdit.setAlignment(Qt.AlignRight)
        
        self.lineEdit = QLineEdit(self)
        self.lineEdit.setGeometry(100, 10, 280, 20)

        self.lblComboCols = QLabel(self)
        self.lblComboCols.setText("%s : " % QApplication.translate("QSphere", "in").title())
        self.lblComboCols.setGeometry(10, 45, 80, 25)
        self.lblComboCols.setAlignment(Qt.AlignRight)
       
        self.comboCols = QComboBox(self)
        self.comboCols.setGeometry(100, 40, 280, 20)
        self.comboCols.setObjectName("comboCols")
        self.comboCols.addItems(self.dataViewer.model().columnNames()) 
  
        self.caseCheckBox = QCheckBox(self)
        self.caseCheckBox.setText(QApplication.translate("QSphere", "Match case"))
        self.caseCheckBox.setGeometry(100, 65, 180, 20)
        
        self.wholeWordCheckBox = QCheckBox(self)
        self.wholeWordCheckBox.setText(QApplication.translate("QSphere", "Match whole word"))
        self.wholeWordCheckBox.setGeometry(100, 85, 180, 20)

        self.model = QStandardItemModel()
        self.view = QTableView()
        self.view.setModel( self.model )
        self.view.horizontalHeader().setVisible(False)
        self.view.verticalHeader().setVisible(False)
        self.view.verticalHeader().setDefaultSectionSize(20)
        self.view.setStyleSheet("QTableView {selection-background-color: #000000;}")
        self.view.setShowGrid(False)
        self.view.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.view.setMinimumSize(525, 20)
        

        self.comboLines = QComboBox(self)
        self.comboLines.setGeometry(100, 110, 280, 40)
        self.comboLines.setObjectName("comboLines")
        self.comboLines.setModel(self.model)
        self.comboLines.setView(self.view)

        self.countButton = QPushButton(self)
        self.countButton.setText("&%s" % (QApplication.translate("QSphere", "Find")))
        self.countButton.setGeometry(10, 110, 80, 25)
        self.countButton.setEnabled(False)
  
        self.closeButton = QPushButton(self)
        self.closeButton.setText("&%s" % (QApplication.translate("QSphere", "Close")))
        self.closeButton.setGeometry(290, 180, 100, 25)
  
        self.lineEdit.textChanged.connect(self.enableFindButton)
        self.comboLines.currentIndexChanged.connect(self.gotoLine)
        self.countButton.clicked.connect(self.countFindClicked)
        self.closeButton.clicked.connect(self.closeDlg)

        self.setWindowTitle(QApplication.translate("QSphere", "Search word"))
        self.lineEdit.setFocus()
        

    def enableFindButton(self, text): self.countButton.setEnabled(not text=="")
    def closeDlg(self): self.close()
    
    def gotoLine(self):
        if self.comboLines.currentIndex() == -1 : return
        model = self.comboLines.model()
        try : itemIndex = int(model.item(self.comboLines.currentIndex(), 0).text()) -1
        except : itemIndex = 0
        index = self.dataViewer.model().index(itemIndex, self.comboCols.currentIndex()) 
        self.dataViewer.selectionModel().setCurrentIndex(index, QItemSelectionModel.Select)
        self.dataViewer.selectionModel().select(index, QItemSelectionModel.ClearAndSelect)        

        
    def countFindClicked(self):
        from qsphere.qsphere_tools import sendMsg
        
        text, ignoreCase, wholeWord = self.getFindText(), self.isCaseSensitive(), self.isWholeWord()
        line = index = 0
        lstPos = []
        self.comboLines.clear()

        if not wholeWord : text = "*%s*" % text 

        self.filter = QRegExp(r"%s" % text)
        self.filter.setCaseSensitivity(Qt.CaseInsensitive) if not self.isCaseSensitive() else self.filter.setCaseSensitivity(Qt.CaseSensitive)
        
        if not wholeWord : self.filter.setPatternSyntax(QRegExp.Wildcard)
        
        listResults = []
        col = self.comboCols.currentIndex()
        for row in range(self.dataViewer.model().rowCount()):
            modelIndex = self.dataViewer.model().index(row, col)
            if self.filter.exactMatch(self.dataViewer.model().data(modelIndex, Qt.DisplayRole)):
                    self.appendItemInModel(self.model, "%s" % (row+1), self.dataViewer.model().data(modelIndex, Qt.DisplayRole))
                    listResults.append("%s" % (row+1))        

        self.view.resizeColumnsToContents()
        self.view.resizeRowsToContents()
        
        title = QApplication.translate("QSphere", "Information")
        msg = QApplication.translate("QSphere","Count")
        sendMsg(self.parent, title , "%s : <u><i>%s</i></ul>" % (msg, len(listResults)), Qgis.Info, 2)

      
    def getFindText(self): return self.lineEdit.text()
    def isCaseSensitive(self): return self.caseCheckBox.isChecked()
    def isWholeWord(self): return self.wholeWordCheckBox.isChecked()
    def appendItemInModel(self, model, line, elt):
        if (line == "") : return
        model.appendRow([QStandardItem(line), QStandardItem(fill(elt.strip(), 70))])



class findDialog(QDialog):
    findNext = pyqtSignal(str, int)
    findPrevious = pyqtSignal(str, int)
  
    def __init__(self, *args, **kwargs):
        super(findDialog, self).__init__(*args, **kwargs)
        self.setMinimumSize(400, 200)
        self.setMaximumSize(400, 200)

        self.groupBoxSearch = QGroupBox(self)
        self.groupBoxSearch.setGeometry(5, 5, 390, 160)
        
        self.lbl = QLabel(self)
        self.lbl.setText(QApplication.translate("QSphere", "Search for:"))
        self.lbl.setGeometry(10, 15, 80, 25)
        self.lbl.setAlignment(Qt.AlignRight)
        self.lineEdit = QLineEdit(self)
        self.lineEdit.setGeometry(100, 10, 280, 20)
        self.ret = 0
  
        self.caseCheckBox = QCheckBox(self)
        self.caseCheckBox.setText(QApplication.translate("QSphere", "Match case"))
        self.caseCheckBox.setGeometry(100, 40, 180, 20)
        
        self.wholeWordCheckBox = QCheckBox(self)
        self.wholeWordCheckBox.setText(QApplication.translate("QSphere", "Match whole word"))
        self.wholeWordCheckBox.setGeometry(100, 60, 180, 20)

        self.model = QStandardItemModel()
        self.view = QTableView()
        self.view.setModel( self.model )
        self.view.horizontalHeader().setVisible(False)
        self.view.verticalHeader().setVisible(False)
        self.view.setColumnHidden(1, True)
        self.view.horizontalHeader().setDefaultSectionSize(400)
        self.view.verticalHeader().setDefaultSectionSize(20)
        self.view.setStyleSheet("QTableView {selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 %s, stop: 1 %s);}" % ("#FFFFFF", "#000000"))
        self.view.setShowGrid(False)
        self.view.setMinimumSize(500, 40)

        self.comboLines = QComboBox(self)
        self.comboLines.setGeometry(100, 90, 280, 40)
        self.comboLines.setObjectName("comboLines")
        self.comboLines.setModel(self.model)
        self.comboLines.setView(self.view)

        self.lblCounter = QLabel(self)
        self.lblCounter.setText("0")
        self.lblCounter.setGeometry(30, 120, 40, 25)
        self.lblCounter.setAlignment(Qt.AlignCenter)
        self.lblCounter.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.countButton = QPushButton(self)
        self.countButton.setText("&%s" % (QApplication.translate("QSphere", "Find")))
        self.countButton.setGeometry(10, 90, 80, 25)
        self.countButton.setEnabled(False)
  
        self.closeButton = QPushButton(self)
        self.closeButton.setText("&%s" % (QApplication.translate("QSphere", "Close")))
        self.closeButton.setGeometry(290, 170, 100, 25)
  
        self.lineEdit.textChanged.connect(self.enableFindButton)
        self.comboLines.currentIndexChanged.connect(self.gotoLine)
        self.countButton.clicked.connect(self.countFindClicked)
        self.closeButton.clicked.connect(self.closeDlg)

        self.setWindowTitle(QApplication.translate("QSphere", "Search word"))
        self.lineEdit.setFocus()
        

    def enableFindButton(self, text): self.countButton.setEnabled(not text=="")
    def closeDlg(self): self.close()
    
    def gotoLine(self):
        if self.comboLines.currentIndex() == -1 : return
        model = self.comboLines.model()
        try : pos = model.item(self.comboLines.currentIndex(),1).text()
        except : pos = "0|0"
        coord = pos.split("|")
        try : line = int(coord[0])
        except : line = 0
        try : index = int(coord[1])
        except : index = 0

        text = self.getFindText()
        self.parent().setCursorPosition(line-1,index-len(text))
        self.parent().findFirst(self.getFindText(), True, self.isCaseSensitive(), self.isWholeWord(), True, True, line-1, index-len(text))

    def countFindClicked(self):
        text, ignoreCase, wholeWord = self.getFindText(), self.isCaseSensitive(), self.isWholeWord()
        line = index = 0
        lstPos = []
        self.comboLines.clear()

        self.parent().setCursorPosition(line,index)
        
        bFind = self.parent().findFirst(text, True, ignoreCase, wholeWord, True, True, line, index)
        line, index = self.parent().getCursorPosition()
        
        while bFind and not (line, index) in lstPos :
            lstPos.append((line, index))
            self.appendItemInModel(self.model, self.parent().text(line), "%s|%s" % (line+1, index))
            bFind = self.parent().findFirst(text, True, ignoreCase, wholeWord, True, True, line, index)
            line, index = self.parent().getCursorPosition()

        from qsphere.qsphere_tools import sendMsg
        self.parent().setCursorPosition(0,0)

        self.lblCounter.setText("%s" % len(lstPos))
             
        title = QApplication.translate("QSphere", "Information")
        msg = QApplication.translate("QSphere","Count")
        sendMsg(self.parent().parent, title , "%s : <u><i>%s</i></ul>" % (msg, len(lstPos)), Qgis.Info, self.parent().duration_info)

        self.gotoLine()
        self.ret = len(lstPos)

        
    def getFindText(self): return self.lineEdit.text()
    def isCaseSensitive(self): return self.caseCheckBox.isChecked()
    def isWholeWord(self): return self.wholeWordCheckBox.isChecked()
    def appendItemInModel(self, model, line, elt):
        if (elt == "") : return
        model.appendRow([QStandardItem(fill(line.strip(),50)), QStandardItem(elt)])


class filteredComboBox(QComboBox):
    def __init__(self, parent=None):
        super(filteredComboBox, self).__init__(parent)
        self.setFocusPolicy(Qt.StrongFocus)
        self.setEditable(True)

        self.pFilterModel = QSortFilterProxyModel(self)
        self.pFilterModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
        self.pFilterModel.setSourceModel(self.model())

        self.completer = QCompleter(self.pFilterModel, self)
        self.completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
        self.setCompleter(self.completer)

        def filter(text): self.pFilterModel.setFilterFixedString("%s" % (text))

        self.lineEdit().textEdited[unicode].connect(filter)
        self.completer.activated.connect(self.on_completer_activated)

    def on_completer_activated(self, text):
        if text:
            index = self.findText("%s" % (text))
            self.setCurrentIndex(index)

    def setModel(self, model):
        super(filteredComboBox, self).setModel(model)
        self.pFilterModel.setSourceModel(model)
        self.completer.setModel(self.pFilterModel)

    def setModelColumn(self, column):
        self.completer.setCompletionColumn(column)
        self.pFilterModel.setFilterKeyColumn(column)
        super(filteredComboBox, self).setModelColumn(column)

class editor(QTextEdit):
       def __init__(self, parent):
           super(editor, self).__init__(parent)
           self.setWindowTitle("QTextEdit Standard Output Redirection")
           self.setStyleSheet("color: white; background-color: black;")
           self.setObjectName(parent.objectName())
           
class dlgLustre(QDialog):
       def  __init__(self, parent, serverLusTRE, isModal):
         super(dlgLustre, self).__init__()
         self.setWindowTitle("LusTRE %s ..." % (QApplication.translate("QSphere","Web services")))
         self.minWidth, self.minHeight = 580, 400
         self.setMinimumSize(self.minWidth,self.minHeight)
         self.setMaximumSize(self.minWidth*2,self.minHeight*2)
         self.resize(self.minWidth,self.minHeight)

         self.parent = parent
         self.iface = self.parent.iface
         self.localLangage = self.parent.localLangage
         self.styleSheetPushButton = self.parent.styleSheetPushButton
         self.serverLusTRE = serverLusTRE
         self.isModal = isModal
         self.countkeywords = 0
         self.groupLusTREInfo = QGroupBox(self)

         from qsphere.qsphere_tools import (getThemeIcon, getOptions, windowIcon)

         windowIcon(self, "lustre_wizard.png")
         getOptions(self)
         
         icon = getThemeIcon("servers.png")
         carIcon = QImage(icon)
         self.ImgServerLusTRE = QLabel(self.groupLusTREInfo)
         self.ImgServerLusTRE.setObjectName("ImgServerLusTRE")
         self.ImgServerLusTRE.setAlignment(Qt.AlignCenter)
         self.ImgServerLusTRE.setPixmap(QPixmap.fromImage(carIcon).scaled(42,42))

         self.lblServerLusTRE = QLabel(self.groupLusTREInfo)
         self.lblServerLusTRE.setObjectName("lblServerLusTRE")
         self.lblServerLusTRE.setAlignment(Qt.AlignLeft)
         self.lblServerLusTRE.setWordWrap(True)
         self.lblServerLusTRE.setText(self.serverLusTRE)

         self.labelKeywordForLusTRE = QLabel(self.groupLusTREInfo)
         self.labelKeywordForLusTRE.setText("%s : " % (QApplication.translate("QSphere","Keyword")))
         self.labelKeywordForLusTRE.setObjectName("labelUserServerCSWT")
         self.labelKeywordForLusTRE.setAlignment(Qt.AlignRight)    

         self.keywordForLusTRE = MyWidgetLineEdit(self.groupLusTREInfo)
         self.keywordForLusTRE.initType(6)
         self.keywordForLusTRE.setObjectName("keywordForLusTRE")
         self.keywordForLusTRE.textChanged.connect(self.keywordForLusTRE.checkRegExp)

         icon = getThemeIcon("find.png") 
         toolTip = QApplication.translate("QSphere","Get suggestion from the server")
         self.getSuggestionButton = myPushButton(self.groupLusTREInfo) 
         self.getSuggestionButton.initPushButton(24, 24, 50, 50, "getSuggestionButton", "", "%s %s" % (toolTip, "LusTRE"), True, icon, 24, 24, True)  

         self.labelParamNbKeywords = QLabel(self.groupLusTREInfo)
         self.labelParamNbKeywords.setObjectName("labelParamNbKeywords")
         self.labelParamNbKeywords.setAlignment(Qt.AlignRight)
         self.labelParamNbKeywords.setWordWrap(True)
         self.labelParamNbKeywords.setText("%s :" % (QApplication.translate("QSphere","Num max results")))
         self.labelParamNbKeywords.setGeometry(100, 75, 120, 25)

         self.paramNbKeywords = QSpinBox(self.groupLusTREInfo)
         self.paramNbKeywords.setGeometry(220, 70, 40, 25)
         self.paramNbKeywords.setMinimum(10)
         self.paramNbKeywords.setMaximum(100)
         self.paramNbKeywords.setValue(10)
         self.paramNbKeywords.setSingleStep(5)
         self.paramNbKeywords.setObjectName("paramNbKeywords")

         self.labelParamLangKeywords = QLabel(self.groupLusTREInfo)
         self.labelParamLangKeywords.setObjectName("labelParamLangKeywords")
         self.labelParamLangKeywords.setAlignment(Qt.AlignRight)
         self.labelParamLangKeywords.setWordWrap(True)
         self.labelParamLangKeywords.setText("%s :" % (QApplication.translate("QSphere","Language")))
         self.labelParamLangKeywords.setGeometry(260, 75, 80, 25)

         self.ParamLangKeywords = QComboBox(self.groupLusTREInfo)
         self.ParamLangKeywords.setGeometry(340, 70, 40, 25)
         self.ParamLangKeywords.addItems(sorted(self.parent.qspherePlugin.dicoLangs))
         self.ParamLangKeywords.setEditable(True)
         self.ParamLangKeywords.setInsertPolicy(QComboBox.InsertAlphabetically)
         self.ParamLangKeywords.setObjectName("ParamLangKeywords")

         self.allChildsActive = QCheckBox(self.groupLusTREInfo)
         self.allChildsActive.setGeometry(390, 70, 150, 25)
         self.allChildsActive.setText(QApplication.translate("QSphere","All children are activated"))
         self.allChildsActive.setObjectName("allChildsActive")
         self.allChildsActive.setChecked(False)         

         self.treeWidgetExpand = myPushButton(self.groupLusTREInfo)
         self.treeWidgetExpand.initPushButton(40, 24, 5, 70, "treeWidgetExpand", "", QApplication.translate("QSphere", "Expand all"), True, getThemeIcon("tree_expand.png"), 40, 24, True)
        
         self.treeWidgetReduce = myPushButton(self.groupLusTREInfo)
         self.treeWidgetReduce.initPushButton(40, 24, 45, 70, "treeWidgetReduce", "", QApplication.translate("QSphere", "Reduce all"), True, getThemeIcon("tree_reduce.png"), 40, 24, True)

         langage = self.parent.qspherePlugin.dicoLangs[self.parent.qspherePlugin.indexLang]
         index = self.ParamLangKeywords.findText(langage) if self.ParamLangKeywords.findText(langage) !=-1 else 0
         self.ParamLangKeywords.setCurrentIndex(index)

         self.treeKeywords = QTreeWidget(self.groupLusTREInfo)
         self.treeKeywords.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.treeKeywords.setObjectName("treeKeywords")
         self.treeKeywords.headerItem().setText(0, "1")
         self.treeKeywords.header().setVisible(False)
         self.treeKeywords.header().setDefaultSectionSize(200)

         self.treeKeywords.setStyleSheet("* { font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black}"
                                         "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }")

         self.labelResultKeywords = QLabel(self)
         self.labelResultKeywords.setAlignment(Qt.AlignLeft)
         self.labelResultKeywords.setObjectName("labelResultKeywords")
         self.labelResultKeywords.setText("")

         self.grid = QGridLayout(self.treeKeywords)
         widget = eval("QgsBusyIndicatorDialog")
         self.movieCtrl = widget(None)
         self.grid.addWidget(self.movieCtrl, 0, 0)
         self.grid.setAlignment(self.movieCtrl, Qt.AlignTop)
         self.movieCtrl.setVisible(False)

         self.barInfo = QgsMessageBar(self)
         self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

         self.quitButton = myPushButton(self)
         self.quitButton.initPushButton(80, 25, 10,  5, "close", QApplication.translate("QSphere","Close"),
                                        QApplication.translate("QSphere","Close"), True, "", 22, 22, False)
         self.quitButton.clicked.connect(self.closeDlg)

         self.initTreeKeywords()
         self.refreshCounter(self.countkeywords)

         self.getSuggestionButton.clicked.connect(self.getSuggestion)
         self.treeWidgetExpand.clicked.connect(self.expandTreeWidget)
         self.treeWidgetReduce.clicked.connect(self.expandTreeWidget)         
         self.treeKeywords.itemDoubleClicked.connect(self.addKeyword)
         
       def changeActivateWindow(self, int) :  self.parent.changeActivateWindow(int)   

       def resizeEvent(self, ev):
                zSize = ev.size()

                self.groupLusTREInfo.setGeometry(10, 10, zSize.width()-20, zSize.height()-60)

                self.ImgServerLusTRE.setGeometry(40, 2, 48, 48)
                self.lblServerLusTRE.setGeometry(90, 10, zSize.width()-110, 40)
                         
                self.labelKeywordForLusTRE.setGeometry(10, 45, 80, 25)
                self.keywordForLusTRE.setGeometry(90, 40, zSize.width()-150, 25)
                self.getSuggestionButton.setGeometry(zSize.width()-55, 40, 24, 24)

                self.treeKeywords.setGeometry(10, 100, self.groupLusTREInfo.width()-20, self.groupLusTREInfo.height()-110)
                self.barInfo.setGeometry(0, 0, zSize.width(), 90)

                self.labelResultKeywords.setGeometry(25, zSize.height()-45, 200, 25)
                self.quitButton.setGeometry(zSize.width()-90, zSize.height()-40, 80, 25)

       def closeEvent(self,event): self.parent.dialogLUSTRE = None
          
       def expandTreeWidget(self):
               if self.sender() == self.treeWidgetExpand : self.treeKeywords.expandAll()
               else :
                   self.treeKeywords.collapseAll()
                   self.treeKeywords.expandItem(self.treeKeywords.itemAt(0, 0))

       def refreshCounter(self, countKeywords):
               keywords = QApplication.translate("QSphere", "keywords") if countKeywords > 1 else QApplication.translate("QSphere", "keyword") 
               self.labelResultKeywords.setText("%s %s %s %s [%s]." % (countKeywords, keywords,
                                                             QApplication.translate("QSphere", "in"), QApplication.translate("QSphere", "language"),
                                                             self.ParamLangKeywords.currentText()))  

       def startMovie(self): self.movieCtrl.setVisible(True)
       def stopMovie(self): self.movieCtrl.setVisible(False)

       def initTreeKeywords(self):
               self.results_items = None
               self.results_items = QTreeWidgetItem()
               self.results_items.setText(0, QApplication.translate("QSphere", "Results"))

               self.treeKeywords.addTopLevelItem(self.results_items)
               self.treeKeywords.expandAll()
               self.treeKeywords.resizeColumnToContents(0)
               self.treeKeywords.resizeColumnToContents(1)
               self.countkeywords = 0

       def qgisProxy(self):
                settings = QSettings() 
                proxyEnabled = settings.value("proxy/proxyEnabled", "")
                proxyType = settings.value("proxy/proxyType", "" )
                proxyHost = settings.value("proxy/proxyHost", "" )
                proxyPort = settings.value("proxy/proxyPort", "" )
                proxyUser = settings.value("proxy/proxyUser", "" )
                proxyPassword = settings.value("proxy/proxyPassword", "" )
                return (proxyEnabled, proxyHost, proxyPort, proxyUser, proxyPassword, proxyType)

       def testUrl(self, useCase):
              from qsphere.qsphere_tools import getThemeIcon, multiGetRequest 
              import urllib

              if self.keywordForLusTRE.text() == "" :
                 self.msgNotValid(self.keywordForLusTRE.text())
                 return    

              self.startMovie()       
              libelleDict = {1: "suggestions"}
              libelleSubDicts = {1: ("title", "language", "conceptUri", "source")}
              langage = self.parent.qspherePlugin.dicoLangs[self.parent.qspherePlugin.indexLang]

              try : searchkeywords = quote(self.keywordForLusTRE.text())
              except : searchkeywords = self.keywordForLusTRE.text().encode('utf-8')
       
              if useCase == 0 : url = self.serverLusTRE
              elif useCase == 1 : url = "%stfes/rest/GetSuggestions?keyword=%s&maxCount=%s&languages=%s&source=true" % (self.serverLusTRE, searchkeywords, self.paramNbKeywords.value(), self.ParamLangKeywords.currentText())
                     
              sites = [('%s' % (url), 'GET', {'User-Agent': 'Mozilla/5.0', 'Accept': '*/*'}, 'None')]
              isValidURL = multiGetRequest(None, self.movieCtrl, sites, 10, 1)[0][1]
              
              mycodelang = self.parent.languesDico[langage]['bibliographic']               
              menuIcon = getThemeIcon("dkeywords.png")
              menuIconMetadata = getThemeIcon("metadata.png")
              menuIconSource = getThemeIcon("source.png")
              menuIconLanguage = getThemeIcon("ressources/images/%s.png" % (mycodelang))

              if isValidURL != None :

                 if useCase > 0 :

                        test = True
                        isValidURL = isValidURL.decode("utf-8", "replace")
                        try : mydict = literal_eval(isValidURL)
                        except :
                              self.msgNotValid(url)
                              test = False

                        if test :     
                               self.treeKeywords.clear()
                               self.initTreeKeywords()

                               zListSubDicts = mydict[libelleDict[useCase]]
                               for dicts in zListSubDicts :
                                   keywordItem = None 
                                   for i in range(len(libelleSubDicts[useCase])):
                                       if i == 0 :  
                                          keywordItem = QTreeWidgetItem()
                                          keywordItem.setIcon(0, QIcon(menuIcon))
                                          if libelleSubDicts[useCase][0] in dicts : keywordItem.setText(0, "%s" % (dicts[libelleSubDicts[useCase][0]]))
                                          if libelleSubDicts[useCase][2] in dicts : keywordItem.setToolTip(0,  "%s" % (dicts[libelleSubDicts[useCase][2]]))
                                          
                                       else :
                                          if keywordItem != None and libelleSubDicts[useCase][i] in dicts :    
                                             keywordItemMeta = QTreeWidgetItem()

                                             if libelleSubDicts[useCase][i] in ("language", "source") :
                                                if libelleSubDicts[useCase][i]== "language" : keywordItemMeta.setIcon(0, QIcon(menuIconLanguage))
                                                else : keywordItemMeta.setIcon(0, QIcon(menuIconSource))
                                                keywordItemMeta.setText(0, "%s : %s" % (QApplication.translate("QSphere", libelleSubDicts[useCase][i]), dicts[libelleSubDicts[useCase][i]]))
                                             else :
                                                keywordItemMeta.setIcon(0, QIcon(menuIconMetadata))    
                                                keywordItemMeta.setText(0, "%s" % (dicts[libelleSubDicts[useCase][i]]))
                                             keywordItem.addChild(keywordItemMeta)
                                   self.results_items.addChild(keywordItem)
                                   self.countkeywords+= 1
                            
              else :
                self.msgNotValid(url)
 
              self.refreshCounter(self.countkeywords)
              self.stopMovie()
      
              return isValidURL

       def msgNotValid(self, url):
           from qsphere.qsphere_tools import sendMsg  
           title = QApplication.translate("QSphere", "Warning" ) 
           msg = "<b><u>%s</u></b><br><i>%s</i>." % (url, QApplication.translate("QSphere", "Not valid"))
           sendMsg(self, title, msg, Qgis.Warning, self.parent.duration_warning)

       def getCapabilities(self): self.testUrl(0)
       def getSuggestion(self) : self.testUrl(1)

       def addKeyword(self, item):
           from qsphere.qsphere_tools import sendMsg  
           if item == None : return

           if item.toolTip(0)=="":
              language =  "%s : " % (QApplication.translate("QSphere", "language"))
              source =  "%s : " % (QApplication.translate("QSphere", "source"))
              if not self.allChildsActive.isChecked():
                 if not (item.text(0).startswith(language) or item.text(0).startswith(source)) : self.parent.loadNavigator(item.text(0)) 
              else :   
                 if item.text(0).startswith(language) : url = "https://www.google.fr/?gws_rd=ssl#q=%s+language+wiki" % (item.text(0).replace(language,""))
                 elif item.text(0).startswith(source) : url = "https://www.google.fr/?gws_rd=ssl#q=%s+thesaurus+wiki" % (item.text(0).replace(source,""))
                 else : url = item.text(0)
                 self.parent.loadNavigator(url)
              return

           dlgMetadata = None
           
           if self.parent.mdi.subWindowList(QMdiArea.CreationOrder)!= [] :
              dialog = self.parent.mdi.activeSubWindow()
              if dialog == None : return
              from qsphere.ui import dlgMetadata
              for child in dialog.children():
                   if type(child) == dlgMetadata :
                      dlgMetadata = child
                      break  
           else :
                 title = QApplication.translate("QSphere", "Warning" ) 
                 msg = QApplication.translate("QSphere","No Windows")
                 picto = Qgis.Warning
                 duration = self.parent.duration_warning
                 sendMsg(self, title, "%s !" % (msg), picto, duration)
                 return


           if dlgMetadata == None :
                 title = QApplication.translate("QSphere", "Warning" ) 
                 msg = QApplication.translate("QSphere","No Metadata Windows")
                 picto = Qgis.Warning
                 duration = self.parent.duration_warning
                 sendMsg(self, title, "%s !" % (msg), picto, duration)
                 return

           state = item.isExpanded()          

           action = dlgMetadata.findChild(QAction,"%s_6_%s" % ("add", "tablemotsclefsf"))
           if action != None : action.triggered.emit(True)

           obj = dlgMetadata.findChild(myTableWidget,"tablemotsclefsf")
           if obj != None :

              for mddcollapsibleGroupBox in dlgMetadata.mddcollapsibleGroupBox:
                  if mddcollapsibleGroupBox.objectName() == "tab2" : mddcollapsibleGroupBox.setCollapsed(False); break ;
                
              widget = obj.cellWidget(obj.rowCount()-1, 0)
              widget.setText(item.text(0))
              widget = obj.cellWidget(obj.rowCount()-1, 1)
              widget.setChecked(True)
              widget = obj.cellWidget(obj.rowCount()-1, 2)
              textSource = item.child(2).text(0)
              textToReplace = "%s : " % (QApplication.translate("QSphere", "source"))
              widget.setText(textSource.replace(textToReplace, ""))
              widget = obj.cellWidget(obj.rowCount()-1, 3)
              widget.setText(datetime.now().replace(microsecond=0).strftime("%Y-%m-%d"))
              obj.selectRow(obj.rowCount()-1)
              dlgMetadata.passChangeMode()
              
           item.setExpanded(not state)


       def closeDlg(self):
           self.parent.dialogLUSTRE = None   
           self.close()


           
class dlgSelectExtensions(QDialog):
    def __init__(self, parent, services, selectedExtensions):
        super(dlgSelectExtensions, self).__init__()
        self.parent = parent
        self.services = services
        self.selectedExtensions = selectedExtensions
        self.minWidth, self.minHeight = 510, 420
        self.setMinimumSize(QSize(self.minWidth,self.minHeight))
        self.setMaximumSize(QSize(self.minWidth,self.minHeight))
        self.setupUi()


    def setupUi(self):
        from qsphere.qsphere_tools import getThemeIcon
        
        self.setObjectName("dlgSelectExtensions")

        self.lblListAllExtensions = QLabel(self)
        self.lblListAllExtensions.setGeometry(10, 10, 240, 20)
        self.lblListAllExtensions.setObjectName("lblListAllExtensions")
        self.lblListAllExtensions.setAlignment(Qt.AlignCenter)

        self.lblListSelectedExtensions = QLabel(self)
        self.lblListSelectedExtensions.setGeometry(260, 10, 240, 20)
        self.lblListSelectedExtensions.setObjectName("lblListSelectedExtensions")
        self.lblListSelectedExtensions.setAlignment(Qt.AlignCenter)

        icon = getThemeIcon("additems.png")
        toolTip = QApplication.translate("QSphere","Add items")
        self.addItemsButton = myPushButton(self) 
        self.addItemsButton.initPushButton(26, 24, 240, 80, "addItemsButton", "", toolTip, True, icon, 26, 24, True)

        icon = getThemeIcon("removeitems.png")
        toolTip = QApplication.translate("QSphere","Remove items")
        self.removeItemsButton = myPushButton(self) 
        self.removeItemsButton.initPushButton(26, 24, 240, 120, "removeItemsButton", "", toolTip, True, icon, 26, 24, True)

        icon = getThemeIcon("new.png")
        toolTip = QApplication.translate("QSphere","Remove all items")
        self.removeAllItemsButton = myPushButton(self) 
        self.removeAllItemsButton.initPushButton(19, 24, 245, 160, "removeAllItemsButton", "", toolTip, True, icon, 19, 24, True)
        

        self.listAllExtensions = QListWidget(self)
        self.listAllExtensions.setGeometry(10, 30, 220, 340)
        self.listAllExtensions.setObjectName("listAllExtensions")
        self.listAllExtensions.setSelectionMode(QAbstractItemView.ExtendedSelection)

        self.listSelectedExtensions = QListWidget(self)
        self.listSelectedExtensions.setGeometry(280, 30, 220, 340)
        self.listSelectedExtensions.setObjectName("listSelectedExtensions")
        self.listSelectedExtensions.setSelectionMode(QAbstractItemView.ExtendedSelection)
  
        self.listAllExtensions.clear()
        self.listSelectedExtensions.clear()
        self.unSelectedExtensions=[a for a in self.services if a not in self.selectedExtensions]
        del self.unSelectedExtensions[0]
        self.majliste(self.listAllExtensions, self.unSelectedExtensions)

        for a in self.selectedExtensions:  self.listSelectedExtensions.addItem(QListWidgetItem(a))

        icon = getThemeIcon("sgbdr/bvalidate.png")
        toolTip = QApplication.translate("QSphere","Validate")
        self.acceptButton = myPushButton(self) 
        self.acceptButton.initPushButton(120, 24, 280, 380, "acceptButton", toolTip, toolTip, True, icon, 24, 24, True)
        
        icon = getThemeIcon("closewindow.png")
        toolTip = QApplication.translate("QSphere","Close")
        self.closeButton = myPushButton(self) 
        self.closeButton.initPushButton(120, 24, 380, 380, "closeButton", "&%s" % toolTip, toolTip, True, icon, 24, 24, True)

        self.listAllExtensions.doubleClicked.connect(self.addSelection)
        self.listSelectedExtensions.doubleClicked.connect(self.removeSelection)

        self.addItemsButton.clicked.connect(self.addSelection)
        self.removeItemsButton.clicked.connect(self.removeSelection)
        self.removeAllItemsButton.clicked.connect(self.cleanSelection)
        self.acceptButton.clicked.connect(self.accept)
        self.closeButton.clicked.connect(self.reject)

        self.retranslateUi()


    def retranslateUi(self):
        self.setWindowTitle(QApplication.translate("QSphere","GDAL/OGR extensions")) 
        self.lblListAllExtensions.setToolTip(QApplication.translate("QSphere","Enabled extensions"))
        self.listAllExtensions.setToolTip(QApplication.translate("QSphere","Double click to add current extension in selected extensions")) 
        self.lblListSelectedExtensions.setToolTip(QApplication.translate("QSphere","Selected extensions"))
        self.listSelectedExtensions.setToolTip(QApplication.translate("QSphere","Double click to remove current extension from selected extensions"))
        self.majLabels()


    def majLabels(self):
        self.lblListAllExtensions.setText("%s (%s)" % (self.lblListAllExtensions.toolTip(), self.listAllExtensions.count()))
        self.lblListSelectedExtensions.setText("%s (%s)" % (self.lblListSelectedExtensions.toolTip(), self.listSelectedExtensions.count()))                                           


    def addSelection(self):
        items = self.listAllExtensions.selectedItems()
        for item in items :
            self.selectedExtensions.append(item.text())
            self.unSelectedExtensions.remove(item.text())
        self.selectedExtensions = sorted(self.selectedExtensions)
        self.unSelectedExtensions = sorted(self.unSelectedExtensions)
        self.majliste(self.listAllExtensions, self.unSelectedExtensions)
        self.majliste(self.listSelectedExtensions, self.selectedExtensions)
        self.majLabels()     

    def removeSelection(self):
        items = self.listSelectedExtensions.selectedItems()
        for item in items :        
                self.selectedExtensions.remove(item.text())
                self.selectedExtensions = sorted(self.selectedExtensions)
                self.unSelectedExtensions.append(item.text())
        self.unSelectedExtensions = sorted(self.unSelectedExtensions)
        self.majliste(self.listAllExtensions, self.unSelectedExtensions)
        self.majliste(self.listSelectedExtensions, self.selectedExtensions)
        self.majLabels()

    def cleanSelection(self):
        for i in range(self.listSelectedExtensions.count()) : self.listSelectedExtensions.item(i).setSelected(True)    
        self.removeSelection()    

   
    def majliste(self, listewidget, liste):
        listewidget.clear()
        for item in liste: listewidget.addItem(item)

    
    def getCompositeExtensions(self):
        listExtensionsSel = [self.listSelectedExtensions.item(i).text() for i in range(self.listSelectedExtensions.count())]
        if '*' in listExtensionsSel : listExtensionsSel.remove('*')
        if listExtensionsSel == [] : listExtensionsSel = ['*'] 
        return "%s" % listExtensionsSel

        
    def closeEvent(self, event):
        event.accept()
    
class toolSearchFilesDialog(QDialog):
      def  __init__(self, parent):
            super(toolSearchFilesDialog, self).__init__()
            self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Search files")))
            self.minWidth, self.minHeight = 800, 800
            self.parent = parent
            self.iface = self.parent.iface
            self.setMinimumSize(self.minWidth,self.minHeight)
            self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
            layout = QVBoxLayout(self)

            from qsphere.qsphere_tools import getThemeIcon, loadSimpleData, getOptions, windowProperties, windowIcon, html

            self.initDirSearchFiles = ""
            self.listFiltersSearch = loadSimpleData(self, "file:/ressources/filters_search.csv") 
            if self.listFiltersSearch == [] : self.listFiltersSearch = ['*']
            windowIcon(self, "toolssearchfiles.png")
            getOptions(self)
            windowProperties(self, self.parent)
                
            self.dataViewer = QTableView(self)
            self.dataViewer.setAccessibleDescription("")

            self.initConteneurs()

            sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
            sizePolicy.setHorizontalStretch(0)
            sizePolicy.setVerticalStretch(3)
            sizePolicy.setHeightForWidth(self.dataViewer.sizePolicy().hasHeightForWidth())
            self.dataViewer.setSizePolicy(sizePolicy)
            self.dataViewer.setSelectionBehavior(QAbstractItemView.SelectRows)
            self.dataViewer.setSelectionMode(QAbstractItemView.SingleSelection)
            self.dataViewer.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
            self.dataViewer.setVisible(True)
            self.dataViewer.setAlternatingRowColors(True)
            self.dataViewer.setStyleSheet("QTableView {alternate-background-color: cyan;background-color: white;}")

            model = baseTableModel("", None, self) 
            self.dataViewer.setModel(model)

            self.mddcollapsibleGroupBox = []

            self.groupParams = QGroupBox(self)
            self.groupParams.setTitle("%s : " % (QApplication.translate("QSphere","Tools and Params")))
            
            self.groupParamSearch = QgsCollapsibleGroupBox(self.groupParams)
            self.groupParamSearch.setTitle("%s : " % (QApplication.translate("QSphere","Params")))
            self.groupParamSearch.setObjectName("groupParamSearch")

            self.mddcollapsibleGroupBox.append(self.groupParamSearch)

            self.groupOptions = QgsCollapsibleGroupBox(self.groupParams)
            self.groupOptions.setTitle("%s : " % (QApplication.translate("QSphere","Options")))
            self.groupOptions.setObjectName("groupOptions")
            self.groupOptions.setCollapsed(False)

            self.mddcollapsibleGroupBox.append(self.groupOptions)

            self.groupParamSearch.collapsedStateChanged.connect(self.changePosGroups)
            self.groupOptions.collapsedStateChanged.connect(self.changePosGroups)

            self.html = html
            self.lastSavedFileQSR = ""

            self.lblPath = QLabel(self.groupParamSearch)
            self.lblPath.setAlignment(Qt.AlignRight)
            self.lblPath.setObjectName("lblPath")
            
            self.lblPath.setText("%s :" % QApplication.translate("QSphere", "Folder"))

            self.txtPath = QComboBox(self.groupParamSearch)
            self.txtPath.setEditable(False)
            self.txtPath.setEnabled(True)
            self.txtPath.setObjectName("txtPath")

            self.lblPattern = QLabel(self.groupParamSearch)
            self.lblPattern.setText("%s :" % QApplication.translate("QSphere", "Pattern"))
            self.lblPattern.setAlignment(Qt.AlignRight)

            self.lblFilters = QLabel(self.groupParamSearch)
            self.lblFilters.setText("%s :" % QApplication.translate("QSphere", "Extensions"))
            self.lblFilters.setAlignment(Qt.AlignRight)

            self.lblPath.setGeometry(5, 25, 65, 25)
            self.lblPattern.setGeometry(5, 55, 65, 25)
            self.lblFilters.setGeometry(5, 85, 65, 25)

            self.pattern = QLineEdit(self.groupParamSearch)
            self.pattern.setText("*")
            self.filters = QLineEdit(self.groupParamSearch)
            self.filters.setText("%s" % self.listFiltersSearch) 
            self.filters.setEnabled(False)
            
            self.chkSensibleCase = QCheckBox(self.groupOptions)
            self.chkSensibleCase.setObjectName("chkSensibleCase")
            self.chkSensibleCase.setText("aaa = AAA")
            self.chkSensibleCase.setToolTip(QApplication.translate("QSphere", "Case sensitive"))
            self.chkSensibleCase.setChecked(False)

            self.chkIncludePath = QCheckBox(self.groupOptions)
            self.chkIncludePath.setObjectName("chkIncludePath")
            self.chkIncludePath.setText(QApplication.translate("QSphere", "Include path"))
            self.chkIncludePath.setToolTip(QApplication.translate("QSphere", "Include path for filtering"))
            self.chkIncludePath.setChecked(False)

            self.chkIncludeVector = QCheckBox(self.groupOptions)
            self.chkIncludeVector.setObjectName("chkIncludeVector")
            self.chkIncludeVector.setText(QApplication.translate("QSphere", "Include vector files"))
            self.chkIncludeVector.setToolTip(QApplication.translate("QSphere", "Include vector files"))
            self.chkIncludeVector.setChecked(True)

            self.chkIncludeRaster = QCheckBox(self.groupOptions)
            self.chkIncludeRaster.setObjectName("chkIncludeRaster")
            self.chkIncludeRaster.setText(QApplication.translate("QSphere", "Include raster files"))
            self.chkIncludeRaster.setToolTip(QApplication.translate("QSphere", "Include raster files"))
            self.chkIncludeRaster.setChecked(True)            

            self.lblSearchFiles = QLabel(self.groupOptions)
            self.lblSearchFiles.setAlignment(Qt.AlignCenter)
            self.lblSearchFiles.setObjectName("lblSearchFiles")
            self.lblSearchFiles.setText("")

            self.searchFiles = myPushButton(self)
            self.searchFiles.initPushButton(64, 64, 220, 55, "searchFiles", "", QApplication.translate("QSphere", "Search files in the path ..."), True, getThemeIcon("searchfiles.png"), 64, 64, True)

            self.movieCtrl = movieCtrl(self, "sablier.gif")
            self.counter = self.movieCtrl.movie.frameCount()

            self.barInfo = QgsMessageBar(self)
            self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

            self.toolBar = QToolBar(self)
            self.toolBar.setObjectName("toolBar")
            self.toolBar.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
            self.toolBar.setWindowTitle(QApplication.translate("QSphere","Results actions"))
            self.actionsForToolBar(self.toolBar)

            self.toolBar.setStyleSheet("QToolBar {icon-size: 48px, 48px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E1E1E1, stop: 0.4 #9B9B9B, stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);  border-width: 2px; border-radius: 10px; border-color: beige;}")

            self.getPath = myPushButton(self.groupParamSearch)
            self.getPath.initPushButton(24, 24, 220, 27, "ChoosePath", "", QApplication.translate("QSphere", "Choose path ..."), True, getThemeIcon("folder.png"), 24, 24, True)


            self.saveFilters = myPushButton(self.groupParamSearch)
            self.saveFilters.initPushButton(24, 24, 50, 35, "saveFilters", "", QApplication.translate("QSphere", "Save filters"), True, getThemeIcon("save.png"), 24, 24, True)
    

            self.editFilters = myPushButton(self.groupParamSearch)
            self.editFilters.initPushButton(24, 24, 50, 35, "editFilters", "", QApplication.translate("QSphere", "Edit filters"), True, getThemeIcon("edit.png"), 24, 24, True)

                
            self.wizardFilters = myPushButton(self.groupParamSearch)
            self.wizardFilters.initPushButton(24, 24, 50, 35, "wizardFilters", "", QApplication.translate("QSphere", "Wizard filters"), True, getThemeIcon("wizards.png"), 24, 24, True)

            self.CloseButton = QPushButton(self)
            self.CloseButton.setObjectName("CloseButton")
            self.CloseButton.setText("&%s" % QApplication.translate("QSphere", "Close"))

            self.listExtensionsSel = []
                                        
            self.getPath.clicked.connect(self.fixePath)
            self.saveFilters.clicked.connect(self.saveFiltersInFile)
            self.editFilters.clicked.connect(self.changeEnabledFilters)
            self.wizardFilters.clicked.connect(self.getListExtensionsGdal)
            self.searchFiles.clicked.connect(self.makeListFiles)
            self.chkIncludeVector.clicked.connect(self.changeEnableWizardFilters)
            self.chkIncludeRaster.clicked.connect(self.changeEnableWizardFilters)
            self.CloseButton.clicked.connect(self.closeDlg)

            self.fixeEnabled()

      def closeDlg(self): self.close()  
      def initConteneurs(self): self.countfiles, self.countdirs, self.dirs = 0, 0, {}               
      def changeEnabledFilters(self) : self.filters.setEnabled(not self.filters.isEnabled()); 
      def changeEnableWizardFilters(self): self.wizardFilters.setEnabled(self.chkIncludeVector.isChecked() or self.chkIncludeRaster.isChecked())

      def getListExtensionsGdal(self) :
           from qsphere.qsphere_tools import (makeListFormatsExtensions, windowIcon)
           listExtensions = makeListFormatsExtensions(self, self.chkIncludeVector.isChecked(), self.chkIncludeRaster.isChecked(), False)
           listExtensionsSel = literal_eval(self.filters.text())
           dialog = dlgSelectExtensions(self, listExtensions, listExtensionsSel)
           windowIcon(dialog, "toolssearchfiles.png")
           if dialog.exec_():
              self.filters.clear()
              self.filters.setText(dialog.getCompositeExtensions())

      def getRadarStats(self):
           values = []
           if len(self.dirs) == 0 : return
           elif len(self.dirs.keys())== 1 :
                for key in self.dirs.keys():
                        dico = self.dirs[key]
                        keys = dico.keys()
                        for subkey in keys :
                            values.append(dico[subkey][0])
           else :
                keys = [key for key in self.dirs.keys()]
                for key in keys :
                    counter = 0
                    dico = self.dirs[key]
                    for subkey in dico : counter+= dico[subkey][0]
                    values.append(counter)
                    
           self.dlgStats = windowPlotStatsAndGraphs(self, "pie", values, tuple(keys), None) 
           self.dlgStats.exec_()
             

      def changePosGroups(self):
                 posH, newHeight = 15, 0
                 dicoHeight = {"groupParamSearch" : 120, "groupOptions" : 70}
                 for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
                     mddcollapsibleGroupBox.setMaximumWidth(self.width()-50)
                     mddcollapsibleGroupBox.setMinimumWidth(self.width()-50)
                     height = dicoHeight[mddcollapsibleGroupBox.objectName()]
                     mddcollapsibleGroupBox.setMaximumHeight(height)
                     mddcollapsibleGroupBox.setMinimumHeight(height)
                     mddcollapsibleGroupBox.setGeometry(5, posH, self.width()-50, mddcollapsibleGroupBox.maximumHeight())
                     posH+= mddcollapsibleGroupBox.height() if not mddcollapsibleGroupBox.isCollapsed() else 15
                     newHeight+= 15 + mddcollapsibleGroupBox.height() if not mddcollapsibleGroupBox.isCollapsed() else 25
                     
                 self.groupParams.setGeometry(10, 10, self.width()-20, min(newHeight+10, self.height()+10))
                 refHeight = self.groupParams.y()+self.groupParams.height()+20

                 self.dataViewer.setGeometry(10, refHeight, self.width()-20, self.height()-refHeight-40)
                 self.CloseButton.setGeometry(self.width()-110, self.height()-30, 100, 25)
                 self.searchFiles.setGeometry(self.width()-100, refHeight-50, 64, 64)
                 self.toolBar.setGeometry(self.width()/2-160, refHeight-40, 320, 40)
                 

      def resizeEvent(self, ev):
          self.changePosGroups()
          self.barInfo.setGeometry(0, 0, self.width(), 45)  
          self.movieCtrl.setGeometry(self.width()/2-32, self.height()/2-32, 64, 64)
          self.txtPath.setGeometry(75, 20, self.width()-195, 25)
          self.pattern.setGeometry(75, 50, self.width()-195, 25)
          self.filters.setGeometry(75, 80, self.width()-235, 25)
          self.getPath.setGeometry(self.txtPath.x()+self.txtPath.width()+10, 20, 24, 24)

          self.saveFilters.setGeometry(self.filters.x()+self.filters.width()+10, 80, 24, 24)      
          self.editFilters.setGeometry(self.filters.x()+self.filters.width()+40, 80, 24, 24)
          self.wizardFilters.setGeometry(self.filters.x()+self.filters.width()+70, 80, 24, 24)
          self.chkSensibleCase.setGeometry(self.filters.x(), 15, 120, 25)
          self.chkIncludePath.setGeometry(self.chkSensibleCase.x()+self.chkSensibleCase.width()+10, 15, 120, 25)

          self.chkIncludeVector.setGeometry(self.filters.x(), 40, 220, 25)
          self.chkIncludeRaster.setGeometry(self.chkIncludeVector.x()+self.chkIncludeVector.width()+10, 40, 220, 25)
          
          self.lblSearchFiles.setGeometry(self.width()-300, 40, 250, 25)

      def actionsForToolBar(self, toolbar):
          from qsphere.qsphere_tools import getThemeIcon

          menuIcon = getThemeIcon("search.png")
          self.findInListResults = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Find in results"), self)
          self.findInListResults.setObjectName("findInListResults")
          self.findInListResults.setShortcut(QKeySequence("Ctrl+Shift+F"))
          toolbar.addAction(self.findInListResults)
          self.findInListResults.triggered.connect(self.findInResults)

          toolbar.addSeparator()

          menuIcon = getThemeIcon("piechart.png")
          self.radarStats = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Get statistics chart"), self)
          self.radarStats.setObjectName("radarStats")
          self.radarStats.setShortcut(QKeySequence("Ctrl+R"))
          toolbar.addAction(self.radarStats)
          self.radarStats.triggered.connect(self.getRadarStats)
          

          menuIcon = getThemeIcon("print.png")
          self.printListResults = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Print search files results"), self)
          self.printListResults.setObjectName("printListResults")
          self.printListResults.setShortcut(QKeySequence("Ctrl+Shift+P"))
          toolbar.addAction(self.printListResults)
          self.printListResults.triggered.connect(self.printResults)
          
          toolbar.addSeparator()

          menuIcon = getThemeIcon("save.png")
          self.saveSearchFiles = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Save search files results"), self)
          self.saveSearchFiles.setObjectName("saveSearchFiles")
          self.saveSearchFiles.setShortcut(QKeySequence("Ctrl+S"))
          toolbar.addAction(self.saveSearchFiles)
          self.saveSearchFiles.triggered.connect(self.saveResultsFiles)

          menuIcon = getThemeIcon("garbage.png")
          self.deleteSearchFiles = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Delete all entries"), self)
          self.deleteSearchFiles.setObjectName("deleteSearchFiles")
          self.deleteSearchFiles.setShortcut(QKeySequence("Ctrl+D"))
          toolbar.addAction(self.deleteSearchFiles)
          self.deleteSearchFiles.triggered.connect(self.deleteResultsFiles)


      def closeEvent(self,event): self.parent.dialogSearchFiles = None

      def fixePath(self):
          from qsphere.qsphere_tools import fixeLabelsFileDialog 
          from os.path import (basename, dirname, exists, splitext)
          
          InitDir = dirname(__file__) if self.txtPath.currentText()=="" else dirname("%s" % (self.txtPath.currentText()))
          title = QApplication.translate("QSphere", "Select the folder to search files")
          dlgFile = QFileDialog(self, title)
          dlgFile.setDirectory(InitDir)
          dlgFile.setFileMode(QFileDialog.DirectoryOnly)

          fixeLabelsFileDialog(self, dlgFile, 0, False)          
          
          if dlgFile.exec_():
             inputDir = dlgFile.selectedFiles()[0] 
             target = "%s/" % (inputDir.replace("\\","/"))
             if self.txtPath.findText(target)==-1 : self.txtPath.addItem(target)
             self.txtPath.setCurrentIndex(self.txtPath.model().rowCount()-1)
             self.initDir = inputDir
          self.fixeEnabled()


      def fixeEnabled(self):
          self.txtPath.setEnabled(False) if self.txtPath.count()<=0 else self.txtPath.setEnabled(True)
          self.searchFiles.setEnabled(False) if self.txtPath.currentText()=="" else self.searchFiles.setEnabled(True)
          cond = (self.dataViewer.model().rowCount()>0)
          self.toolBar.setEnabled(cond)

      def saveFiltersInFile(self):
          try : listExtensions = literal_eval(self.filters.text())
          except : self.msgError(False, None); self.fixeEnabled(); return
          if not type(listExtensions) in (list, tuple) : self.msgError(False, None); self.fixeEnabled(); return

          fileName = join(dirname(__file__), "ressources/filters_search.csv")
          with open(fileName, 'w', encoding="utf-8", errors="surrogateescape") as inFile :
                          for extension in listExtensions : inFile.write('"%s"\n' % extension)
          self.msgError(True, fileName)      
              
      def msgError(self, result, fileName):
          from qsphere.qsphere_tools import sendMsg
          if not result :
                  msg = "%s<br>%s" % (QApplication.translate("QSphere","Error on unsupported content :"), QApplication.translate("QSphere","Not valid"))
                  sendMsg(self, "information" , msg, Qgis.Warning, self.duration_warning)    
          else :
                  msg = "%s :<br>%s<br>%s" % (QApplication.translate("QSphere", "Save the current list"), fileName, QApplication.translate("QSphere","Successfull"))
                  sendMsg(self, "information" , msg, Qgis.Info, self.duration_info)                   

      def makeListFiles(self):
          from os.path import exists
          from qsphere.qsphere_tools import sendMsg
          if not exists(self.txtPath.currentText()) :
                 title = QApplication.translate("QSphere","Information")
                 msg = "%s %s :<br>%s" % (QApplication.translate("QSphere","Error"), QApplication.translate("QSphere","folder"), self.txtPath.currentText())  
                 sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
                 self.txtPath.removeItem(self.txtPath.currentIndex())

                 self.initConteneurs()
                 
                 self.refreshCounter(self.countfiles, self.countdirs)        
                 try : self.txtPath.setCurrentIndex(0)
                 except : pass
                 self.fixeEnabled()
                 return 

               
          self.countfiles, self.countdirs, self.dirs = 0, 1, {}
          self.refreshCounter(self.countfiles, self.countdirs)

          try : listExtensions = literal_eval(self.filters.text())
          except : self.msgError(False, None); self.fixeEnabled(); return
          if not type(listExtensions) in (list, tuple) : self.msgError(False, None); self.fixeEnabled(); return

          self.movieCtrl.startMovie()
          
          self.vectorLayer = QgsVectorLayer('None?crs=epsg:2154&field=nom:string(255)&field=size:int(20)&field=sizeform:string(20)&field=datecrea:date&field=datemod:date&field=dateread:date', 'fichiers' , "memory")
          self.vectorLayer.startEditing()

          for extension in listExtensions :

                  self.filter = QRegExp("%s.%s" % (self.pattern.text(), extension))

                  self.countfiles, self.countdirs = 0, 1 

                  self.filter.setCaseSensitivity(Qt.CaseInsensitive) if not self.chkSensibleCase.isChecked() else self.filter.setCaseSensitivity(Qt.CaseSensitive)
                  self.filter.setPatternSyntax(QRegExp.Wildcard)

                  from qsphere.qsphere_tools import getFilesInFolder
                  self.countfiles, self.countdirs, dirs = getFilesInFolder(self, self.txtPath.currentText(), self.vectorLayer, self.counter, self.chkIncludePath.isChecked(), extension)

                  self.dirs[extension] = dirs

                  self.refreshCounter(self.countfiles, self.countdirs)

          self.vectorLayer.commitChanges()
          self.movieCtrl.stopMovie()
          QgsProject.instance().addMapLayer(self.vectorLayer, False) 

          fieldsNames = [field.name() for field in self.vectorLayer.fields()] 
          self.vectorLayer.selectAll()
          model = baseTableModel(fieldsNames, self.vectorLayer.selectedFeatures(), self) 
          self.dataViewer.setModel(model)

          self.fixeEnabled()
 

      def refreshCounter(self, countfiles, countdirs):
         sFile = QApplication.translate("QSphere", "files") if self.countfiles > 1 else QApplication.translate("QSphere", "file") 
         sFolder = QApplication.translate("QSphere", "folders") if self.countdirs > 1 else QApplication.translate("QSphere", "folder")
         self.lblSearchFiles.setText("%s %s %s %s %s." % (countfiles, sFile,
                                                      QApplication.translate("QSphere", "in"),
                                                      countdirs, sFolder))   

         
      def loadFile(self, Item):
          fileName = ""
          if self.sender() == self.listFiles :
            if self.listFiles.count()==0 : return
            if Item == None : return
            if Item.type()==1 :
                 self.infosFile.setText("")
                 self.printInfosFile.setEnabled(False)
                 return
            if not Item : return
            fileName = Item.toolTip()
          elif self.sender() == self.treeFiles :
            if Item == None : return
            if Item.toolTip(0)=="":
                 self.infosFile.setText("")
                 self.printInfosFile.setEnabled(False)
                 return
            fileName = Item.toolTip(0)
          if fileName == "": return
          self.synchronize(self.sender())

          from qsphere.qsphere_tools import fileInfosToHtml     
          infosHtml = fileInfosToHtml(self, QFileInfo(fileName))
          self.infosFile.setText(self.html % (infosHtml))
          self.printInfosFile.setEnabled(True)

 
      def findInResults(self):
          self.findDialog = findDialogToolSearchFiles(self, self.dataViewer)
          ret = self.findDialog.exec_()
          self.findDialog.lineEdit.setFocus()

      def deleteResultsFiles(self):
          model = baseTableModel("", None, self) 
          self.dataViewer.setModel(model)
          self.fixeEnabled()

      def saveResultsFiles(self):
              from qsphere.qsphere_tools import saveResultsInFile
              saveResultsInFile(self, self.initDirSearchFiles, self.vectorLayer)

      def printInfosForTheFile(self):
          dlgPrint = printerSphere(self, self.infosFile, QApplication.translate("QSphere", "Print informations for the file"), QPrinter.Landscape) 
          
      def printResults(self):
          document = QTextDocument()
          cursor = QTextCursor(document)
          model = self.dataViewer.model()
          if model == None : return

          table = cursor.insertTable(model.rowCount()+1, model.columnCount())
          for col in model.columnNames(): cursor.insertText("%s" % col);cursor.movePosition(QTextCursor.NextCell)
          for row in range(model.rowCount()) :
                    for column in range(table.columns()):
                        data = "%s" % model.getData(row, column)
                        cursor.insertText(data[:100])
                        cursor.movePosition(QTextCursor.NextCell)          

          dlgPrint = printerSphere(self, document, QApplication.translate("QSphere", "Print list of results"), QPrinter.Portrait)
          

                        
class loadFromLayersDialog(QDialog):

       def  __init__(self, iface, parent):
             super(loadFromLayersDialog, self).__init__()
             self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Load metadata file for layers")))
             self.minWidth, self.minHeight = 600, 500
             self.iface = iface
             self.parent = parent
             self.setMinimumSize(self.minWidth,self.minHeight)
             self.setMaximumSize(self.minWidth*2,self.minHeight*2)
             layout = QVBoxLayout(self)

             self.groupLayersInfo = QGroupBox(self)

             self.suffixe = "qsp"

             from qsphere.qsphere_tools import getThemeIcon
             icon = getThemeIcon("layers.png")
             carIcon = QImage(icon)
             self.imgQgisLayers = QLabel(self.groupLayersInfo)
             self.imgQgisLayers.setObjectName("imgQgisLayers")
             self.imgQgisLayers.setAlignment(Qt.AlignCenter)
             self.imgQgisLayers.setPixmap(QPixmap.fromImage(carIcon).scaled(48,48))        

             self.lblMetadataFilters = QLabel(self.groupLayersInfo)
             self.lblMetadataFilters.setObjectName("lblMetadataFilters")
             self.lblMetadataFilters.setAlignment(Qt.AlignLeft)
             self.lblMetadataFilters.setWordWrap(True)
             self.lblMetadataFilters.setText("%s :" % QApplication.translate("QSphere","Filters"))

             self.groupBoxObj = QGroupBox(self.groupLayersInfo)
             self.groupBoxObj.setObjectName("groupBoxObj")

             self.radioQsp = QRadioButton(self.groupBoxObj)
             self.radioQsp.setGeometry(QRect(10, 15,70,23))
             self.radioQsp.setObjectName("radioQsp")
             self.radioQsp.setText("QSP")

             self.radioXml = QRadioButton(self.groupBoxObj)
             self.radioXml.setGeometry(QRect(80, 15,70,23))
             self.radioXml.setObjectName("radioXml")
             self.radioXml.setText("XML")

             self.radioQmd = QRadioButton(self.groupBoxObj)
             self.radioQmd.setGeometry(QRect(150, 15,70,23))
             self.radioQmd.setObjectName("radioQmd")
             self.radioQmd.setText("QMD")
             
             self.radioAll = QRadioButton(self.groupBoxObj)
             self.radioAll.setGeometry(QRect(220, 15,80,23))
             self.radioAll.setObjectName("radioAll")
             self.radioAll.setText("All")

             self.radioQsp.clicked.connect(self.FixeFilter)
             self.radioXml.clicked.connect(self.FixeFilter)
             self.radioQmd.clicked.connect(self.FixeFilter)
             self.radioAll.clicked.connect(self.FixeFilter)

             self.radioQsp.setChecked(True)

             self.lblQgisLayers = QLabel(self.groupLayersInfo)
             self.lblQgisLayers.setObjectName("lblQgisLayers")
             self.lblQgisLayers.setAlignment(Qt.AlignLeft)
             self.lblQgisLayers.setWordWrap(True)
             self.lblQgisLayers.setText("%s :" % QApplication.translate("QSphere","Layers"))         

             self.listQgisLayers = QListWidget(self.groupLayersInfo)
             self.listQgisLayers.setObjectName("listQgisLayers")
             self.listQgisLayers.setStyleSheet("* { font-size:12px; background-color: #5D5D5D; padding: 4px ; color: rgb(255,255,255)}"
                                               "QListWidget::item:selected {background-color: rgb(255,255,255); color: black;}")
             self.listQgisLayers.setSelectionMode(QAbstractItemView.MultiSelection)
             self.listQgisLayers.setEditTriggers(QAbstractItemView.NoEditTriggers)
             self.listQgisLayers.setIconSize(QSize(80, 26))

             self.FixeFilter() 

             self.buttonBox = QDialogButtonBox()
             self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
             self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
             self.buttonBox.accepted.connect(self.accept)
             self.buttonBox.rejected.connect(self.reject)
             self.listQgisLayers.itemDoubleClicked.connect(self.loadOnDblClick)
             
             layout.addWidget(self.groupLayersInfo) 
             layout.addWidget(self.buttonBox) 
             self.setLayout(layout)


       def loadOnDblClick(self, item):
            if item.toolTip()!= "" and item.toolTip().find("|")>-1 :
                 infos = item.toolTip().split("|")
                 if len(infos)== 2:
                      fileName, layerId = infos[0], infos[1]
                      if fileName!="" :
                         self.setWindowOpacity(0.5) 
                         self.parent.loadMetadataFile(fileName, True, layerId)     
                         self.parent.InitDir = dirname(fileName)
                         self.setWindowOpacity(1.0)

       def FixeFilter(self):
           if self.radioXml.isChecked() : self.suffixe = ("xml", "meta.xml")
           elif self.radioQmd.isChecked() : self.suffixe = "qmd"
           elif  self.radioAll.isChecked() : self.suffixe = ("qsp", "qmd", "xml", "meta.xml")
           else :  self.suffixe = "qsp"
           self.updateListLayers()
                
       def updateListLayers(self):
            from qsphere.qsphere_tools import getThemeIcon 

            font = QFont()
            font.setPointSize(8) 
            font.setWeight(10) 
            font.setBold(True)

            qDefaultIconLayer = QIcon(getThemeIcon("modenone.png"))
            self.listQgisLayers.clear()

            root = QgsProject.instance().layerTreeRoot()
            if root != None :
               for layer in root.children():
                  xmlFiles = self.getLayerXMLorQSP(layer.layer(), "", self.suffixe)
                  if xmlFiles != [] :
                       for xmlFile in xmlFiles :
                            qIconLayer = QIcon(getThemeIcon("mode%s.png" % xmlFile[1]))
                            item = QListWidgetItem(qIconLayer, "%s" % (fill(layer.name(), 50)), None, 1)
                            item.setFont(font)
                            item.setBackground(QBrush(QColor(0,255,0,125),Qt.SolidPattern))
                            item.setToolTip("%s|%s" % (xmlFile[0], xmlFile[2]))
                            self.listQgisLayers.addItem(item)
                  else :
                       item = QListWidgetItem(qDefaultIconLayer, "%s" % (fill(layer.layer().name(), 50)), None, 1)
                       item.setFlags(Qt.ItemIsEnabled)
                       item.setBackground(QBrush(QColor(255,0,0,125),Qt.SolidPattern))
                       item.setToolTip("")
                       self.listQgisLayers.addItem(item)

       def getLayerXMLorQSP(self, layer, WithPrefix, WithSuffix):
           from qsphere.qsphere_tools import (correctPath, getExtension)
           from os.path import (basename, dirname, exists, splitext)
           
           xmlFiles = [] 
           if layer != None :
               fileName = layer.source()
               layerid = layer.id()
               fileDir = correctPath(dirname(fileName))
               fileOnlyName = basename(fileName)
               textension = splitext(fileName)
               extension = textension[len(textension)-1].lower()
               if type(WithSuffix)== str:
                  baseName = "%s.%s" % (textension[0], WithSuffix)
                  if exists(baseName): xmlFiles.append((baseName, WithSuffix, layerid))
               elif type(WithSuffix)== tuple:
                  FileNameWithoutExt = splitext(fileOnlyName)[0]
                  for suffix in WithSuffix :
                      basenames = ("%s.%s" % (textension[0], suffix),
                                   "%smd_%s.%s" % (fileDir, FileNameWithoutExt, suffix),
                                   "%s%s.qsp.%s" % (fileDir, FileNameWithoutExt, suffix),
                                   "%smd_%s.qsp.%s" % (fileDir, FileNameWithoutExt, suffix))

                      for elt in basenames :
                          if exists(elt): xmlFiles.append((elt, suffix, layerid))
           return xmlFiles

       def qgisLayersList(self):
         return [item.toolTip() for item in self.listQgisLayers.selectedItems()]


       def resizeEvent(self,ev):
         self.groupLayersInfo.setGeometry(10, 10, self.width()-20, self.height()-50)
         self.imgQgisLayers.setGeometry(5, 1, 40, 40)
         self.lblMetadataFilters.setGeometry(60, 20, 80, 20)
         self.groupBoxObj.setGeometry(140, 5, self.width()-200, 40)
         self.lblQgisLayers.setGeometry(60, 40, self.width()-120, 20)
         self.listQgisLayers.setGeometry(10, 60, self.width()-50, self.height()-90)

         
class dlgAuthCswt(QDialog):
       def  __init__(self, user, pwd, serverCswt, pgConnect):
         super(dlgAuthCswt, self).__init__()
         if not pgConnect : self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Authentification for the CSW-T Server")))
         else : self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Authentification for PostgreSQL Server")))
         self.minWidth, self.minHeight = 400, 160
         self.setMinimumSize(self.minWidth,self.minHeight)
         self.setMaximumSize(self.minWidth*1.5,self.minHeight)
         layout = QVBoxLayout(self)

         self.groupCswtInfo = QGroupBox(self)

         from qsphere.qsphere_tools import getThemeIcon
         icon = getThemeIcon("cswt.png")
         carIcon = QImage(icon)
         self.imgServerCswt = QLabel(self.groupCswtInfo)
         self.imgServerCswt.setObjectName("imgServerCswt")
         self.imgServerCswt.setAlignment(Qt.AlignCenter)
         self.imgServerCswt.setPixmap(QPixmap.fromImage(carIcon).scaled(42,42))        

         self.lblServerCswt = QLabel(self.groupCswtInfo)
         self.lblServerCswt.setObjectName("lblServerCswt")
         self.lblServerCswt.setAlignment(Qt.AlignLeft)
         self.lblServerCswt.setWordWrap(True)
         self.lblServerCswt.setText(serverCswt)

         self.lblUserServerCswt = QLabel(self.groupCswtInfo)
         self.lblUserServerCswt.setText("%s : " % (QApplication.translate("QSphere","User for CSW-T")))
         self.lblUserServerCswt.setObjectName("lblUserServerCswt")
         self.lblUserServerCswt.setAlignment(Qt.AlignRight)    

         self.userCswt = MyWidgetLineEdit(self.groupCswtInfo)
         self.userCswt.initType(6)
         self.userCswt.setObjectName("userCswt")
         self.userCswt.textChanged.connect(self.userCswt.checkRegExp)

         self.userCswt.setText(user)

         self.lblPwdUserServerCswt = QLabel(self.groupCswtInfo)
         self.lblPwdUserServerCswt.setText("%s : " % (QApplication.translate("QSphere","Password for the connexion")))
         self.lblPwdUserServerCswt.setObjectName("lblPwdUserServerCswt")
         self.lblPwdUserServerCswt.setAlignment(Qt.AlignRight)  

         self.pwdUserCswt = MyWidgetLineEdit(self.groupCswtInfo)
         self.pwdUserCswt.initType(9)
         self.pwdUserCswt.setObjectName("pwdUserCswt")
         self.pwdUserCswt.textChanged.connect(self.pwdUserCswt.checkRegExp)

         self.pwdUserCswt.setText(pwd)

         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)

         if (serverCswt == "" or serverCswt == None) and not pgConnect : self.acceptButton.setEnabled(False)
         
         layout.addWidget(self.groupCswtInfo) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)


       def authInfos(self): return (self.userCswt.text(), self.pwdUserCswt.text())


       def resizeEvent(self,ev):
         self.groupCswtInfo.setGeometry(10, 10, self.width()-20, self.height()-50)
         self.imgServerCswt.setGeometry(5, 1, 40, 40)
         self.lblServerCswt.setGeometry(60, 10, self.width()-80, 40)
         self.lblUserServerCswt.setGeometry(10, 45, 180, 25)
         self.userCswt.setGeometry(190, 40, self.width()-220, 25)
         self.lblPwdUserServerCswt.setGeometry(10, 75, 180, 25)
         self.pwdUserCswt.setGeometry(190, 70, self.width()-220 , 25)
         

class dlgCrs(QgsProjectionSelectionDialog): 
       def __init__(self, parent, text):
         super(dlgCrs, self).__init__()               
         self.parent = parent
         crs = QgsCoordinateReferenceSystem(text) 
         self.setCrs(crs) if crs.isValid() else self.setCrs(QgsCoordinateReferenceSystem("EPSG:3857"))
         self.minWidth, self.minHeight = 500, 600
         self.setMinimumSize(self.minWidth, self.minHeight)


class dlgDataFormat(QDialog):
       def  __init__(self, listFormats, itemText):
         super(dlgDataFormat, self).__init__()
         self.setWindowTitle(QApplication.translate("QSphere","GDAL/OGR formats"))
         self.minWidth, self.minHeight = 500, 300
         self.setMinimumSize(self.minWidth,self.minHeight)

         from qsphere.qsphere_tools import windowIcon
         windowIcon(self, "format.png")
         
         layout = QVBoxLayout(self)
         self.gdalOgrFormats = listFormats
         self.listFormats = QListView(self)
         self.listFormats.setGeometry(QRect(10, 10, 200, 400))
         self.modele = QStringListModel()
         self.modele.setStringList(self.gdalOgrFormats)
         self.listFormats.setModel(self.modele)
         self.listFormats.setSelectionMode(QAbstractItemView.SingleSelection)
         self.listFormats.setEditTriggers(QAbstractItemView.NoEditTriggers)
         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Close"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Select Item"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
         
         layout.addWidget(self.listFormats) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)

         if itemText != "" :
            selection = self.listFormats.model() 
            itemIndex = self.gdalOgrFormats.index(itemText) if itemText in self.gdalOgrFormats else -1
            if itemIndex != -1 :
                   index = selection.index(itemIndex,0)
                   self.listFormats.selectionModel().setCurrentIndex(index, QItemSelectionModel.Select)

         self.listFormats.doubleClicked.connect(self.accept)

       def format(self):
         selection = self.listFormats.selectionModel()
         indexElementSelectionne = int(selection.currentIndex().row())
         return "%s" % (self.gdalOgrFormats[indexElementSelectionne])
        

class dlgTopology(QDialog):
       def  __init__(self, dictinfosTopology):
         super(dlgTopology, self).__init__()
         self.setWindowTitle(QApplication.translate("QSphere","Topology informations :"))
         layout = QVBoxLayout(self)
         self.minWidth, self.minHeight = 400, 160
         self.setMinimumSize(self.minWidth, self.minHeight)   
         self.zoneInfos = QTableWidget(0, 2, self)
         self.zoneInfos.setObjectName("zoneInfos")
         self.zoneInfos.setGeometry(10, 10, 300, 130)

         from qsphere.qsphere_tools import windowIcon
         windowIcon(self, "topological.png")

         self._listTopo = ("abstract", "full planar graph", "full surface graph", "full topology 3D", "geometryOnly", "planar graph", "surface graph", "topology1D", "topology3D", "unknow")
         self._listTypeVect = ("complex", "composite", "curve", "point", "solid", "surface","unknow")
         
         try : mydict = literal_eval("%s" % (dictinfosTopology))
         except : mydict = None

         from qsphere.qsphere_tools import dicoHasKey

         indexTopology, indexTypeVector = 0, 0
         if mydict!= None and type(mydict)== dict :
            indexTopology = self._listTopo.index(mydict['TopologyLevelCode']) if dicoHasKey(mydict, 'TopologyLevelCode') else 0
            indexTypeVector = self._listTypeVect.index(mydict['GeometricObjectTypeCode']) if dicoHasKey(mydict, 'GeometricObjectTypeCode') else 0    

         dims = (160, 200)
         for i in range(len(dims)): self.zoneInfos.setColumnWidth(i, dims[i])         
         self.zoneInfos.verticalHeader().setVisible(False)
         self.zoneInfos.horizontalHeader().setVisible(False)

         self.zoneInfos.insertRow(0)

         widget = QLineEdit()
         widget.setMinimumSize(160, 30)
         widget.setMaximumSize(160, 30)
         widget.setText("TopologyLevelCode")
         widget.setEnabled(False)
         self.zoneInfos.setCellWidget(0, 0, widget)

         widget = QComboBox()
         widget.setMinimumSize(200, 30)
         widget.setMaximumSize(200, 30)
         widget.addItems(self._listTopo)
         widget.setCurrentIndex(indexTopology)
         self.zoneInfos.setCellWidget(0, 1, widget)

         self.zoneInfos.insertRow(1)

         widget = QLineEdit()
         widget.setMinimumSize(160, 30)
         widget.setMaximumSize(160, 30)
         widget.setText("GeometricObjectTypeCode")
         widget.setEnabled(False)
         self.zoneInfos.setCellWidget(1, 0, widget)

         widget = QComboBox()
         widget.setMinimumSize(200, 30)
         widget.setMaximumSize(200, 30)
         widget.addItems(self._listTypeVect)
         widget.setCurrentIndex(indexTypeVector)
         self.zoneInfos.setCellWidget(1, 1, widget)

         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
  
         layout.addWidget(self.zoneInfos) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)


       def infosTopology(self):
         return "{'TopologyLevelCode':'%s', 'GeometricObjectTypeCode':'%s'}" % (self.zoneInfos.cellWidget(0, 1).currentText(), self.zoneInfos.cellWidget(1, 1).currentText())

class myComboBox(QComboBox):
      def __init__(self, *args): super(myComboBox, self).__init__( *args)

      def focusInEvent(self, event):
          self.setStyleSheet("QComboBox {color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)}"
                             "QComboBox QAbstractItemView {border: 2px solid darkgray; selection-background-color: darkgray;}"
                             )
          if self.parent().objectName()!= "formTableWidget" : getCurrentItem(self, False)
                 
      def focusOutEvent(self, event):
             self.setStyleSheet("""QComboBox {color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)}"""
                                """QComboBox QAbstractItemView {border: 2px solid darkgray; selection-background-color: darkgray;}"""
                               )



class myAction(QAction):
      def __init__(self, name, text, icon, parent): 
         super(myAction, self).__init__(parent)
         self.setIcon(QIcon(icon))
         self.setText(text)
         self.setObjectName(name)


class mySlider(QSlider):
      def __init__(self, *args): super(mySlider, self).__init__(*args)

      def initSlider(self, sizeWidth, sizeHeight, coordX, coordY, name, toolTip, isGeom, isResizable, orientation, valueMin, valueMax, value):
            self.setMinimumSize(sizeWidth, sizeHeight)
            if not isResizable : self.setMaximumSize(sizeWidth, sizeHeight)
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            self.setObjectName(name)
            self.setOrientation(orientation)
            self.setStyleSheet( "QSlider { color: black; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D); border: 1px solid white; border-radius: 3px; padding: 1px 18px 1px 3px; }" )
            self.setToolTip("%s" % (toolTip))
            self.setMinimum(valueMin)
            self.setMaximum(valueMax)
            self.setSingleStep(1)
            self.setValue(value)


class myUserComboBox(QComboBox): 
      def __init__(self, *args):
          super(myUserComboBox, self).__init__(*args)
          self.setStyleSheet("background-color:red;")
          self.setAcceptDrops(True)
          self.view().setDragDropMode(QAbstractItemView.DragDrop)
          
      def checkRegExp(self):          
          self.setStyleSheet("background-color:red;")
          if self.currentText()!="" : self.setStyleSheet("background-color:#AEEE00;")    

class myWebComboBox(filteredComboBox): 
      def __init__(self, *args):
          filteredComboBox.__init__(self, *args) 
          self.regex = QRegExp(r"^(http|https|ftp)\://[a-zA-Z0-9\-\.]+.[\w]{2,4}(:[a-zA-Z0-9]*)?/?([a-zA-Z0-9\-\._\?\,\'/\\\+&amp;%\$#\=~])*$", Qt.CaseSensitive)
          self.setStyleSheet("background-color:red;")
          self.setAcceptDrops(True)
          self.view().setDragDropMode(QAbstractItemView.DragDrop)

      def dropEvent(self, event):
          if event.mimeData().hasFormat('text/plain'):

             newItem = event.mimeData().text()     

             if self.index(u'%s' % (newItem)) == -1 : event.accept()
             else : event.ignore()
          
      def keyPressEvent(self, event):
          if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter :
             if self.parent().objectName() == "dlgNavigator" :    
                dlg = self.parent()
                if dlg.eUrl.findText(dlg.eUrl.currentText())==-1 : dlg.eUrl.addItem(dlg.eUrl.currentText())
                dlg.getUrl()
                return
          filteredComboBox.keyPressEvent(self, event) 


      def checkRegExp(self):          
          self.setStyleSheet("background-color:red;")
          if self.currentText().find(" ")!= -1 : self.setEditText(self.currentText().replace(" ","%20"))

          tagLocalFile = ["file:", "filexml:", "fileHTML:",  "about:"]
          cond = False
          for elt in tagLocalFile :
              if self.currentText().startswith(elt): cond = True ; break ;

          if cond : self.setStyleSheet("background-color:#AEEE00;")
          else :
               if self.regex.exactMatch(self.currentText()) : self.setStyleSheet("background-color:#AEEE00;")
               elif self.currentText()=="#blank" : self.setStyleSheet("background-color:#AEEE00;")
  

class MySimpleWidgetLineEditST(QLineEdit):
      def __init__(self, *args):
          super(MySimpleWidgetLineEditST, self).__init__(*args)
          self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)")
          self.setAutoFillBackground(True)
          
      def focusInEvent(self, event): 
              try : getCurrentItem(self, True)
              except : pass

      def focusOutEvent(self, event): self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)")

      def keyPressEvent(self, event): QLineEdit.keyPressEvent(self,event)

class MySimpleWidgetLineEdit(QLineEdit):
      def __init__(self, *args):
          super(MySimpleWidgetLineEdit, self).__init__(*args)
          self.setDragEnabled(True)

      def dropEvent(self, event):
          if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'):

                  if event.mimeData().hasFormat('text/plain'):
                         event.accept()
                         try : self.setText(event.mimeData().text())
                         except : pass

                  elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
                     index = self.indexAt(event.pos())
                     encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
                     stream = QDataStream(encoded, QIODevice.ReadOnly)
                     while not stream.atEnd():
                          row = stream.readInt()
                          column = stream.readInt()
                          map = stream.readQVariantMap()
                          if len(map.values())==1:
                                text = "%s" % (map.values()[0])
                                if  index.row()>=0 and index.column()>=0 :
                                       try : self.item(index.row(), index.column()).setText(text)
                                       except :
                                             try : self.cellWidget(index.row(), index.column()).setText(text)
                                             except :
                                                 try : self.cellWidget(index.row(), index.column()).setValue(float(text))
                                                 except :
                                                     try : self.cellWidget(index.row(), index.column()).setValue(int(text))
                                                     except :
                                                          try :
                                                              indexWidget = self.cellWidget(index.row(), index.column()).findText(text, Qt.MatchFixedString)
                                                              if indexWidget == -1 : indexWidget = 0
                                                              self.cellWidget(index.row(), index.column()).setCurrentIndex(indexWidget)
                                                          except : pass
                     event.accept()
          else :
                 title = QApplication.translate("QSphere","Information")
                 msg = QApplication.translate("QSphere","Bad format !")
                 QMessageBox.information(None, title, msg) 
                 event.ignore()
          
      def focusInEvent(self, event):
              try : getCurrentItem(self, False)
              except : pass


class myCheckBox(QCheckBox):
      def __init__(self, *args): super(myCheckBox, self).__init__(*args)
      def focusInEvent(self, event):
              try : getCurrentItem(self, False)
              except : pass

     
class MyWidgetLineEdit(QLineEdit):
      def __init__(self, *args):
          super(MyWidgetLineEdit, self).__init__(*args)
          self.setDragEnabled(True)
          self.setAcceptDrops(True)
 
      def initType(self, indexType):
          if indexType == 0 : self.regex = QRegExp(r"(^[\w.-]+@[\w.-]+\.[a-zA-Z]{2,6}$)", Qt.CaseSensitive)
          elif indexType == 2 : self.regex = QRegExp(r"(^(([0-8][0-9])|(9[0-5]))[0-9]{3}$)", Qt.CaseSensitive)
          elif indexType == 3 : self.regex = QRegExp(r"(^EPSG:+[0-9]{4,6}$)", Qt.CaseSensitive)
          elif indexType in (4, 40, 41) : self.regex = QRegExp(r"^(http|https|ftp)\://[a-zA-Z0-9\-\.]+.[\w]{2,4}(:[a-zA-Z0-9]*)?/?([a-zA-Z0-9\-\._\?\,\'/\\\+&amp=~])*$", Qt.CaseSensitive)
          elif indexType == 9 : self.setEchoMode(QLineEdit.Password)
          self.setStyleSheet("background-color:red;") if indexType != 3 else self.setStyleSheet("background-color:#AEEE00;")
          if indexType in (1, 5) : self.setAlignment(Qt.AlignCenter) 
          self.indexType = indexType

      def dragMoveEvent(self, event): pass
       
      def dragEnterEvent(self, event):
           if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'): event.accept()
           else : event.ignore()

      def dropEvent(self, event):
          if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'):
                  objSource = event.source()
                  first, zeroMode = False, False
                  
                  if event.mimeData().hasFormat('text/plain'):
                         event.accept()
                         try : self.setText(event.mimeData().text())
                         except : pass
                  
                  elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
                     try : index = self.indexAt(event.pos())
                     except : index = None 
                     encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
                     stream = QDataStream(encoded, QIODevice.ReadOnly)

                     while not stream.atEnd():
                          row = stream.readInt()
                          column = stream.readInt()
                          map = stream.readQVariantMap()
                            
                          if len(map.values())==1: text = "%s" % (map.values()[0])
                          elif objSource.metaObject().className() =="QTableView" :
                               if not first :
                                  text = objSource.model().item(row, column).text()
                                  first, zeroMode = True, True
                          
                          if index == None :
                             self.setText(text)
                          else :       
                                if  index.row()>=0 and index.column()>=0 :
                                       try : self.item(index.row(), index.column()).setText(text)
                                       except :
                                             try : self.cellWidget(index.row(), index.column()).setText(text)
                                             except :
                                                 try : self.cellWidget(index.row(), index.column()).setValue(float(text))
                                                 except :
                                                     try : self.cellWidget(index.row(), index.column()).setValue(int(text))
                                                     except :
                                                          try :
                                                              indexWidget = self.cellWidget(index.row(), index.column()).findText(text, Qt.MatchFixedString)
                                                              if indexWidget == -1 : indexWidget = 0
                                                              self.cellWidget(index.row(), index.column()).setCurrentIndex(indexWidget)
                                                          except : pass
                          if zeroMode : break   
                     event.accept()
          else :
                 title = QApplication.translate("QSphere","Information")
                 msg = QApplication.translate("QSphere","Bad format !")
                 QMessageBox.information(None, title, msg) 
                 event.ignore()

      def focusInEvent(self, event):
              try : getCurrentItem(self, False)
              except : pass
              
      def keyPressEvent(self, event):
          if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter :
             if self.parent().objectName() == "dlgNavigator" :    
                dlg = self.parent()
                dlg.getUrl()
                return
          QLineEdit.keyPressEvent(self,event)

      def checkRegExp(self):
          from qsphere.qsphere_tools import verifdatechaine
          
          self.setStyleSheet("background-color:red;")
          if self.indexType in (0, 3, 4, 40, 41):
             cond = False    
             if self.indexType in (0, 4, 40, 41):
                cond = True
                if self.indexType in (4, 40, 41) and self.text().find(" ")!= -1 : self.setText(self.text().replace(" ","%20"))
                if self.indexType == 40 : cond = (self.text().lower().endswith("csw-publication") or self.text().lower().endswith("csw-all") or self.text().lower().endswith("csw")) 
             else :
                 text = self.text().replace("EPSG:","")
                 if text != "" :
                        try :  
                                id = long(text)
                                cond = QgsCoordinateReferenceSystem().createFromSrid(id)
                        except : cond = False        
             if self.regex.exactMatch(self.text()) and cond : self.setStyleSheet("background-color:#AEEE00;")
        
          elif self.indexType in (6,9) :
             if self.text()!= "" : self.setStyleSheet("background-color:#AEEE00;")
          elif self.indexType == 3:               
               text = self.text()
               if text != "" :
                  cond = QgsCoordinateReferenceSystem().createFromUserInput(text) 
               if cond : self.setStyleSheet("background-color:#AEEE00;")              
          elif self.indexType == 5 :
             dateValue = self.text()
             dateTest = verifdatechaine(dateValue, "-")
             if dateTest : self.setStyleSheet("background-color:#AEEE00;")
          elif self.indexType == 1 :              
             dateValueLower = self.text().split(" ")[0]
             dateValueUpper = self.text().split(" ")[1]
             dateTestLower = verifdatechaine(dateValueLower, "-")
             dateTestUpper = verifdatechaine(dateValueUpper, "-")
             if dateTestLower and dateTestUpper:
                L = dateValueLower.split("-")
                a, m, j = int(L[0]), int(L[1]), int(L[2])
                d0 = date(a, m, j)
                L = dateValueUpper.split("-")
                a, m, j = int(L[0]), int(L[1]), int(L[2])
                d1 = date(a, m, j)
                delta = d1 - d0
                if (delta.days)>=0 : self.setStyleSheet("background-color:#AEEE00;")

                
          if self.indexType == 2 :
              if self.regex.exactMatch(self.text()):
                 if self.regex.pattern() == "(^(([0-8][0-9])|(9[0-5]))[0-9]{3}$)" :
                    if self.text()!="00000" : self.setStyleSheet("background-color:#AEEE00;") 
                 else : self.setStyleSheet("background-color:#AEEE00;")

class myWidgetField(QLineEdit):
      def __init__(self, *args): super(myWidgetField, self).__init__(*args)
 
      def initType(self, indexType):
          self.inactiveTextInPlace = True
          self.regex = QRegExp(r"^[À-Ÿà-ÿA-Za-z0-9\\-\\_]+$", Qt.CaseSensitive)
          self.indexType = indexType
          self.setText("")      
          self.inactiveTextStyleSheet()

      def focusInEvent(self, event):
          if (self.objectName().find("tableWidget")!=-1):
                 row = int(self.objectName().split("_")[2])
                 self.parent().parent().selectRow(row)
          if self.inactiveTextInPlace: self.setStyleSheet("")

      def focusOutEvent(self, event):
         if self.inactiveTextInPlace: self.inactiveTextStyleSheet()
         
      def inactiveTextStyleSheet(self): self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)") if self.text()!= "" else self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(255, 0, 0, 125)")
        
      def keyPressEvent(self, event):
          QLineEdit.keyPressEvent(self,event)
          self.VerifExpReg()

      def VerifExpReg(self):
          if self.text()== "" : self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(255, 0, 0, 125)"); return ;
          self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)")
          if self.text().find(" ")!= -1 : self.setText(self.text().replace(" ","_"))
          if self.regex.indexIn(self.text(), 0)==-1 :
             text = self.text() 
             self.setText(text.replace(text[-1], ""))
          else : 
              self.setText(self.RemplaceAccent(self.text()))

      def RemplaceAccent(self, text):
          from qsphere.qsphere_tools import dicoHasKey    
          listcarac = {"¥" : "Y", "µ" : "u", "À" : "A", "Á" : "A", "Â" : "A", "Ã" : "A", "Ä" : "A", "Å" : "A", "Æ" : "A", "Ç" : "C", "È" : "E", "É" : "E", "Ê" : "E", "Ë" : "E", "Ì" : "I", \
                       "Í" : "I", "Î" : "I", "Ï" : "I", "Ð" : "D", "Ñ" : "N", "Ò" : "O", "Ó" : "O", "Ô" : "O", "Õ" : "O", "Ö" : "O", "Ø" : "O", "Ù" : "U", "Ú" : "U", "Û" : "U", "Ü" : "U", \
                       "Ý" : "Y", "ß" : "s", "à" : "a", "á" : "a", "â" : "a", "ã" : "a", "ä" : "a", "å" : "a", "æ" : "a", "ç" : "c", "è" : "e", "é" : "e", "ê" : "e", "ë" : "e", "ì" : "i", \
                       "í" : "i", "î" : "i", "ï" : "i", "ð" : "o", "ñ" : "n", "ò" : "o", "ó" : "o", "ô" : "o", "õ" : "o", "ö" : "o", "ø" : "o", "ù" : "u", "ú" : "u", "û" : "u", "ü" : "u",  \
                       "ý" : "y", "ÿ" : "y"}
          if dicoHasKey(listcarac, text[-1]) : text = text.replace(text[-1], listcarac[text[-1]])
          return text 

class myLabel(QLabel):
     def __init__(self, *args):
             super(myLabel, self).__init__(*args)
             self.setStyleSheet("QLabel { border-top : 1px solid yellow;  border-bottom : 1px solid yellow;  border-left: 1px solid yellow;  border-right : 1px solid yellow }")
             self.setWindowOpacity(0.5)
             self.setAutoFillBackground(False)
             self.setUpdatesEnabled(True)

class myButton(QPushButton):
     def __init__(self, *args): super(myButton, self).__init__(*args)
            
     def initButton(self, sizeWidth, sizeHeight, coordX, coordY, name, text, toolTip, isEnable, isGeom):
            self.setMinimumSize(sizeWidth, sizeHeight)
            self.setMaximumSize(sizeWidth, sizeHeight)
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            self.setObjectName(name)
            self.setToolTip("%s" % (toolTip))
            self.setText(text)
            self.setEnabled(isEnable)

class myPushButton(QPushButton):
     def __init__(self, *args): super(myPushButton, self).__init__(*args)
            
     def initPushButton(self, sizeWidth, sizeHeight, coordX, coordY, name, text, toolTip, isGeom, icon, iconWidth, iconHeight, isStyleSheeted):
            self.setMinimumSize(sizeWidth, sizeHeight)
            self.setMaximumSize(sizeWidth, sizeHeight)
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            if icon != "" :
               self.setIcon(QIcon(icon))
               self.setIconSize(QSize(iconWidth, iconHeight))
            self.setToolTip(toolTip)
            if isStyleSheeted : self.setStyleSheet(""" QPushButton {border: none;}""")
            self.setObjectName(name)
            if text != "" : self.setText(text)

class myPushButtonFunny(QPushButton):
     def __init__(self, *args): super(myPushButtonFunny, self).__init__(*args)
            
     def initPushButton(self, sizeWidth, sizeHeight, coordX, coordY, name, text, toolTip, isGeom, icon, iconWidth, iconHeight, isStyleSheeted):
            self.setMinimumSize(sizeWidth, sizeHeight)
            self.setMaximumSize(sizeWidth, sizeHeight)
            self.iconWidth = iconWidth
            self.iconHeight = iconHeight
            self.selfFocused = False
            self.subMenuVisble = False
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            if icon != "" :
               self.setIcon(QIcon(icon))
               self.setIconSize(QSize(self.iconWidth , self.iconHeight))
            self.setToolTip(toolTip)
            if isStyleSheeted : self.setStyleSheet(" QPushButton {border-width: 0px; border-radius: 10px;  border-color: beige;}")
            self.setObjectName(name)
            if text != "" : self.setText(text)

     def enterEvent(self, event):
          if not self.selfFocused :
             self.setIconSize(QSize(self.iconWidth *0.8, self.iconHeight *0.8))
             self.selfFocused = True
          
     def leaveEvent(self, event):
        if self.subMenuVisble == False : self.restoreButton()
     
     def restoreButton(self):
           self.selfFocused = False    
           self.setIconSize(QSize(self.iconWidth, self.iconHeight)) 

class mySpinBox(QDoubleSpinBox):
     def __init__(self, *args): super(mySpinBox, self).__init__(*args)
     
     def initDoubleSpinBox(self, parent, gui):
            self.parent = parent
            self.gui = gui 
            self.setFocusPolicy(Qt.StrongFocus)
            self.installEventFilter(self)

     def eventFilter(self, object, event):
          if not event : return False
          if event == None : return False
          if event.type() == QEvent.FocusIn:
             extentIndex = int(object.objectName().split("_")[2])
             try :
                 self.parent.setToolTip("%s" % (extentIndex))
                 if self.parent.metaObject().className() in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                    self.parent.selectRow(extentIndex)
                    if self.parent.parent().objectName() == "ViewerTableWidget" : self.parent.parent().getCurrentItem()
                 return True
             except : return False
          else : return False


class myTextEdit(QTextEdit):           
     def __init__(self, *args): super(myTextEdit, self).__init__(*args)
     
     def initTextEdit(self, sizeWidth, sizeHeight, coordX, coordY, name, isGeom, isUndoRedo, isReadOnly, isRichText, isResizable):
            self.setMinimumSize(sizeWidth, sizeHeight)
            if not isResizable : self.setMaximumSize(sizeWidth, sizeHeight)
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            self.setUndoRedoEnabled(isUndoRedo)
            self.setReadOnly(isReadOnly)
            self.setAcceptRichText(isRichText)
            self.setObjectName(name)
            self.setAcceptDrops(True)

     def dragEnterEvent(self, event): event.accept()
     def dragLeaveEvent(self, event): event.accept()
     def dragMoveEvent(self, event): event.accept()
     def dropEvent(self, event):
          if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'):
                  cursor = self.textCursor()
                  if event.mimeData().hasFormat('text/plain'): cursor.insertText(event.mimeData().text())
                  elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
                     encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
                     stream = QDataStream(encoded, QIODevice.ReadOnly)
                     while not stream.atEnd():
                          row = stream.readInt()
                          column = stream.readInt()
                          map = stream.readQVariantMap()
                          if len(map.values())==1:
                                text = "%s" % (map.values()[0])
                                try : cursor.insertText(text) 
                                except : pass
                     event.accept()
          else :
                 title = QApplication.translate("QSphere","Information")
                 msg = QApplication.translate("QSphere","Bad format !")
                 QMessageBox.information(None, title, msg) 
                 event.ignore()         
  

class myTableWidget(QTableWidget):           
       def __init__(self, *args):
              super(myTableWidget, self).__init__(*args)
              self.setDragEnabled(True)
              self.setAcceptDrops(True)
              self.setDragDropOverwriteMode(False)
              self.setDropIndicatorShown(True)
              self.setSelectionMode(QAbstractItemView.SingleSelection)
              self.setSelectionBehavior(QAbstractItemView.SelectRows) 
              self.setDragDropMode(QAbstractItemView.DragDrop)
              self.originalWidth, self.dims = 0, 0
         
       def fixeMenu(self, menu): self.menu = menu

       def dropEvent(self, event):
           if event.mimeData().hasFormat('text/plain'):
              index = self.indexAt(event.pos())
              if  index.row()>=0 and index.column()>=0 :
                  try : self.item(index.row(), index.column()).setText(text)
                  except :                     
                         try : self.cellWidget(index.row(), index.column()).setText(event.mimeData().text())
                         except :       
                                   try : self.cellWidget(index.row(), index.column()).setValue(float(event.mimeData().text()))
                                   except :
                                       try : self.cellWidget(index.row(), index.column()).setValue(int(event.mimeData().text()))
                                       except :
                                            try :
                                                indexWidget = self.cellWidget(index.row(), index.column()).findText(event.mimeData().text(), Qt.MatchFixedString)
                                                if indexWidget == -1 : indexWidget = 0
                                                self.cellWidget(index.row(), index.column()).setCurrentIndex(indexWidget)
                                            except :
                                                 try:
                                                      itemComment = QTableWidgetItem()
                                                      itemComment.setText("%s" % (event.mimeData().text()))
                                                      self.setItem(index.row(), index.column(), itemComment)
                                                      self.item( index.row(), index.column() ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled|Qt.ItemIsEditable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled)
                                                 except : pass

              else :  self.refDialog.addDragDropLine(self, event.mimeData().text())
                         
              event.accept()
          
           elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
              objSource = event.source()
              first, zeroMode = False, False
              
              index = self.indexAt(event.pos())
              encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
              stream = QDataStream(encoded, QIODevice.ReadOnly)
              
              while not stream.atEnd():
                   row = stream.readInt()
                   column = stream.readInt()
                   map = stream.readQVariantMap()

                   if len(map.values())==1: text = "%s" % (map.values()[0])
                   elif objSource.metaObject().className() =="QTableView" :
                        if not first :
                           text = objSource.model().item(row, column).text()
                           first, zeroMode = True, True

                   if  index.row()>=0 and index.column()>=0 :
                         try : self.item(index.row(), index.column()).setText(text)
                         except :
                               try : self.cellWidget(index.row(), index.column()).setText(text)
                               except :
                                   try : self.cellWidget(index.row(), index.column()).setValue(float(text))
                                   except :
                                       try : self.cellWidget(index.row(), index.column()).setValue(int(text))
                                       except :
                                            try :
                                                indexWidget = self.cellWidget(index.row(), index.column()).findText(text, Qt.MatchFixedString)
                                                if indexWidget == -1 : indexWidget = 0
                                                self.cellWidget(index.row(), index.column()).setCurrentIndex(indexWidget)
                                            except : pass

                   else : self.refDialog.addDragDropLine(self, text)

                   if zeroMode : break
                   
              event.accept()
               
           else : event.ignore()
               
       def dragMoveEvent(self, event): pass
       
       def dragEnterEvent(self, event):
           if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'): event.accept()
           else : event.ignore()
     
       def contextMenuEvent(self, event): action = self.menu.exec_(self.mapToGlobal(event.pos()))


class myProgressBar(QProgressBar):
     def __init__(self, *args):  super(myProgressBar, self).__init__(*args)

     def initTextEdit(self, sizeWidth, sizeHeight, coordX, coordY, name, isGeom, defaultValue, text, align, isStyleSheeted):
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            self.setProperty("value", defaultValue)
            self.setAlignment(align)
            self.setTextVisible(text)
            self.setObjectName(name)
            if isStyleSheeted == 0 :
                 self.setStyleSheet(
                      "QProgressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
                      "QProgressBar::chunk {background-color: #6C96C6; width: 10px; margin: 0.5px;}"
                  )

def getCurrentItem(self, isStyleSheeted):
        try : 
                infos = self.objectName().split("_")
                tableWidget = self.parent().parent()
                row = int(infos[2])
                col = tableWidget.currentColumn() if len(infos) == 4 else 0 
                tableWidget.setCurrentCell(row, col)
                tableWidget.selectRow(row)
                tableWidget.parent().getCurrentItem()
                if isStyleSheeted : self.setStyleSheet("")
        except : pass

