# -*- coding:utf-8 -*-
from qgis.core import Qgis
from qgis.gui import QgsMessageBar

from qgis.PyQt.QtCore import (QFileInfo, Qt)
from qgis.PyQt.QtGui import (QIcon, QKeySequence)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QComboBox, QFileDialog, QGroupBox,
                                 QLabel, QMenu, QProgressBar, QPushButton, QSizePolicy, QTableWidget)

from qsphere.importexport import exportDictionnaryToXml
from qsphere.qsphere_objmaker import (movieCtrl, myPushButton, myPushButtonFunny, myTableWidget)
from qsphere.qsphere_tools import (addLineWidget, changeSizeCols, dicoHasKey, fileNameWithExtension, fixeLabelsFileDialog,
                                   getAllItems, getExtension, getHelp, getOptions, getThemeIcon, getWidget,
                                   initSizeCols, isFloatNumber, isIntNumber, patternZipPostalCode, sendMsg, textForMenu )
from qsphere.ui_catalogue import (formWidgetDialog, contextMnuMDDActions, goToLocalisator, testUrl)
from qsphere.xmlisoparser import xmlIsoParser

from configparser import ConfigParser
from os.path import (basename, dirname)



class uiDlgContact(object):
   
    def setupUi(self):
        self.maxWidth, self.maxHeight = 960, 580
        self.setMinimumSize(self.maxWidth, self.maxHeight)
        self.setObjectName("dlgContacts")
        self.resize(self.maxWidth, self.maxHeight)
        self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
            
        self.dims = self.parent.tablesWidget["tableroles"][0]
        self.headers = self.parent.tablesWidget["tableroles"][1]
        self.originalWidth = 800

        getOptions(self)
        
        self.initDirContacts = self.InitDir
        
        self.rootWindowTitle = QApplication.translate("QSphere", "Contacts list file")
        self.setWindowTitle("%s : %s" % (self.rootWindowTitle, ""))
        self.groupStyleSheet = "QGroupBox {background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E0E0E0, stop: 1 #FFFFFF); border: 2px solid gray; border-radius: 5px; margin-top: 1ex; })"

        self.myPathPrint, self.myPathReLoadFile, self.myPathLoadFile, self.myPathAddFile = "print.png", "reload.png", "open.png", "add.png"
        self.myPathViewFile, self.myPathNewList, self.myPathSave, self.myPathSaveAs = "navigatorweb.png", "new.png", "save.png", "saveas.png"
        self.paramsLineWidget = ((2, 0, 0),(-2, 0, ""), (-2, 0, ""), (2, 5, self.indexCountry), (0, 2, ""), (-2, 0, ""), (0, 0, ""), (-1, 0, ""), (-1, 0, ""), (0, 4, ""))

        self.filtreQSP = "QSphere %s (*.qsp)" % (QApplication.translate("QSphere","project"))

        self.groupBoxDirectoryActions = QGroupBox(self)
        self.groupBoxDirectoryActions.setObjectName("groupBoxDirectoryActions")
        self.groupBoxDirectoryActions.setStyleSheet(self.groupStyleSheet)

        self.lblDirectory = QLabel(self.groupBoxDirectoryActions)
        self.lblDirectory.setGeometry(10, 15,  120, 25)
        self.lblDirectory.setText("%s : " % (QApplication.translate("QSphere", "List of Contacts")))
        self.lblDirectory.setAlignment(Qt.AlignRight)

        self.directory = QComboBox(self.groupBoxDirectoryActions)
        self.directory.setGeometry(125, 15, 250, 25)
        self.directory.setEditable(False)
        self.directory.setEnabled(True)
        self.directory.setObjectName("directory")
        self.directory.addItem("")


        toolTip =  QApplication.translate("QSphere", "Refresh list of contacts with current file")
        self.reloadDirectory = myPushButton(self.groupBoxDirectoryActions)
        self.reloadDirectory.initPushButton(24, 24, 10, 5, "reloadDirectory", "", toolTip, True, getThemeIcon(self.myPathReLoadFile), 24, 24, True)
        self.reloadDirectory.setShortcut(QKeySequence("F5"))

        toolTip = QApplication.translate("QSphere", "View current contacts list file ...")
        self.viewDirectory = myPushButton(self.groupBoxDirectoryActions)
        self.viewDirectory.initPushButton(24, 24, 50, 5, "viewDirectory", "", toolTip, True, getThemeIcon(self.myPathViewFile), 24, 24, True)
        self.viewDirectory.setShortcut(QKeySequence("Ctrl+W"))

        toolTip = QApplication.translate("QSphere", "Choose contacts list file ...")
        self.loadDirectory = myPushButton(self.groupBoxDirectoryActions)
        self.loadDirectory.initPushButton(24, 24, 90, 5, "loadDirectory", "", toolTip, True, getThemeIcon(self.myPathLoadFile), 24, 24, True)
        self.loadDirectory.setShortcut(QKeySequence("Ctrl+O"))

        toolTip =  QApplication.translate("QSphere", "New list file in current list ...")
        self.newDirectory = myPushButton(self.groupBoxDirectoryActions)
        self.newDirectory.initPushButton(24, 24, 130, 45, "newDirectory", "", toolTip, True, getThemeIcon(self.myPathNewList), 24, 24, True)
        self.newDirectory.setShortcut(QKeySequence("Ctrl+N"))

        toolTip = QApplication.translate("QSphere", "Add contacts list file in current list ...")
        self.addDirectory = myPushButton(self.groupBoxDirectoryActions)
        self.addDirectory.initPushButton(24, 24, 165, 45, "addDirectory", "", toolTip, True, getThemeIcon(self.myPathAddFile), 24, 24, True)
        self.addDirectory.setShortcut(QKeySequence("Ctrl+A"))

        toolTip = QApplication.translate("QSphere", "Save the current list")      
        self.saveButton = myPushButton(self.groupBoxDirectoryActions)
        self.saveButton.initPushButton(24, 24, 210, 45, "saveButton", "", toolTip, True, getThemeIcon(self.myPathSave), 24, 24, True)
        self.saveButton.setShortcut(QKeySequence("Ctrl+S"))

        icon = getThemeIcon("qspherehelp.png")
        self.HelpButton = myPushButton(self) 
        self.HelpButton.initPushButton(48, 48, -50, -50, "HelpButton", "", "", True, icon, 48, 48, True)
        self.HelpButton.setShortcut(QKeySequence("F1"))

        self.ActionsSaveButton = myPushButton(self.groupBoxDirectoryActions) 
        self.ActionsSaveButton.initPushButton(56, 24, 240, 45, "ActionsSaveButton", "", QApplication.translate("QSphere","Other saving actions ..."), True, getThemeIcon("saveactions.png"), 56, 24, True)

        self.progressBar = QProgressBar(self.groupBoxDirectoryActions)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setAlignment(Qt.AlignCenter)
        self.progressBar.setTextVisible(True)
        self.progressBar.setObjectName("progressBar")
        self.progressBar.setStyleSheet(
             "QProgressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
             "QProgressBar::chunk {background-color: #6C96C6; width: 20px;}"
        )
        self.progressBar.setVisible(False)
        self.progressBar.setValue(0)
        self.progressBar.setGeometry(420, 50, 320, 15)

        self.tableWidget = myTableWidget(0, len(self.dims), self) 
        self.tableWidget.setTabKeyNavigation(False)
        self.tableWidget.setObjectName("tableroles")
        self.tableWidget.setStyleSheet("QTableWidget {selection-background-color: rgb(50, 125, 180); }")

        self.badNetwork = None
        self.showWarning = False
        self.listOfConformities = self.qspherePlugin.listOfConformities
        self.listOfRules, self.dictListOfRules = self.qspherePlugin.listOfRules, self.qspherePlugin.dictListOfRules
        self.listOfThesaurus , self.dateListOfThesaurus = self.qspherePlugin.listOfThesaurus , self.qspherePlugin.dateListOfThesaurus
        self.listTypeDates = self.qspherePlugin.listTypeDates
        self.listDegres = self.qspherePlugin.listDegres
        self.tablesWidget = self.qspherePlugin.tablesWidget
        self.paramsWidget = self.qspherePlugin.paramsLineWidget

        icon = getThemeIcon("actions.png") 
        toolTip = QApplication.translate("QSphere","Actions ...")
        self.actionButton = myPushButtonFunny(self) 
        self.actionButton.initPushButton(40, 24, 40, 90, "actionButton", "", toolTip, True, icon, 40, 24, True)
        
        contextMnuMDDActions(self, 0, self.tableWidget, "%s" % (self.tableWidget.objectName()), 6, self.actionButton, "")

        icon = getThemeIcon("wizform.png") 
        toolTip = QApplication.translate("QSphere","Call the form wizard Tables ...")
        self.wizardButton = myPushButtonFunny(self) 
        self.wizardButton.initPushButton(50, 34, 40, 90, "wizardButton", "", "%s (F8)" % toolTip, True, icon, 50, 34, True)

        self.groupBoxActionsMeta = QGroupBox(self)
        self.groupBoxActionsMeta.setStyleSheet(self.groupStyleSheet)

        toolTip = QApplication.translate("QSphere","Move down")
        self.moveDown = myPushButton(self.groupBoxActionsMeta) 
        self.moveDown.initPushButton(24, 24, 10, 10, "moveDown", "", toolTip, True, getThemeIcon("movedown.png"), 24, 24, True)
        self.moveDown.setAutoRepeat(True)
        self.moveDown.setShortcut(QKeySequence.MoveToNextLine)

        self.labelPosition = QLabel(self.groupBoxActionsMeta)
        self.labelPosition.setGeometry(40, 10, 100, 25)
        self.labelPosition.setText("%s :" % (QApplication.translate("QSphere","Current contact")))
        
        self.currentContact = QLabel(self.groupBoxActionsMeta)
        self.currentContact.setGeometry(140, 10, 40, 25)

        toolTip = QApplication.translate("QSphere","Move up")
        self.moveUp = myPushButton(self.groupBoxActionsMeta) 
        self.moveUp.initPushButton(24, 24, 180, 10, "moveUp", "", toolTip, True, getThemeIcon("moveup.png"), 24, 24, True)
        self.moveUp.setAutoRepeat(True)
        self.moveUp.setShortcut(QKeySequence.MoveToPreviousLine)

        icon = getThemeIcon("pastresponsible.png")
        toolTip = QApplication.translate("QSphere","Actions for current contact")
        self.actionsButton = myPushButton(self.groupBoxActionsMeta) 
        self.actionsButton.initPushButton(60, 24, 220, 10, "Actions", "", toolTip, True, icon, 60, 24, True) 
        self.FixeMenu()

        toolTip = QApplication.translate("QSphere","Add an item")
        self.addOneContact = myPushButton(self.groupBoxActionsMeta)
        self.addOneContact.initPushButton(60, 25, 290, 10, "add_6_tableroles", "", toolTip, True, getThemeIcon("addresponsible.png"), 60, 25, True)

        toolTip = QApplication.translate("QSphere","Add all contacts to the sheet")
        self.addAllContacts = myPushButton(self.groupBoxActionsMeta) 
        self.addAllContacts.initPushButton(60, 25, 360, 10, "addAll_6_tableroles", "", toolTip, True, getThemeIcon("sendtoresponsible.png"), 60, 25, True)

        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

        self.movieCtrl = movieCtrl(self, "sablier.gif")

        self.closeButton = QPushButton(self)
        self.closeButton.setObjectName("closeButton")
        self.closeButton.setText("&%s" % QApplication.translate("QSphere", "Close"))

        self.actionsButton.setEnabled(self.notSoloMode)
        self.addOneContact.setEnabled(self.notSoloMode) 
        self.addAllContacts.setEnabled(self.notSoloMode)

        self.directory.currentIndexChanged.connect(self.isEnableButton)
        self.viewDirectory.clicked.connect(self.getWebViewer)
        self.loadDirectory.clicked.connect(self.loadXmlDirectory)
        self.reloadDirectory.clicked.connect(self.loadXmlDirectory)
        self.addDirectory.clicked.connect(self.loadXmlDirectory)
        self.newDirectory.clicked.connect(self.initDirectory)
        self.HelpButton.clicked.connect(self.clickHelp)

        self.wizardButton.clicked.connect(self.openFormWizardMode)

        self.moveUp.clicked.connect(self.moveSelectionUp)
        self.moveDown.clicked.connect(self.moveSelectionDown)
        self.addOneContact.clicked.connect(self.sendToParentWidget)
        self.addAllContacts.clicked.connect(self.sendAllToParentWidget)
        self.saveButton.clicked.connect(self.save)
        self.contextMnuSaveActions()
        self.closeButton.clicked.connect(self.close)
        
        self.initTableWidget()
        self.getCurrentItem()
        self.isEnableButton()

 
    def contextMnuSaveActions(self):
        contextMenu = QMenu()

        menuIcon = getThemeIcon("saveas.png")
        text = QApplication.translate("QSphere","Save as ...")
        self.saveAsButton = QAction(QIcon(menuIcon), text, self)
        self.saveAsButton.setObjectName("saveAsButton")
        self.saveAsButton.setShortcut(QKeySequence("Ctrl+Shift+S"))
        contextMenu.addAction(self.saveAsButton)
        self.saveAsButton.triggered.connect(self.saveAs)

        contextMenu.addSeparator()
        
        menuIcon = getThemeIcon("savecopy.png")
        text = QApplication.translate("QSphere","Save as a copy ...")
        self.saveAsCopy = QAction(QIcon(menuIcon), text, self)
        self.saveAsCopy.setObjectName("saveAsCopy")
        self.saveAsCopy.setShortcut(QKeySequence("Ctrl+Shift+C"))
        contextMenu.addAction(self.saveAsCopy)
        self.saveAsCopy.triggered.connect(self.saveAs)

        self.ActionsSaveButton.setMenu(contextMenu)

    def clickHelp(self): getHelp(self)

    def checkRules(self): return 

    def resizeEvent(self,ev):
        self.barInfo.setGeometry(0, 0, self.width(), 90)
        self.directory.setGeometry(130, 10, self.width()-260, 25)
        self.reloadDirectory.setGeometry(self.width()-120, 10, 24, 24)
        self.viewDirectory.setGeometry(self.width()-90, 10, 24, 24)
        self.loadDirectory.setGeometry(self.width()-60, 10, 24, 24)
        self.tableWidget.setGeometry(10, 90,  self.width()-80, self.height()-140)
        self.groupBoxDirectoryActions.setGeometry(10, 5, self.width()-20, 80)
        self.actionButton.setGeometry(self.width()-70, 90, 50, 40)
        self.wizardButton.setGeometry(self.width()-65, 140, 58, 58)

        self.groupBoxActionsMeta.setGeometry(self.width()-630, self.height()-50, 460, 40)
        self.closeButton.setGeometry(self.width()-165, self.height()-40, 100, 25)

        if self.parent : changeSizeCols(self, self.tableWidget, self.dims, (self.width()-70)/float(self.originalWidth))

    def callInitSizeCols(self): initSizeCols(self, self.tableWidget, self.dims)

    def isEnableButton(self): self.viewDirectory.setEnabled(self.directory.currentText() != "")

    def getCurrentItem(self):
        self.currentContact.setText("%s / %s" % (self.tableWidget.currentRow()+1, self.tableWidget.rowCount()))
        self.groupBoxActionsMeta.setEnabled(False) if self.tableWidget.rowCount()== 0 else self.groupBoxActionsMeta.setEnabled(True)

    def updatePatternZipPostalCode(self): patternZipPostalCode(self, self.sender())
        
    def initTableWidget(self):
        for i in range(len(self.dims)): self.tableWidget.setColumnWidth(i, self.dims[i])
        self.tableWidget.setHorizontalHeaderLabels(self.headers)              
        self.tableWidget.verticalHeader().setVisible(False)
        self.tableWidget.setSelectionMode(QAbstractItemView.SingleSelection)
        self.tableWidget.setSelectionBehavior(QAbstractItemView.SelectRows)

    def addLine(self):
          row = (self.tableWidget.currentRow()+1) if self.sender().objectName() == "addAfterContact" else self.tableWidget.rowCount()
          self.tableWidget.insertRow(row)
          for i in range(len(self.paramsLineWidget)): addLineWidget(self, self.tableWidget, row, i, self.paramsLineWidget[i][0], self.paramsLineWidget[i][1], self.paramsLineWidget[i][2])
          if self.sender().objectName() == "addAfterContact" : self.renameAllWidgets(self.tableWidget)
          self.tableWidget.selectRow(row)
          widget = self.tableWidget.cellWidget(row, 0)
          if not widget.isEnabled() : widget.setEnabled(True)
     
          self.getCurrentItem()
    
    def menuActions(self):
        self.contextMenuActions = QMenu()
        icon = getThemeIcon("coller.png")
        widget = self.parent.findChild(myTableWidget, "tableroles")
        text = QApplication.translate("QSphere", "Organization")
        if widget :
            for i in range(widget.rowCount()):
                action = self.contextMenuActions.addAction(QIcon(icon), "%s %s" % (text, i))
                action.setObjectName("%s %s" % (text, i))
                action.triggered.connect(self.sendToParentWidget)

    def importContactsFromSheet(self):
        iTableWidget = self.parent.findChild(myTableWidget, "tableroles")
        oTableWidget = self.tableWidget
        if iTableWidget and oTableWidget :
            self.progressBar.setVisible(True)
            for iRow in range(iTableWidget.rowCount()):
                self.addLine()
                self.progressBar.setValue(int(100 * iRow/iTableWidget.rowCount()))
                self.getCellsWidgets(iTableWidget, oTableWidget, iRow, oTableWidget.rowCount()-1, True)
            self.progressBar.setValue(0)
            self.progressBar.setVisible(False)

    def sendAllToParentWidget(self):
        iTableWidget = self.tableWidget
        oTableWidget = self.parent.findChild(myTableWidget, "tableroles")
        if iTableWidget and oTableWidget :
            self.parent.progressBar.setVisible(True)
            self.progressBar.setVisible(True)
            for iRow in range(iTableWidget.rowCount()):
                if iRow > oTableWidget.rowCount()-1 :
                    self.parent.addLine()
                    oRow = oTableWidget.rowCount()-1
                else : oRow = iRow    
                self.parent.progressBar.setValue(int(100 * iRow/iTableWidget.rowCount()))
                self.progressBar.setValue(int(100 * iRow/iTableWidget.rowCount()))
                self.getCellsWidgets(iTableWidget, oTableWidget, iRow, oRow, False)
            self.parent.passChangeMode()
            self.parent.progressBar.setValue(0)
            self.progressBar.setValue(0)            
            self.parent.progressBar.setVisible(False)
            self.progressBar.setVisible(False)

    def sendToParentWidget(self):
        iRow = self.tableWidget.currentRow()
        iTableWidget = self.tableWidget 
        oTableWidget = self.parent.findChild(myTableWidget, "tableroles")
        if iTableWidget and oTableWidget :
             if self.sender().objectName() in ("add_6_tableroles", "addAll_6_tableroles") :
                self.parent.addLine()
                oRow = oTableWidget.rowCount()-1
             else :
                try : oRow = int(self.sender().objectName().split(" ")[1])
                except : oRow = 0
                
             self.getCellsWidgets(iTableWidget, oTableWidget, iRow, oRow, False)
             self.parent.passChangeMode()

    def getCellsWidgets(self, iTableWidget, oTableWidget, iRow, oRow, isEditable):
        for j in range(oTableWidget.columnCount()):
            iWidget = iTableWidget.cellWidget(iRow, j)
            oWidget = oTableWidget.cellWidget(oRow, j)
            if oWidget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox"):
               if j == 0 :
                  if oRow > 1 : oWidget.setCurrentIndex(iWidget.currentIndex())
                  else :
                     if isEditable : oWidget.setCurrentIndex(iWidget.currentIndex())
               else : oWidget.setCurrentIndex(iWidget.currentIndex())   
            else : oWidget.setText(iWidget.text())
        self.FixeMenu()             

    def FixeMenu(self):
        if self.notSoloMode :
           self.menuActions()
           self.actionsButton.setMenu(self.contextMenuActions)


    def CheckTable(self, obj):
        for i in range(obj.rowCount()):
            if i in (0, 1) :
                obj.cellWidget(i,0).setEnabled(False)
                if i == 0 : obj.cellWidget(i,0).setCurrentIndex(2)
                else : obj.cellWidget(i,0).setCurrentIndex(6)
            else :
                obj.cellWidget(i,0).setEnabled(True)
                if obj.cellWidget(i,0).currentIndex() in (2,6) : obj.cellWidget(i,0).setCurrentIndex(1)


    def deleteLastLine(self):
        if self.tableWidget.rowCount() > 0 :
           row = self.tableWidget.rowCount()-1
           self.tableWidget.removeRow(row)
           self.getCurrentItem()
           row-=1
           self.tableWidget.selectRow(row)
           try : self.tableWidget.cellWidget(row, 0).setFocus()
           except : pass
           self.tableWidget.selectRow(row)

    def deleteCurrentLine(self):
        if self.tableWidget.currentRow() < 0 : return
        row =  self.tableWidget.currentRow()
        if self.tableWidget.rowCount()== row+1 : self.deleteLastLine()
        else :
            self.tableWidget.removeRow(row)
            self.renameAllWidgets(self.tableWidget)
            self.tableWidget.selectRow(row)
            try : self.tableWidget.cellWidget(row, 0).setFocus()
            except : pass
            self.tableWidget.selectRow(row)
            self.getCurrentItem()
            

    def deleteAllLines(self):
        self.progressBar.setVisible(True)
        self.tableWidget.setUpdatesEnabled(False)
        rows = self.tableWidget.rowCount()
        for i in range(rows, 0, -1):    
            self.progressBar.setValue(int(100 * i/rows))
            self.tableWidget.removeRow(self.tableWidget.rowCount()-1)
        self.progressBar.setValue(0)
        self.progressBar.setVisible(False)
        self.tableWidget.setUpdatesEnabled(True)
        self.getCurrentItem()

    def renameAllWidgets(self, obj):
        widgetNames = {0:"myComboBox", 1:"MySimpleWidgetLineEditST", 2:"MySimpleWidgetLineEditST", 3:"myComboBox", 4: "MyWidgetLineEdit", 5: "MySimpleWidgetLineEditST", 6: "MyWidgetLineEdit", 7:"MySimpleWidgetLineEditST", 8:"MyWidgetLineEdit", 9: "myPushButton"}
        for i in range(obj.rowCount()):
            for j in range(obj.columnCount()):
                widget = obj.cellWidget(i, j)
                fullName = "%s_%s_%s" % (obj.objectName(), widgetNames[j] ,i)
                widget.setObjectName(fullName) 

    def moveSelectionUp(self):
        if self.tableWidget.rowCount()== 0 : return
        if self.tableWidget.currentRow() > 0 : self.tableWidget.selectRow(self.tableWidget.currentRow()-1)

    def moveSelectionDown(self):
        if self.tableWidget.rowCount()== 0 : return
        if self.tableWidget.currentRow() <  (self.tableWidget.rowCount()-1) : self.tableWidget.selectRow(self.tableWidget.currentRow()+1)
        
    def moveLineUp(self):
        row = self.tableWidget.currentRow()
        if row > 0:
            self.tableWidget.insertRow(row-1)
            for i in range(self.tableWidget.columnCount()):
               originalCellWidget = (self.tableWidget.cellWidget(row+1,i))
               newCellWidget = addLineWidget(self, self.tableWidget, row-1,  i, self.paramsLineWidget[i][0], self.paramsLineWidget[i][1], self.paramsLineWidget[i][2])
               if originalCellWidget.metaObject().className() in ("MyWidgetLineEdit", "MySimpleWidgetLineEdit", "MySimpleWidgetLineEditST") : newCellWidget.setText(originalCellWidget.text()) 
               elif originalCellWidget.metaObject().className() == "myComboBox" :
                   if i in (0,3) :
                      newCellWidget.clear()
                      items = getAllItems(originalCellWidget)
                      newCellWidget.addItems(items)
                   newCellWidget.setCurrentIndex(originalCellWidget.currentIndex())
                   newCellWidget.setEnabled(originalCellWidget.isEnabled())
            self.tableWidget.selectRow(row-1)

            self.tableWidget.removeRow(row+1)
            self.renameAllWidgets(self.tableWidget)
            self.getCurrentItem()
               
    def moveLineDown(self):

        row = self.tableWidget.currentRow()
        boundary = self.tableWidget.rowCount()-1 if self.sender().objectName().startswith("moveDown") else self.tableWidget.rowCount()
        if row < 0 and self.sender().objectName().startswith("Clone") : return
        if row < boundary :
            rowTarget = (row+2) if self.sender().objectName().startswith("moveDown") else (row+1)
            
            self.tableWidget.insertRow(rowTarget)
            for i in range(self.tableWidget.columnCount()):
               originalCellWidget = self.tableWidget.cellWidget(row, i)
               newCellWidget = addLineWidget(self, self.tableWidget, rowTarget,  i, self.paramsLineWidget[i][0], self.paramsLineWidget[i][1], self.paramsLineWidget[i][2])

               if originalCellWidget.metaObject().className() in ("MyWidgetLineEdit", "MySimpleWidgetLineEdit", "MySimpleWidgetLineEditST") :
                   newCellWidget.setText(originalCellWidget.text()) 
               elif originalCellWidget.metaObject().className() == "myComboBox" :
                   if i in (0,3) :
                      newCellWidget.clear()
                      items = getAllItems(originalCellWidget)
                      newCellWidget.addItems(items)
                   newCellWidget.setCurrentIndex(originalCellWidget.currentIndex())
                   newCellWidget.setEnabled(originalCellWidget.isEnabled())
              
            if self.sender().objectName().startswith("moveDown"): 
               self.tableWidget.selectRow(rowTarget) 
               self.tableWidget.removeRow(row)
            
            self.renameAllWidgets(self.tableWidget)
            self.getCurrentItem()

        
    def close(self): self.reject()

    def initDirectory(self):
        self.setInfos("")
        self.deleteAllLines()

    def getWebViewer(self):
        if self.directory.currentText()!= "" :
            from qsphere.ui import dlgNavigator
            self.dlgNavigator = dlgNavigator(self.iface, "filexml:%s" % (self.directory.currentText()), False, [], self.localLangage, self, None, True)
            self.dlgNavigator.exec_()

    def importQspContacts(self, fileName):
           config = ConfigParser()
           config.read(fileName)
           sections = config.sections()
           section = "tableroles"
           try : rows = int(config.get("tableroles","zRows"))
           except : rows = int(config.get("tableroles","rows"))
           try : cols = int(config.get("tableroles","zCols"))
           except : cols = int(config.get("tableroles","cols"))
           
           self.progressBar.setVisible(True)
           
           for row in range(rows):
                self.progressBar.setValue(int(100 * row/rows))
                self.addLine()
                counter = self.tableWidget.rowCount()-1
                
                infos = config.get(section,'zRow_%s' % (row))
                classWidgetValues = infos.split("|")
                
                for col in range(cols) :
                    infos = config.get(section,'zWidget_%s' % (col))
                    classWidgetInfos = infos.rstrip().split("|")
                    classWidgetInfo = "%s" % (classWidgetInfos[0])
                    if classWidgetInfo == "standard" :
                        item[col] = QTableWidgetItem()
                        item[col].setText("%s" % (classWidgetValues[col]))
                        zObj.setItem(counter,col, item[col])
                    else :
                        if isIntNumber(classWidgetValues[col]) and classWidgetInfos[0] not in("QLineEdit","MySimpleWidgetLineEdit", "MyWidgetLineEdit") : value = int(classWidgetValues[col])
                        elif isFloatNumber(classWidgetValues[col]) and classWidgetInfos[0] not in("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit") : value = float(classWidgetValues[col])
                        else : value = "%s" % (classWidgetValues[col])

                        if int(classWidgetInfos[2])!= -1 : addLineWidget(self, self.tableWidget, counter,  col, self.paramsLineWidget[col][0], self.paramsLineWidget[col][1], value)                                      
                        else : addLineWidget(self, self.tableWidget, counter,  col, self.Paramsself.AnnuaireLineWidget[col][0], self.paramsLineWidget[col][1], ("%s" % (classWidgetValues[col-1]), value))

                try : self.tableWidget.cellWidget(counter, 0).setFocus()
                except : pass

           self.progressBar.setVisible(False)
           self.progressBar.setValue(0)
           self.renameAllWidgets(self.tableWidget)
           self.getCurrentItem()

    def badIso(self):
        msg = QApplication.translate("QSphere","Invalid XML document !")
        sendMsg(self, "information" , msg, Qgis.Warning, self.duration_warning) 
       
    def loadXmlDirectory(self):
        if self.sender().objectName() == "reloadDirectory" :
            fileName = self.directory.currentText()
            if fileName == "" : return
            self.deleteAllLines()
            self.loadDataXmlDictionnary(fileName)
        else :
            title = self.sender().toolTip()
            InitDir = dirname(__file__) if self.initDirContacts == "" else self.initDirContacts
            dlgFile = QFileDialog(self, title)
            elt = QApplication.translate("QSphere","files").title()
            if self.sender() == self.loadDirectory : dlgFile.setNameFilters(("%s eXtensible Markup Language (*.xml *.XML)" % (elt), ))
            else : dlgFile.setNameFilters(("%s eXtensible Markup Language (*.xml *.XML)" % (elt), self.filtreQSP))
            dlgFile.setViewMode(QFileDialog.Detail)
            dlgFile.setDirectory(InitDir)
            dlgFile.setFileMode(QFileDialog.ExistingFile) 
            dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

            fixeLabelsFileDialog(self, dlgFile, 0, True)
            
            if dlgFile.exec_():
                fileName = "%s" % (dlgFile.selectedFiles()[0])
                inputFile = QFileInfo(fileName) 
                extension = inputFile.suffix().lower() 
                if fileName != "" :
                   if extension == "xml" : self.loadDataXmlDictionnary(fileName)
                   elif extension == "qsp" : self.importQspContacts(fileName)
                   self.setInfos(fileName)
                       

    def loadDataXmlDictionnary(self, fileName):
       self.listImportCategories = []
       iso = xmlIsoParser(fileName, None, 'MEDDE', self.localLangage)

       if iso == None :
          self.badIso()
          return 

       cond = True 
       try : iso.getTagDictionnary()
       except : cond = False
       if not cond : 
          self.badIso()
          return            

       try : iso.createISOdataStructure(False)
       except :
             msg =  QApplication.translate("QSphere","No contacts could be extract from the XML file !")
             sendMsg(self, "Information" , "%s :\n%s" % (msg, fileName), Qgis.Info, self.duration_info)              
             return
       
       if self.sender().objectName() == "loadDirectory" : self.deleteAllLines()

       contacts = iso.pointsofcontact
       for contact in contacts :
           self.addLine() 
           row = self.tableWidget.rowCount()-1  
           for i in range(len(contact)):
               index = 0
               if self.tableWidget.cellWidget(row, i).metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                     if i == 0:
                         indexes = [keyitem for keyitem, item in self.dictListOfRules.items() if item == contact[i]] 
                         if indexes!=[] : index = indexes[0]  
                     else : index = self.tableWidget.cellWidget(row, i).findText(contact[i].capitalize())
                     if index == -1 : index = 0
                     self.tableWidget.cellWidget(row, i).setCurrentIndex(index) 
               else : self.tableWidget.cellWidget(row, i).setText("%s" % (contact[i]))

           
    def save(self):
        fileName = self.directory.currentText()
        if fileName != "" : self.saveFile(fileName)
        else : self.saveAs()


    def saveFile(self, fileName):
        if self.tableWidget.rowCount()> 0 :
            exportDictionnaryToXml(self, self.tableWidget, fileName)
            msg =  QApplication.translate("QSphere","The dictionnary was saved as")
            sendMsg(self, "Information" , "%s :\n%s" % (msg, fileName), Qgis.Info, self.duration_info)
           
    def saveAs(self):
        if self.tableWidget.rowCount() == 0 : return

        elt = QApplication.translate("QSphere","files").title()
        self.extensions = ("%s eXtensible Markup Language (*.xml)" % (elt),)
        self.extensionsNames = {"xml" : "%s eXtensible Markup Language (*.xml)" % (elt)}

        if self.sender() == self.saveAsButton : title = QApplication.translate("QSphere", "Save as an XML Dictionnary")
        else : title = QApplication.translate("QSphere","Save as a copy ...")

        if self.windowTitle()!= self.rootWindowTitle:
            fileName = self.windowTitle().replace(self.rootWindowTitle, "")
            self.InitName = basename(fileName).split(".")[0]
            extension = getExtension(fileName)
            extension = extension.replace(".","")
        
        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters(self.extensions)
        dlgFile.selectNameFilter(self.extensionsNames["xml"])
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.initDirContacts)
        dlgFile.setAcceptMode(QFileDialog.AcceptSave)
        dlgFile.selectFile(self.InitName)

        fixeLabelsFileDialog(self, dlgFile, 1, True)
        
        if dlgFile.exec_():
            fichier = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
            self.saveFile(fichier)
            if self.sender()!= self.saveAsCopy :
                self.fichier = fichier
                self.setWindowTitle("%s : %s" % (self.rootWindowTitle, self.fichier))
                self.initDirContacts = dirname(self.fichier)
                if self.directory.findText(self.fichier)==-1 : self.directory.addItem("%s" % (self.fichier))
                self.directory.setCurrentIndex(self.directory.findText(self.fichier))


    def setInfos(self, fileName):
        if self.sender().objectName() in ("loadDirectory", "saveAsButton", "saveButton") :
            self.setWindowTitle("%s : %s" % (self.rootWindowTitle, basename(fileName)))
            if self.directory.findText(fileName)==-1 : self.directory.addItem("%s" % (fileName))
            self.directory.setCurrentIndex(self.directory.findText(fileName))
        if self.sender().objectName() == "newDirectory" :
            self.setWindowTitle("%s : %s" % (self.rootWindowTitle, ""))
            self.directory.setCurrentIndex(0)
        if fileName!= "" : self.initDirContacts = dirname(fileName)



    def openFormWizardMode(self):
        
        if self.tableWidget.rowCount()==0:
           makeLine(self, self.tableWidget, True, False, -1, True)
           self.tableWidget.selectRow(0)
           widget = self.tableWidget.cellWidget(0, 0)
           if not widget.isEnabled() : widget.setEnabled(True)

        self.paramsLineWidget = self.paramsWidget
        dialog = formWidgetDialog(self.tableWidget, self)
        
        dialog.exec_()
        self.paramsLineWidget = ((2, 0, 0),(-2, 0, ""), (-2, 0, ""), (2, 5, self.indexCountry), (0, 2, ""), (-2, 0, ""), (0, 0, ""), (-1, 0, ""), (-1, 0, ""), (0, 4, ""))


    def testUrl(self): testUrl(self)
    def goToLocalisator(self): goToLocalisator(self)


def makeLine(self, obj, drawResult, init, row, isCount):
    collection = []
    objWidgets = []

    if row == -1 : row = obj.rowCount()

    params = self.paramsWidget[obj.objectName()]
    
    if obj.objectName() == "tableroles" :
       obj.insertRow(row)
       if row == 0 : value = 2
       elif row == 1 : value = 6
       else : value = 1

       for i in range(len(params)):
           if i== 0 : widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], value)
           else : widget =  addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
           objWidgets.append(widget)
       collection.append(objWidgets)
       
       if init :
          row = obj.rowCount()
          obj.insertRow(row)
          for i in range(len(params)):
              if i== 0 and row == 1: widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], 6)
              else : widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
              objWidgets.append(widget)
          collection.append(objWidgets)
                
    
    return objWidgets
