# -*- coding:utf-8 -*-
#/* Type Safari 5.1 to 6.0 */
from qgis.core import Qgis
from qgis.gui import QgsMessageBar

from qgis.PyQt.QtCore import (QBuffer, QByteArray, QEventLoop, QFile, QFileInfo, QIODevice, QObject,
                              QSize, Qt, QUrl, pyqtSlot, pyqtSignal)
from qgis.PyQt.QtGui import (QFont, QImage, QIcon, QKeySequence, QMovie, QPainter, QPixmap, QStandardItem, QStandardItemModel)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.PyQt.QtNetwork import (QNetworkAccessManager, QNetworkReply, QNetworkRequest, QNetworkCookieJar)
from qgis.PyQt.QtWidgets import (QAction, QApplication, QComboBox, QFileDialog, QGroupBox, QHBoxLayout, QLabel,
                                 QLineEdit, QMdiArea, QMenu, QMessageBox, QPushButton, QRadioButton, QSizePolicy,
                                 QTableView, QTabWidget, QTableWidget, QTextEdit, QToolBar, QWidget)

#5.8.0
from qgis.PyQt.QtWidgets import (QCheckBox, QDialog)
#5.8.0

from qgis.PyQt import uic

from PyQt5.QtWebKit import QWebSettings
from PyQt5.QtWebKitWidgets import (QWebInspector, QWebPage, QWebView)
from PyQt5.QtXmlPatterns import QXmlQuery

from qsphere.qsphere_objmaker import (expandableGroupBox, movieCtrl, myPushButton, myPushButtonFunny, myTableWidget, myTextEdit,
                                      myMsgDlgBox, myWebComboBox, printerSphere)
from qsphere.qsphere_tools import (changeButtonIcon, cleanHtml, colorizeWords, convertSize, correctPath, fileInfosToHtml, fileNameWithExtension,
                                   fixeLabelsFileDialog, fileRessourceExist, fixeValuesForExtent, getHelp, getListXslt, getOptions, getThemeIcon,
                                   getWidget, isRasterFile, loadSimpleData, markerText,  sendMsg, testXmlData, textForMenu, windowIcon)  
from qsphere.xmlisoparser import xmlIsoParser

from codecs import open as codecsopen
from io import (StringIO, BytesIO, TextIOWrapper)
from os.path import (basename, dirname, exists, join, splitext)
from urllib.parse import (urlencode, unquote_plus, unquote, urlparse)
from webbrowser import open as webbrowseropen
from xml.etree import ElementTree as ET


class FindDialog(QDialog):
    def __init__(self, *args, **kwargs):
        super(FindDialog, self).__init__(*args, **kwargs)

        self.setMinimumSize(QSize(400, 160))
        self.setMaximumSize(QSize(400, 160))

        self.groupBoxSearch = QGroupBox(self)
        self.groupBoxSearch.setGeometry(5, 5, 390, 100)
        
        self.label = QLabel(self)
        self.label.setText(QApplication.translate("QSphere","Search for:"))
        self.label.setGeometry(10, 15, 80, 25)
        self.label.setAlignment(Qt.AlignRight)
        self.lineEdit = QLineEdit(self)
        self.lineEdit.setGeometry(100, 10, 280, 20)
        self.ret = 0
  
        self.caseCheckBox = QCheckBox(self)
        self.caseCheckBox.setText(QApplication.translate("QSphere", "Match case"))
        self.caseCheckBox.setGeometry(100, 40, 180, 20)
        
        self.firstModeCheckBox = QCheckBox(self)
        self.firstModeCheckBox.setText(QApplication.translate("QSphere","Find in results"))
        self.firstModeCheckBox.setGeometry(100, 60, 180, 20)

        self.countButton = QPushButton(self)
        self.countButton.setText(QApplication.translate("QSphere", "Find"))   
        self.countButton.setGeometry(290, 70, 100, 25)
        self.countButton.setEnabled(False)

        self.clearButton = QPushButton(self)
        self.clearButton.setText(QApplication.translate("QSphere", "Undo"))
        self.clearButton.setGeometry(180, 120, 100, 25)
  
        self.closeButton = QPushButton(self)
        self.closeButton.setText("&%s" % QApplication.translate("QSphere", "Close"))
        self.closeButton.setGeometry(290, 120, 100, 25)
  
        self.lineEdit.textChanged.connect(self.enableFindButton)
        self.countButton.clicked.connect(self.countFindClicked)
        self.clearButton.clicked.connect(self.clearSelection)
        self.closeButton.clicked.connect(self.closeme)

        self.counter = 0
        self.setWindowTitle(QApplication.translate("QSphere", "Search word"))
        self.lineEdit.setFocus()
        

    def enableFindButton(self, text): self.countButton.setEnabled(not text=="")

    def closeEvent(self,event): self.parent().page().findText(""); self.clearSelection();
    def closeme(self): self.parent().page().findText(""); self.clearSelection(); self.close()
    
    def countFindClicked(self):
        self.clearSelection()
        pattern = self.getFindText()

        flags = QWebPage.FindWrapsAroundDocument
        if not self.firstModeCheckBox.isChecked() : flags = QWebPage.HighlightAllOccurrences
        if self.caseCheckBox.isChecked() : flags |= QWebPage.FindCaseSensitively
        self.parent().page().findText(pattern, flags)

    def clearSelection(self) :
        self.parent().page().findText("", QWebPage.HighlightAllOccurrences)
        if self.sender() == self.clearButton :
           self.parent().page().findText(""); 
           self.lineEdit.setText("")
        
    def getFindText(self): return self.lineEdit.text()


class uiDlgNavigator(object):
    def setupUi(self):
        self.maxWidth, self.maxHeight, self.minWidth, self.minHeight = 900, 800, 500, 90
        self.resize(self.maxWidth,self.maxHeight)
        self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowMinimizeButtonHint | Qt.WindowCloseButtonHint)
        self.setObjectName("uiDlgNavigator")

        self.listUrlQueries = loadSimpleData(self, "file:/ressources/query_url.csv")
        if self.listUrlQueries == [] : self.listUrlQueries = ["xml_iso19139?uuid", "iso19139.xml?id", "importerDonnees.do?importer", "importerDonnees.do;jsessionid", "xml.metadata.get?id", "xml.metadata.get?uuid"]
    
        self.initDir = ""
        self.initExt = "xml" 
        self.initDirSearchFiles = ""
        self.oldurl = ""
        self.d = {}

        self.fileExtension = ".html"

        self.setWindowTitle(QApplication.translate("QSphere", "Web navigator ..."))

        getOptions(self)
        
        self.extensions = ("All (*.htm*)", "HTML (*.html)", "HTM (*.htm)")
        self.html = ""
        self.countfiles, self.countdirs = 0, 0

        self.eUrl = myWebComboBox(self) 
        self.eUrl.setEditable(True)
        self.eUrl.setInsertPolicy(QComboBox.InsertAlphabetically)
        self.eUrl.currentTextChanged.connect(self.eUrl.checkRegExp)
        self.eUrl.setGeometry(60, 10, self.width()-140, 25)

        self.mainTabWidget = QTabWidget(self)
        self.mainTabWidget.setObjectName("mainTabWidget")
        self.mainTabWidget.setTabPosition(QTabWidget.North) 
        self.mainTabWidget.setTabShape(QTabWidget.Rounded)
        self.mainTabWidget.setUsesScrollButtons(True)
        self.mainTabWidget.setTabsClosable(True)
        self.mainTabWidget.setMovable(True)
        self.mainTabWidget.setDocumentMode(True)

        self.mainTabWidget.tabCloseRequested.connect(self.closeTabs)
        self.mainTabWidget.currentChanged.connect(self.changeUrl)

        self.myPathIconvRightArrow, self.myPathIconvLeftArrow = "rightarrow.png", "leftarrow.png"
        self.myPathHome, self.myPathGoURL = "home.png", "gourl.png"
        self.myPathLoadXML, self.myPathLoadHTML = "loadxml.png", "loadxml.png"
        self.myPathPrint, self.myPathLoadFiles, self.myPathSave = "print.png", "open.png", "save.png"
        self.myReloadPage, self.myFilesNavigator, self.mySearchMenu = "reload.png", "filesnavigator_true.png", "searchmenu.png"
        self.myPathSearch = "toolssearchfiles.png" #5.8.0
        self.myPathEdit, self.myWaitingGIF = "editxml.png", "sablier.gif"
        self.myPathOptions = "options.png"

        self.tabWidget = self.addNewPage()
        self.mainTabWidget.addTab(self.tabWidget, QIcon(getThemeIcon("webpage.png")), "Page %s" % self.mainTabWidget.count())

        self.backAction = myPushButtonFunny(self) 
        self.backAction.initPushButton(25, 25,  5, 10, "backAction", "", 
                                         "%s ..." % QApplication.translate("QSphere","Previous page"), True, getThemeIcon(self.myPathIconvLeftArrow), 25, 25, True)
        self.backAction.setShortcut(QKeySequence("F2"))
        self.backAction.clicked.connect(self.getBack)

        self.forwardAction = myPushButtonFunny(self) 
        self.forwardAction.initPushButton(25, 25,  30, 10, "forwardAction", "", 
                                         "%s ..." % QApplication.translate("QSphere","Next page"), True, getThemeIcon(self.myPathIconvRightArrow), 25, 25, True)
        self.forwardAction.setShortcut(QKeySequence("F3"))
        self.forwardAction.clicked.connect(self.getForward)

        self.homeAction = myPushButtonFunny(self) 
        self.homeAction.initPushButton(25, 25,  55, 10, "homeAction", "", 
                                         "%s ..." % QApplication.translate("QSphere","Home page"), True, getThemeIcon(self.myPathHome), 25, 25, True)
        self.homeAction.setShortcut(QKeySequence("Ctrl+H"))
        self.homeAction.clicked.connect(self.getHome)


        self.reloadPage = myPushButtonFunny(self) 
        self.reloadPage.initPushButton(25, 25,  10, 10, "reloadPage", "", 
                                         "%s ..." % QApplication.translate("QSphere","Reload current page"), True, getThemeIcon(self.myReloadPage), 25, 25, True)
        self.reloadPage.setShortcut(QKeySequence("F5"))
        self.reloadPage.clicked.connect(self.reloadUrl)

                
        self.webViewerActions = myPushButtonFunny(self) 
        self.webViewerActions.initPushButton(40, 24,  10, 10, "webViewerActions", "", 
                                         "%s ..." % QApplication.translate("QSphere","Actions"), True, getThemeIcon("actions.png"), 40, 24, True)

        self.contextMnuWebViewerActions(self.webViewerActions)

        self.CloseButton = QPushButton(self)
        self.CloseButton.setObjectName("CloseButton")
        self.CloseButton.setText("&%s" % QApplication.translate("QSphere", "Close"))

        self.filter = "*.xml" 
        self.listXslt = getListXslt(self)

        self.movie = QMovie(getThemeIcon(self.myWaitingGIF))


        self.groupBoxParamsXML = expandableGroupBox(self)
        self.groupBoxParamsXML.initGroupBox(self, "groupBoxParamsXML", self.width()-190, 50, 24, 24, 180, 150, True)

        self.labelPrimaryXslt = QLabel(self.groupBoxParamsXML)
        self.labelPrimaryXslt.setText("%s : " % QApplication.translate("QSphere", "Primary XSLT"))
        self.labelPrimaryXslt.setAlignment(Qt.AlignLeft)
        self.primaryXslt = QComboBox(self.groupBoxParamsXML)
        self.primaryXslt.addItems(self.listXslt)
        
        self.labelSecondaryXslt = QLabel(self.groupBoxParamsXML)
        self.labelSecondaryXslt.setText("%s : " %  QApplication.translate("QSphere", "Secondary XSLT"))
        self.labelSecondaryXslt.setAlignment(Qt.AlignLeft)
        self.secondaryXslt = QComboBox(self.groupBoxParamsXML)
        self.secondaryXslt.addItems(self.listXslt)

        self.labelPrimaryXslt.setGeometry(5, 25, 100, 20)
        self.primaryXslt.setGeometry(5, 50, 150, 20)
        self.labelSecondaryXslt.setGeometry(5, 80, 100, 20)
        self.secondaryXslt.setGeometry(5, 105, 150, 20)


        font = QFont()
        font.setPointSize(7) 
        font.setWeight(8) 
        font.setBold(True)

        self.labelResultFiles = QLabel(self)
        self.labelResultFiles.setAlignment(Qt.AlignLeft)
        self.labelResultFiles.setObjectName("labelResultFiles")
        self.labelResultFiles.setText("")
        self.labelResultFiles.setFont(font)
        self.labelResultFiles.setStyleSheet(" QLabel {background-color: white;  border-width: 2px; border-radius: 10px; text-align: center; margin-top: 5px;}")
        self.labelResultFiles.setVisible(False)   

        self.movieCtrl = movieCtrl(self, "attente.gif")

        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

        test, value = fileRessourceExist(self, "xml/xsl/transformation_%s.xsl" % (self.localLangage))

        if self.parent != None :
            if self.parent.objectName()=="dlgContacts" :
                if self.homeurl != "" :
                    if self.homeurl.startswith("filexml:") :
                       inputFile = self.homeurl.replace("filexml:", "")
                       if exists(inputFile) :
                            isoParser = xmlIsoParser(inputFile, None, 'MEDDE', self.localLangage)
                            cond = isoParser.getTagDictionnary()
                            if not cond : test, value = fileRessourceExist(self, "xml/xsl/contacts_%s.xsl" % (self.localLangage))

        if value != "" : value = basename(value)
        index = self.listXslt.index(value) if value in (self.listXslt) else 0
        self.primaryXslt.setCurrentIndex(index)

        test, value = fileRessourceExist(self, "xml/xsl/iso19115_%s.xsl" % (self.localLangage))
        if value != "" : value = basename(value)
        index = self.listXslt.index(value) if value in (self.listXslt) else 0
        self.secondaryXslt.setCurrentIndex(index)
       
        
        self.primaryXslt.currentIndexChanged[str].connect(self.reloadUrl)
        self.secondaryXslt.currentIndexChanged[str].connect(self.reloadUrl)
        self.mainTabWidget.currentWidget().viewHtml.urlChanged.connect(self.changePageURL)
        self.mainTabWidget.currentWidget().viewHtml.linkClicked.connect(self.changePageURL)
        self.CloseButton.clicked.connect(self.closeDlg)

        if self.homeurl == "" : self.eUrl.currentIndexChanged[str].connect(self.getUrl)

        self.eUrl.addItems(sorted(self.serverNavigator))
        if self.homeurl != "" :
            if self.eUrl.findText(self.homeurl)==-1 : self.eUrl.addItem(self.homeurl)
            self.eUrl.setCurrentIndex(self.eUrl.findText(self.homeurl))
            self.getUrl()
            self.eUrl.currentIndexChanged[str].connect(self.getUrl)    
        else : self.homeurl = self.eUrl.currentText()


    def contextMnuWebViewerActions(self, obj): 
        actions = {"loafFile" : (self.myPathLoadFiles, "Open Files", "Ctrl+O" , self.loadTheFile, True),
                   "~0" : (),
                   "addTabPage" : ("add.png", "Add a new tab", "Ctrl+N" , self.addNewTab, True),
                   "closeAllTabsPage" : ("closewindow.png", "Delete all entries", "Ctrl+T" , self.closeAllTabs, True),
                   "~1" : (),
                   "searchInPage" : (self.myPathSearch, "Search word" , "Ctrl+F", self.findInPage, True),
                   "~2" : (),
                   "printPage" : (self.myPathPrint, "Print" , "Ctrl+P", self.printCurrentPage, True),
                   "savePage" : (self.myPathSave, "Save as a HTML file" , "Ctrl+S", self.saveHtml, True),
                   "~3" : (),
                   "optionsPage" : (self.myPathOptions, "Options" , None, self.optionsPage, True),
                   }
        self.builderContextMenu(obj, actions)

    #ADDING - 5.8.0
    def findInPage(self):
        #return
        ret = self.mainTabWidget.currentWidget().viewHtml.findDialog.exec_()
        self.mainTabWidget.currentWidget().viewHtml.findDialog.lineEdit.setFocus()
        if self.mainTabWidget.currentWidget().viewHtml.findDialog.ret > 0 :
           self.mainTabWidget.currentWidget().viewHtml.find_text =self.mainTabWidget.currentWidget().viewHtml.findDialog.getFindText()
        
    def optionsPage(self):
        pass

 
    def builderContextMenu(self, obj, actions):
        contextMnu = QMenu()
        
        for key in actions :
            if key.startswith("~") : contextMnu.addSeparator()
            elif key.startswith("list-") :
                    icon = getThemeIcon(actions[key][0])
                    subMenu = QMenu(QApplication.translate("QSphere", actions[key][1]), self)
                    subMenu.setIcon(QIcon(icon))
                    i = 0
                    for elt in actions[key][3] :
                        urlServer = QAction(QIcon(icon), elt, self)
                        urlServer.setObjectName("urlServer%s" % (i))
                        subMenu.addAction(urlServer)
                        urlServer.triggered.connect(self.shortCut)
                        i+= 1
                    contextMnu.addMenu(subMenu)     
            else :   
                icon = getThemeIcon(actions[key][0])
                #5.8.0
                maxDim, pattern = 60, "%s ... %s"
                text = textForMenu(pattern if actions[key][2] else None, QApplication.translate("QSphere", actions[key][1]), maxDim, actions[key][2])
                action = QAction(QIcon(icon), text, self)
                if actions[key][2] != "" : action.setShortcut(QKeySequence(actions[key][2]))
                setattr(contextMnu, key, action)
                action.setEnabled(actions[key][4])
                contextMnu.addAction(action)
                action.triggered.connect(actions[key][3])

        obj.setMenu(contextMnu)

    def closeAllTabs(self):
        if self.mainTabWidget.count()==1: return
        msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Warning"),
                                            "%s ?" % QApplication.translate("QSphere","Delete all entries"), "qsphere_stop.png", None, self.parent)
        windowIcon(msgDlgBox , "globe.png")
        ret = msgDlgBox.exec_()
        if ret == 1 :        
            for i in range(self.mainTabWidget.count()-1, 0, -1):
                self.mainTabWidget.removeTab(i)
            self.mainTabWidget.setCurrentIndex(0)   
            self.eUrl.setCurrentIndex(self.eUrl.findText("#blank"))

    def closeTabs(self, int):
        if int > 0 :
            self.mainTabWidget.removeTab(int)
        else :
            msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Warning"),
                                                QApplication.translate("QSphere","This tab can't be close.\nWould you like to clear content ?"), "qsphere_stop.png", None, self.parent)
            windowIcon(msgDlgBox , "globe.png")
            ret = msgDlgBox.exec_()
            if ret == 1 :
               self.eUrl.setCurrentIndex(self.eUrl.findText("#blank"))


    def changeUrl(self, int):
        if self.mainTabWidget.currentWidget().viewHtml.lastURL == "": self.mainTabWidget.currentWidget().viewHtml.lastURL = "#blank"
        self.eUrl.blockSignals(True)
        if type(self.mainTabWidget.currentWidget().viewHtml.lastURL) != str :
           try : self.eUrl.setCurrentIndex(self.eUrl.findText(self.mainTabWidget.currentWidget().viewHtml.lastURL.toString()))
           except : self.eUrl.setCurrentIndex(self.eUrl.findText(self.mainTabWidget.currentWidget().viewHtml.lastURL))
        else : self.eUrl.setCurrentIndex(self.eUrl.findText(self.mainTabWidget.currentWidget().viewHtml.lastURL))
        self.eUrl.blockSignals(False)
        

    def addNewTab(self):
        self.tabWidget = self.addNewPage()
        self.mainTabWidget.addTab(self.tabWidget, QIcon(getThemeIcon("webpage.png")), "Page %s" % self.mainTabWidget.count())
        self.mainTabWidget.setCurrentIndex(self.mainTabWidget.count()-1)

    def addNewPage(self):
        tabWidget = QTabWidget(self)
        tabWidget.setObjectName("tabWidget")
        tabWidget.setStyleSheet(
            "QTabBar::tab:selected {color: white; background-color: #5D5D5D ; font: 75 8pt 'Arial Rounded MT Bold'; border: 1px solid black; text-decoration: underline;}"
            "QTabBar::tab:!selected {color: black; background-color: rgb(255,255,255);border-bottom-color: #5D5D5D;}"
            "QTabWidget:pane {margin: 1px,1px,1px,1px;background-color: rgb(255,255,255);}"
            "QTabBar::tab {background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #cccccc, stop: 1 #5D5D5D);  border-width: 2px; border-radius: 5px; border-color: black; height: 36px;width: 140px;}"
            "QTabWidget{background-color: rgb(0, 0, 0); font: 75 8pt 'Arial Rounded MT Bold';}"
            "QTabWidget::tab-bar {left: 5px; bottom: 0px;background-color: rgb(255,255,255);}"
            "QTabBar::scroller {width: 0px;}"
        )
        tabWidget.setTabPosition(QTabWidget.South) 
        tabWidget.setTabShape(QTabWidget.Rounded)
        tabWidget.setMovable(True)
        tabWidget.setUsesScrollButtons(True)
        
        title = (QApplication.translate("QSphere","Web page"), QApplication.translate("QSphere", "Analyzer"))
        toolTip = (QApplication.translate("QSphere", "HTML page for help"), QApplication.translate("QSphere", "HTML page analyzer"))
        icons = ("pageweb.png", "pageanalyse.png")

        tabWidget.setIconSize(QSize(48, 42))

        for i in range(1,len(title)+1):
            tab = QWidget(self)
            tab.setObjectName("tab%s" % (i))
            tabWidget.addTab(tab, QIcon(getThemeIcon(icons[i-1])), title[i-1])
            tabWidget.setTabToolTip(i-1, toolTip[i-1])

            if i == 1:
                tabWidget.viewHtml = webView(self.mainTabWidget, tab, self) 
                tabWidget.viewHtml.initWebView(self, self.parent, self.duration_info, self.duration_warning)
                tabWidget.viewHtml.setGeometry(0, 0, self.mainTabWidget.width()-5, self.mainTabWidget.height()-20)
                tabWidget.viewHtml.setObjectName("viewHtml")
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.JavaEnabled, True)
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.PluginsEnabled, True)
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.AutoLoadImages, True)
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.LocalContentCanAccessRemoteUrls, True)
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
                tabWidget.viewHtml.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
                tabWidget.viewHtml.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
                tabWidget.viewHtml.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
                tabWidget.viewHtml.page().setForwardUnsupportedContent(True)
                self.manager = QNetworkAccessManager(tabWidget.viewHtml)
                self.jar = QNetworkCookieJar(self.manager)
                self.manager.setCookieJar(self.jar)

                #5.8.0
                tabWidget.viewHtml.findDialog = FindDialog(tabWidget.viewHtml)
                #5.8.0
                
            elif i == 2:
                tabWidget.inspector = QWebInspector(tab)
                tabWidget.inspector.setGeometry(0, 0, self.mainTabWidget.width()-5, self.mainTabWidget.height()-20)
                tabWidget.inspector.setPage(tabWidget.viewHtml.page())
                tabWidget.inspector.show()
                tabWidget.inspector.setVisible(True)

        return tabWidget

    def saveCurrentFilters(self):
        fichier = join(dirname(__file__), "ressources/filters_search.csv") 
        with codecsopen(fichier,'w','utf8') as f:
             for index in range(self.Filters.model().rowCount()):
                 if self.Filters.itemText(index)!= "" : f.write( "\"%s\"\n" % (self.Filters.itemText(index)))
        if f!= None : f.close()
        
        title = QApplication.translate("QSphere","Information")
        msg = QApplication.translate("QSphere","The file ini was saved as")
        sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, fichier), Qgis.Info, self.duration_info)
        
    def getHelp(self): getHelp(self.parent)

    def reject(self): self.killWindow()
    def closeDlg(self): self.killWindow()
    def killWindow(self):
        if self.parent != None  and not self.isModal :
           if self.senderBut != None :
              if self.senderBut.objectName().startswith("help_") : changeButtonIcon(self.parent, self.senderBut,"info.png", 24, 24)
           try :   
               if (self) in self.parent.childswindows :
                   self.parent.childswindows.remove(self)
                   self.parent.nb_window_childs.setText("%s" % (len(self.parent.childswindows)))
           except: pass
           try : self.parent.windowsNoMDIChilds.remove(self)
           except : pass
           self.parent.webNavigator = None
        self.d = {}
        self.reject()
        
    def printCurrentPage(self):
        orientation = QPrinter.Portrait if self.isXml else QPrinter.Landscape
        dlgPrint = printerSphere(self, self.mainTabWidget.currentWidget().viewHtml, QApplication.translate("QSphere", "Print current page"), orientation)

        
    def resizeEvent(self, ev):
        if self.groupBoxParamsXML.state == 1: self.groupBoxParamsXML.setGeometry(self.width()-190, 50, 180, 150)
        else : self.groupBoxParamsXML.setGeometry(self.width()-self.groupBoxParamsXML.width()-190, 50, self.groupBoxParamsXML.width(), self.groupBoxParamsXML.height())
        self.groupBoxParamsXML.updateSizes(self.width()-190, 50)
       

        self.backAction.setGeometry(0, 10, 25, 25)
        self.forwardAction.setGeometry(30, 10, 25, 25)

        self.eUrl.setGeometry(60, 10, self.width()-150, 25)

        self.reloadPage.setGeometry(self.width()-85, 10, 25, 25)
        self.homeAction.setGeometry(self.width()-60, 10, 25, 25)
        self.webViewerActions.setGeometry(self.width()-45, 10, 40, 24)
        
        self.movieCtrl.setGeometry(int(self.width()/2)-64, int(self.height()/2)-64, 64, 64)
        self.labelResultFiles.setGeometry(int(self.width()/2)-50, int(self.height()/2)-40, 32, 20) 
        self.mainTabWidget.setGeometry(10, 40, self.width()-20, self.height()-50)
        
        self.CloseButton.setGeometry(self.width()-120, self.height()-30, 100, 25)
        
        orindex = self.mainTabWidget.currentIndex()
        for i in range(self.mainTabWidget.count()):
            self.mainTabWidget.setCurrentIndex(i)
            tab = self.mainTabWidget.currentWidget()
            
            tab.setGeometry(10, 10, self.mainTabWidget.width()-20, self.mainTabWidget.height()-20)
            tab.viewHtml.setGeometry(0, 0, self.mainTabWidget.width()-10, self.mainTabWidget.height()-20)
            tab.inspector.setGeometry(0, 0, self.mainTabWidget.width()-10, self.mainTabWidget.height()-20)
            
        self.mainTabWidget.setCurrentIndex(orindex)
        self.barInfo.setGeometry(0, 40, self.width(), 90)    
        

    def reloadUrl(self):
        self.mainTabWidget.currentWidget().viewHtml.pageAction(QWebPage.Reload)
        self.getUrl()
        
    def getBack(self):
        self.mainTabWidget.currentWidget().viewHtml.history().back()
        url = "%s" % (unicode(self.mainTabWidget.currentWidget().viewHtml.history().currentItem().url().toString()))
        if self.eUrl.findText(url) == -1 : self.eUrl.addItem(url)
        self.eUrl.setCurrentIndex(self.eUrl.findText(url))
        fixeEnabledHistory(self, self.mainTabWidget.currentWidget().viewHtml)
        
    def getForward(self):
        self.mainTabWidget.currentWidget().viewHtml.history().forward()
        url = "%s" % (unicode(self.mainTabWidget.currentWidget().viewHtml.history().currentItem().url().toString()))
        if self.eUrl.findText(url) == -1 : self.eUrl.addItem(url)
        self.eUrl.setCurrentIndex(self.eUrl.findText(url))        
        fixeEnabledHistory(self, self.mainTabWidget.currentWidget().viewHtml)

    def saveHtml(self):
        url = self.eUrl.currentText() 
        
        if url.startswith("filexml:") : url = url.split("filexml:")[1]
        if url.startswith("file:") : url = url.split("file:")[1]
        if url.startswith("fileHTML:") : url = url.split("fileHTML:")[1]
        
        if exists(unquote(url)) :
            fileName = basename(url).split(".")[0]
            initDir = dirname(url) if self.initDir == "" else self.initDir
            initDir = correctPath(initDir)
        else :
            initDir = dirname(__file__) if self.initDir == "" else self.initDir
            fileName = self.mainTabWidget.tabText(self.mainTabWidget.currentIndex())
            for carac in ["/", "\\", ":", "*", "?", "\"", "<", ">" ,"|"] : fileName = fileName.replace(carac, "")
        
        title = QApplication.translate("QSphere","Save as a HTML file")
        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters(self.extensions)
        dlgFile.setDefaultSuffix("html")
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(initDir)
        dlgFile.selectFile(fileName)
        dlgFile.setAcceptMode(QFileDialog.AcceptSave)
        
        fixeLabelsFileDialog(self, dlgFile, 1, True)
        
        if dlgFile.exec_():
            fileName = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
            html = self.mainTabWidget.currentWidget().viewHtml.page().currentFrame().toHtml() 

            with open(fileName, "w", encoding="utf-8", errors="surrogateescape") as inFile :
                try :
                    inFile.write(unquote_plus(html))
                    title = QApplication.translate("QSphere","Information")
                    success = QApplication.translate("QSphere","Successfull")
                    sendMsg(self, title , "%s :<br>%s<br>%s" % (title, fileName, success) , Qgis.Info, self.duration_info)                
                except :
                    try :
                        inFile.write("%s" % (html))
                        title = QApplication.translate("QSphere","Information")
                        success = QApplication.translate("QSphere","Successfull")
                        sendMsg(self, title , "%s :<br>%s<br>%s" % (title, fileName, success) , Qgis.Info, self.duration_info)
                    except :
                        inFile.write("<html><style>h1 {font-size: 1.5em;}</style><h1>error</h1></html>")
                        title = QApplication.translate("QSphere","Warning")
                        success = QApplication.translate("QSphere","Error")
                        sendMsg(self, title , "%s :<br>%s<br>%s" % (title, fileName, success) , Qgis.Warning, self.duration_warning)
                                                      
            
    def shortCutInspire(self):  self.endLoadTheFile("http://cnig.gouv.fr/?page_id=2916")
    
    def shortCut(self): self.endLoadTheFile(self.sender().text())
      
    def getUrl(self):
        url = self.eUrl.currentText()

        tagLocalFile = ["file:", "filexml:", "fileHTML:",  "about:", "#blank"]
        cond = False
        for elt in tagLocalFile :
            if url.startswith(elt): cond = True ; break ;
            
        if url == "#blank" :
            self.mainTabWidget.currentWidget().viewHtml.setHtml("")
            self.mainTabWidget.setTabText(0, "Page 0")
            self.mainTabWidget.currentWidget().viewHtml.lastURL = url
            return


        if url == "" : return
        if not cond :    
            if (url[0:7] != "http://" and url[0:6] != "ftp://" and url[0:8] != "https://") and not(exists(url.split("#")[0])) :
                url = "http://%s" % (url) if url != "" else url
                if self.eUrl.findText(url)==-1 : self.eUrl.addItem(url)
                self.eUrl.setCurrentIndex(self.eUrl.findText(url))
                self.mainTabWidget.currentWidget().viewHtml.lastURL = url
                
            if self.eUrl.styleSheet() == "background-color:red;" and not exists(url.split("#")[0]) :
               title = QApplication.translate("QSphere","Information") 
               msg = QApplication.translate("QSphere","Invalid URL ! Use open function if necessary for local resources ...")
               sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
               return
        else :
                urlLocalFile = ""
                for elt in tagLocalFile :
                    if url.startswith(elt):
                        urlLocalFile = url.replace(elt, "")
                        break
                if urlLocalFile == "" : return
 

                if exists(unquote(urlLocalFile)) :
                    url = unquote(url)

                    inputFile = QFileInfo(urlLocalFile)
                    suffixe = inputFile.suffix().lower()


                    if suffixe in ("gif", "jpeg", "jpg", "png")  :
                        self.mainTabWidget.currentWidget().viewHtml.setHtml("<img src='file:///%s'>" % urlLocalFile)
                        self.eUrl.currentTextChanged.disconnect(self.eUrl.checkRegExp)
                        try : self.eUrl.currentIndexChanged[str].disconnect(self.getUrl)
                        except : pass
                        self.eUrl.setStyleSheet("QComboBox {background-color:#AEEE00;}"
                              "QComboBox QAbstractItemView {background-color:#AEEE00; min-width:%spx;}" % (self.minWidth)
                              )   
                        self.eUrl.currentTextChanged.connect(self.eUrl.checkRegExp)

                        try : self.eUrl.currentIndexChanged[str].connect(self.getUrl)
                        except : pass

                        self.mainTabWidget.currentWidget().viewHtml.lastURL = url
  
                    else : getLink(self, url, False, self.mainTabWidget.currentWidget().viewHtml)

                else: getLink(self, url, False, self.mainTabWidget.currentWidget().viewHtml)
   
                self.mainTabWidget.currentWidget().viewHtml.lastURL = url

                return

    
        if self.eUrl.currentText() != "" and self.eUrl.styleSheet()!="background-color:red;" :
            getLink(self, self.eUrl.currentText(), False, self.mainTabWidget.currentWidget().viewHtml)
        else :
            if self.eUrl.currentText() != "" :
                title = QApplication.translate("QSphere","Information")
                msg = QApplication.translate("QSphere","URL not natively supported ! Use open function if necessary for local resources ...")
                sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)


    def selfActiveLink(self, fileName):
        if exists(fileName) :
            inputFile = QFileInfo(fileName)
            suffix = inputFile.suffix().lower()
            if suffix == "gbmeta" : fileName = "%s.xml" % fileName
            rootUrl = "filexml:" if suffix in ("xml", "qss", "gbmeta") else "file:"
            self.homeurl = "%s%s" % (rootUrl, fileName.replace("\\", "/"))
            fixeEnabledHistory(self, self.mainTabWidget.currentWidget().viewHtml)
            self.mainTabWidget.currentWidget().viewHtml.lastURL = self.homeurl

            index = self.eUrl.findText(self.homeurl)
            if index == -1 :
                self.eUrl.addItem(self.homeurl)
                index = self.eUrl.findText(self.homeurl)
            self.eUrl.setCurrentIndex(index)
            self.eUrl.setCurrentText(self.homeurl)
    
    def loadTheFile(self):
        title = QApplication.translate("QSphere","View a file (XML ISO 19139, HTML ...)")
        initDir = dirname(__file__) if self.initDir == "" else self.initDir
        dlgFile = QFileDialog(self, title)
        elt0 = QApplication.translate("QSphere","files").title()
        elt1 = QApplication.translate("QSphere","project").title()
        elt2 = QApplication.translate("QSphere","Sessions").title()
        namedFiltres = ["%s CSV (*.csv)" % (elt0), "%s gbmeta (*.gbmeta)" % (elt0), "%s images (*.gif *.jpg *.jpeg *.png)" % (elt0),
                        "%s HTML (*.htm*)" % (elt0), "%s Web (*.htm* *.xml)" % (elt0),  
                        "QGis %s (*.qmd)" % elt0, "%s QSphere (*.qsp)" % (elt1), "%s QSphere (*.qsr)" % (elt0), "%s QSphere (*.qss)" % (elt2), 
                        "%s eXtensible Markup Language (*.xml *.XML)" % (elt0), "%s eXtensible Markup Language (*.xsd *.XSD)" % (elt0)]
        dlgFile.setNameFilters(namedFiltres) 
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(initDir)
        dlgFile.setFileMode(QFileDialog.ExistingFile)
        index = 0
        for elt in namedFiltres :
            if elt.find(".%s" % self.initExt) !=-1 : break
            index+= 1
        if index > len(namedFiltres)-1: index = 0    
        dlgFile.selectNameFilter(namedFiltres[index])
        dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

        fixeLabelsFileDialog(self, dlgFile, 0, True)
        
        if dlgFile.exec_():
            fileName = "%s" % (dlgFile.selectedFiles()[0])
            suffix = QFileInfo(fileName).suffix().lower()
            fileName = "filexml:%s" % fileName if suffix in ("xml", "gbmeta") else "file:%s" % fileName
            if fileName!="" :
                self.initDir = dirname(fileName)
                self.initExt = suffix
                if suffix == "gbmeta" : fileName = "%s.xml" % fileName
                self.endLoadTheFile(fileName)

    def endLoadTheFile(self, fileName):
        if self.mainTabWidget.parent().eUrl.findText(fileName)==-1 : self.mainTabWidget.parent().eUrl.addItem(fileName)
        self.mainTabWidget.parent().eUrl.setCurrentIndex(self.mainTabWidget.parent().eUrl.findText(fileName))

    def getHome(self):
        fixeEnabledHistory(self, self.mainTabWidget.currentWidget().viewHtml)
        self.mainTabWidget.currentWidget().viewHtml.lastURL = self.homeurl
        self.eUrl.setCurrentIndex(self.eUrl.findText(self.homeurl))

    def changePageURL(self):
        page = self.mainTabWidget.currentWidget().viewHtml.page()
        linkact = page.action(QWebPage.OpenLink)


class Timeout(Exception):
    """A timeout (usually on page load) has been reached."""
           
class webPage(QWebPage):
    formSubmitted = pyqtSignal(QUrl)

    def eventsLoop(self, wait=0.01):
        self.application.processEvents()
        time.sleep(wait)    
    
    def __init__(self, dialog, parent=None):
        super(webPage, self).__init__(parent)
        self.dialog = dialog
        self.mainFrame().javaScriptWindowObjectCleared.connect(self.refreshJavaScript) 
        self.mainFrame().loadFinished.connect(self._on_load_finished)
        self.mainFrame().loadStarted.connect(self._on_load_started)
        self.headers = []
        self.loadTimeOut = -1
        self.loadSuccess = False
        self.setSettings()        

    def setSettings(self):
        self.settings().setAttribute(QWebSettings.LocalStorageDatabaseEnabled, True)
        self.settings().setAttribute(QWebSettings.JavascriptCanOpenWindows, True)
        self.settings().setAttribute(QWebSettings.AutoLoadImages, False)        

    def setHeaders(self, headers): self.headers = headers

    def _on_load_started(self):
        self.loadSuccess = False
        self.loadLast = 0
        self.dialog.movieCtrl.startMovie()
        self.dialog.labelResultFiles.setVisible(True)
  
    def _on_load_finished(self):
        self.loadSuccess = True
        self.dialog.labelResultFiles.setVisible(False)
        self.dialog.movieCtrl.stopMovie()
            
    def refreshJavaScript(self): self.mainFrame().addToJavaScriptWindowObject('APP', self)

    def make_request(self, url):
        url = QUrl(url)
        req = QNetworkRequest(url)
        for header in self.headers:
            val = self.headers[header]
            req.setRawHeader(header, val)
        return req


    def load(self, url, headers=None, body=None, loadTimeOut=-1, delay=None):
        if not headers: self.headers = []
        if not body: body = ""
        req = self.make_request(url)
        self.loadSuccess = False
        self.loadTimeOut = loadTimeOut
  
        self.mainFrame().load(req, QNetworkAccessManager.GetOperation, body)
        self.waitEnding()
        if delay: self.waitDelays(delay)
  
    def waitEnding(self):
        while not self.loadSuccess:
            self.eventsLoop()
            self.loadLast += 1
            if self.loadTimeOut > 0 and self.loadLast >= self.loadTimeOut * 100:
                raise Timeout("Timeout reached: %d seconds" % self.loadTimeOut)
  
    def waitDelays(self, seconds):
  
        for j in range(1, seconds):
            for i in range(1, 100):
                self.waitEnding()
                self.eventsLoop()
    
    def html(self): return unicode(self.mainFrame().toHtml())

    def acceptNavigationRequest(self, frame, req, navType):
        elts, urlQuery, queryTest = req.url().toString().split("/"), [], False
        urlQuery = elts[-1].split("=")

        if urlQuery != [] :
           if urlQuery[0] in self.parent.dialog.listUrlQueries : queryTest = True
           if queryTest :
              self.parent.mainTabWidget.currentWidget().viewHtml.telech = telecharge(req.url(), self.parent.dialog, None, "*.xml", self.parent.dialog.duration_info, self.parent.dialog.duration_warning, False)
              self.parent.mainTabWidget.currentWidget().viewHtml.telech.finTelecharge.connect(self.parent.mainTabWidget.currentWidget().viewHtml.telecharge_ok)
              if  self.parent.objectName() == "tempoViewer" : self.parent.close()
              return super(webPage, self).acceptNavigationRequest(frame, req, navType)
        
        if navType == QWebPage.NavigationTypeFormSubmitted :
            self.properties = {}
            for header in req.rawHeaderList(): self.properties["%s" % (header)] = "%s" % (req.rawHeader(header))
            self.formSubmitted.emit(req.url())
        return super(webPage, self).acceptNavigationRequest(frame, req, navType)

    
class webView(QWebView):
    def __init__(self, mainTabWidget, parent, dialog):
        QWebView.__init__(self, parent)
        self.toCopy = ""
        self.current_url = QUrl("")
        self.lastURL = "#blank"
        self.fileName = self.TypeMime = None
        self.parent = parent
        self.dialog = dialog
        self.mainTabWidget = mainTabWidget
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.context_menu_requested)
        self.setAcceptDrops(True)

        self.setPage(webPage(self.dialog))

        self.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.page().parent = self
        self.page().unsupportedContent.connect(self.unsupportedContent)
        self.page().linkClicked.connect(self.GoLink) 
        self.page().formSubmitted.connect(self.handleFormSubmitted)

        self.isPageOkLoaded = False
        self.page().loadProgress.connect(self.progressPage)
        self.loadFinished.connect(self.changeNameTab)


    def dropEvent(self, event):
        source = event.mimeData().text()
        source = source.split("\n")
        for elt in source :
                if elt != "" :
                   eltFile = elt.replace("file:///", "")
                   sourceFile = QFileInfo(eltFile)
                   if sourceFile.isFile():
                      if sourceFile.suffix().lower() == "xml" : elt = elt.replace("file:///", "filexml:") 
                   self.dialog.addNewTab() 
                   self.dialog.endLoadTheFile(elt)

    def progressPage(self, progress):
        if not self.isPageOkLoaded :
            self.dialog.labelResultFiles.setText("%s %%" % progress)
            self.dialog.labelResultFiles.repaint()
            
            if progress == 100 :
                self.dialog.labelResultFiles.setVisible(False)
                self.dialog.movieCtrl.stopMovie()


    def changeNameTab(self, isOkLoaded):
        title = QApplication.translate("QSphere", "No title")
        if self.title()!="": title = self.title()
        else :
            try : title = unquote(basename(self.mainTabWidget.parent().eUrl.currentText())) 
            except : pass
            if title in ("", "#blank"): title = QApplication.translate("QSphere", "No title") 
        self.mainTabWidget.setTabText(self.mainTabWidget.currentIndex(), title)
        self.mainTabWidget.setTabIcon(self.mainTabWidget.currentIndex(), self.icon())

    def createWindow(self, windowType):
        if windowType == QWebPage.WebBrowserWindow:
            self.webView = webView(self.mainTabWidget, self.parent, self.dialog)
            self.webView.setObjectName("tempoViewer")
            self.webView.dialog = self.dialog 
            self.webView.Metadata = self.Metadata 
            self.webView.duration_info = self.duration_info
            self.webView.duration_warning = self.duration_warning 
            self.webView.setAttribute(Qt.WA_DeleteOnClose, True)
            return self.webView
        return super(webView, self).createWindow(windowType)

    def handleFormSubmitted(self, url):
        path = url.path()
        if path.lower().find("search") == -1 :
            document = self.page().mainFrame()
            inputs = document.findAllElements("input")
            data, fileName = {}, ""
            for item in  inputs :
                data[item.attribute("name")] = item.attribute("value")
                if item.attribute("name") == "nom" : fileName = item.attribute("value")

            if data != {} : data = urlencode(data)
            headers = {"Content-Type" : "application/x-www-form-urlencoded"}
            performUrl = "%s?%s" % (url.toString(), data)
            data = None
            self.telech = telecharge(QUrl(performUrl), self.dialog, fileName, "*.zip", self.duration_info, self.duration_warning, False)
            self.telech.finTelecharge.connect(self.mainTabWidget.currentWidget().viewHtml.telecharge_ok)
        

    def initWebView(self, dialog, metadata, duration_info, duration_warning):
        self.dialog = dialog
        self.Metadata = metadata
        self.duration_info = duration_info
        self.duration_warning = duration_warning
        
    def context_menu_requested(self, point):
        self.imageurl = None
        context_menu = QMenu()
        page = self.page()
        frame = page.currentFrame()
        if unicode(self.selectedText()):
            menuIcon = getThemeIcon("copier.png")
            text = (QApplication.translate("QSphere", "Copy"))
            self.copy = context_menu.addAction(QIcon(menuIcon), text)
            self.toCopy = unicode(self.selectedText())
            self.copy.triggered.connect(self.makeToCopy)

        hit_test = frame.hitTestContent(point)
        
        if unicode(hit_test.linkUrl().toString()):
            menuIcon = getThemeIcon("copierlink.png")

            text = QApplication.translate("QSphere", "Open link in new tab")
            self.openURLinNewTab = context_menu.addAction(QIcon(menuIcon), text)
            self.toOpenUrl = unicode(hit_test.linkUrl().toString())
            self.openURLinNewTab.triggered.connect(self.makeOpenLink)

            text = QApplication.translate("QSphere", "Open link in my default navigator")
            self.openUrl = context_menu.addAction(QIcon(menuIcon), text)
            self.toOpenUrl = unicode(hit_test.linkUrl().toString())
            self.openUrl.triggered.connect(self.makeOpenUrl)

            text = QApplication.translate("QSphere", "Force open link")
            self.openLink = context_menu.addAction(QIcon(menuIcon), text)
            self.ToOpenLink = unicode(hit_test.linkUrl().toString())
            self.openLink.triggered.connect(self.makeOpenLink)

            context_menu.addSeparator()

            text = (QApplication.translate("QSphere", "Copy link"))
            self.copyURL = context_menu.addAction(QIcon(menuIcon), text)
            self.toCopyUrl = unicode(hit_test.linkUrl().toString())
            self.copyURL.triggered.connect(self.makeToCopyUrl)

            
        context_menu.addSeparator()

           
        menuIcon = getThemeIcon("selectall.png")
        text = (QApplication.translate("QSphere", "Select all"))
        self.selall = QAction(QIcon(menuIcon), text, self)
        context_menu.addAction(self.selall)
        self.selall.triggered.connect(self.selectAll)
        
        context_menu.addSeparator()
        
        menuIcon = getThemeIcon("coller.png")
        text = (QApplication.translate("QSphere", "Paste"))
        self.paste = QAction(QIcon(menuIcon), text, self)
        context_menu.addAction(self.paste)
        self.paste.triggered.connect(self.makeToPaste)
        
        if unicode(page.mainFrame().hitTestContent(point).element().tagName()).lower()== "img":
            self.imageurl =  page.mainFrame().hitTestContent(point).imageUrl()
            tempoSRC = page.mainFrame().hitTestContent(point).element().attribute("src")
            tempoSRC = tempoSRC.replace("\\","/")

            if tempoSRC!="" :

               if tempoSRC.startswith("data:image") : self.fileName = "monimage.jpg"
               else : self.fileName = tempoSRC.rsplit("/",1)[1]
               
               try : self.TypeMime = "*.%s" % (self.fileName.rsplit(".",1)[1])
               except : pass
            
            context_menu.addSeparator()

            menuIcon = getThemeIcon("saveas.png")
            text = (QApplication.translate("QSphere", "Save image as ..."))
            self.downloadimg = QAction(QIcon(menuIcon), text, self)
            context_menu.addAction(self.downloadimg)
            self.downloadimg.triggered.connect(self.downloadImg)

            self.paste.setEnabled(False)
   
        context_menu.exec_(self.mapToGlobal(point))


    def downloadImg(self):
        self.telech = telecharge(self.imageurl, self.dialog, self.fileName, self.TypeMime, self.duration_info, self.duration_warning, True)
        self.telech.finTelecharge.connect(self.telecharge_ok)

    def unsupportedContent(self, reply, outfd=None):
        if not reply.error():
            if type(self.current_url) != str  : url = self.current_url.toString()
            else : url = self.current_url 
            try : 
                elts = urlparse(url)
                inputFile = elts.path.split("/")[-1]
                extension = inputFile.split(".")[-1]
            except : inputFile, extension = "", "*"    
           
            self.telech = telecharge(reply.url(), self.dialog, inputFile, "*.%s" % (extension), self.duration_info, self.duration_warning, False)
            self.telech.finTelecharge.connect(self.telecharge_ok)
        else :
            title = QApplication.translate("QSphere","Information")
            msg = QApplication.translate("QSphere","Error on unsupported content :")
            sendMsg(self.dialog, title , "%s %s" % (msg, reply.errorString()), Qgis.Warning, self.duration_warning)


    def telecharge_ok(self, ok):
        if ok:
            title = QApplication.translate("QSphere","Information")
            msg = QApplication.translate("QSphere","Donwload success ! File save as :")
            sendMsg(self.dialog, title , "%s %s" % (msg, self.telech.fileName), Qgis.Info, self.duration_info) 
        else:
            title = QApplication.translate("QSphere","Information")
            msg = QApplication.translate("QSphere","Donwload error ! File not save ...")
            sendMsg(self.dialog, title , msg, Qgis.Warning, self.duration_warning)
    
    def makeOpenLink(self):
        url = self.ToOpenLink
        if self.sender() == self.openURLinNewTab : self.dialog.addNewTab()
        reLinked = False
        if self.dialog.eUrl.findText(url)==-1 : self.dialog.eUrl.addItem(url)
        else : reLinked = True
        self.dialog.eUrl.setCurrentIndex(self.dialog.eUrl.findText(url))
        self.current_url = QUrl(url)
        if reLinked : getLink(self.dialog, self.current_url, True, self)

    
    def makeOpenUrl(self):
        try : webbrowseropen(self.toOpenUrl)
        except :
            title = QApplication.translate("QSphere","Information")
            msg = QApplication.translate("QSphere","Try to open Web navigator for https protocol ...")
            sendMsg(self.dialog, title , msg, Qgis.Warning, 3)
        
    def makeToCopy(self): QApplication.clipboard().setText(self.toCopy)
    def makeToCopyUrl(self): QApplication.clipboard().setText(self.toCopyUrl)
    def makeToPaste(self): self.page().triggerAction(QWebPage.Paste)
    def selectAll(self): self.page().triggerAction(QWebPage.SelectAll)
    
    def GoLink(self, url):
        #ADDING !!
        text =  url.toString().split("/")[-1]
        doc = self.dialog.mainTabWidget.currentWidget().viewHtml.page().mainFrame().documentElement()
        elems = doc.findAll("a")
        for elem in elems.toList() :
            if elem.attribute("href")==text :
               if elem.attribute("target").lower()== "_blank" : self.dialog.addNewTab()

        
        reLinked, self.isPageOkLoaded  = False, False
        self.dialog.movieCtrl.startMovie()
        self.dialog.labelResultFiles.setVisible(True)
        url = url.toString()
        if self.dialog.eUrl.findText(url)==-1 and url != "": self.dialog.eUrl.addItem(url)
        else : reLinked = True
        if url != "" : self.dialog.eUrl.setCurrentIndex(self.dialog.eUrl.findText(url))
        self.current_url = url
        if reLinked : getLink(self.dialog, self.current_url, True, self)
        self.dialog.labelResultFiles.setVisible(False)
        self.dialog.movieCtrl.stopMovie()


def fixeEnabledHistory(dialog, webViewer):
    action = dialog.findChild(QAction, "backAction")
    if action :
       action.setEnabled(True) if webViewer.history().backItems(100) else action.setEnabled(False)
    action = dialog.findChild(QAction, "fowardAction")
    if action :       
        action.setEnabled(True) if webViewer.history().forwardItems(100)  else action.setEnabled(False)
    

def checkListOfXslt(self, url):
    if self.listXslt == [] and url.startswith("filexml:"): url = url.replace("filexml:", "")
    return url   
    
def getLink(dialog, url, isUnicode, webViewer) :

        anchor = dialog.html = ""
        isXml = False

        url = checkListOfXslt(dialog, url)
        webViewer.lastURL = url
       
        try :
            if isUnicode : url = "%s" % (unicode(url.toString()))
        except : return
        
        if url.find("xml:")!=-1 or url.find("file:")!=-1 :
           dialog.eUrl.setStyleSheet("QComboBox {background-color:#AEEE00;}"
                                     "QComboBox QAbstractItemView {background-color:#AEEE00; min-width:%spx;}" % (dialog.minWidth)
                                      )
        dialog.fileExtension, dialog.isXml, dialog.iconName = ".html", False, "navigatorweb.png"

    
        if url.find("file:")!=-1 or url.find("fileHTML:")!=-1 :
           refItem = url

           url = url.replace("file:///","")
           if url.find("file:")!=-1 : url = url.replace("file:","")
           if url.find("fileHTML:")!=-1 : url = url.replace("fileHTML:","")
           if url.find(".html#")!= -1 :
              temp = url.split("#")
              url = temp[0]
              anchor = temp[1]
           webViewer.setStyleSheet("background-color:rgb(255,255,255); padding: 7px ; color:rgb(255,255,255)")


           if exists(url) :
              inputFile = QFileInfo(url)
              suffixe = inputFile.suffix().lower()

              if suffixe == "xsd" :
                 f = open(url, "r")
                 text = f.read().replace("<", "<li>&lt;")
                 f.close()

                 style = """<style>
body {font-size: 1.2em;font-family: "Helvetica Neue", Helvetica, sans-serif; margin: 50px;}
.custom-counter {margin: 0;padding: 0;list-style-type: none;}
.custom-counter li { counter-increment: step-counter; margin-bottom: 5px;}
.custom-counter li::before {content: counter(step-counter); margin-right: 20px;font-size: 80%; background-color: rgb(180, 180, 180);color: white;font-weight: bold; padding: 3px 8px; border-radius: 11px;}
</style>"""

                 baseUrl = QUrl.fromLocalFile(url)
                 webViewer.setHtml("<html>%s<body><ol class='custom-counter'>%s</ol><br><br></body></html>" % (style, text), baseUrl)
                 

              elif suffixe in ("qsp", "qsr") :
                  baseUrl = QUrl.fromLocalFile(url)
                  with open(url, 'r', encoding="utf-8", errors="surrogateescape") as myfile : myfileText = myfile.readlines()
                  html = ""
                  for k in range(len(myfileText)): html+= "%s" % (markerText(dialog, myfileText[k]))
                  webViewer.setHtml(html, baseUrl)

              elif suffixe in ("qmd", "qss") :
                  baseUrl = QUrl.fromLocalFile(url)
                  with open(url, 'r', encoding="utf-8", errors="surrogateescape") as myfile : html = myfile.read()
                  webViewer.setHtml(colorizeWords(html), baseUrl)  
                  

              else : webViewer.load(QUrl("file:///%s" % url))


           return

          
        elif url.find("filexml:")!=-1 :
            dialog.isXml = True
            url = url.replace("filexml:","")
            url = unquote(url)

            dialog.initDir = correctPath(dirname(url))
            fileName, dialog.fileExtension = splitext(url)

            if dialog.fileExtension.lower() == ".xml" :
                gbmeta = False
                if url.endswith(".gbmeta.xml") : url = url[:len(url)-4]; gbmeta = True
                if gbmeta :
                    index = dialog.listXslt.index("gbmeta.xsl") if "gbmeta.xsl" in (dialog.listXslt) else 0
                    dialog.primaryXslt.currentIndexChanged[str].disconnect(dialog.reloadUrl)
                    dialog.primaryXslt.setCurrentIndex(index)
                    dialog.primaryXslt.currentIndexChanged[str].connect(dialog.reloadUrl)

                else :
                    index = dialog.listXslt.index("gbmeta.xsl")
                    if dialog.primaryXslt.currentText() == "gbmeta.xsl" :
                       index = dialog.listXslt.index("transformation_fr.xsl") if "transformation_fr.xsl" in (dialog.listXslt) else 0 
                       dialog.primaryXslt.currentIndexChanged[str].disconnect(dialog.reloadUrl)
                       dialog.primaryXslt.setCurrentIndex(index)
                       dialog.primaryXslt.currentIndexChanged[str].connect(dialog.reloadUrl)
                    
                
                baseUrl = ""
                qry = QXmlQuery(QXmlQuery.XSLT20)
                test, xsl = fileRessourceExist(dialog, "xml/xsl/%s" % (dialog.primaryXslt.currentText()))
                if xsl!= "" : baseUrl = QUrl.fromLocalFile(xsl)
                qry.setFocus(QUrl.fromLocalFile(url)) 
                qry.setQuery(QUrl.fromLocalFile(xsl)) 
                html = qry.evaluateToString()

                if html is None:
                   test, xsl = fileRessourceExist(dialog, "xml/xsl/%s" % (dialog.secondaryXslt.currentText()))
                   if xsl!= "" : baseUrl = QUrl.fromLocalFile(xsl)
                   qry.setFocus(QUrl.fromLocalFile(url))
                   qry.setQuery(QUrl.fromLocalFile(xsl)) 
                   html = qry.evaluateToString()
                   if html is None :
                      html = QApplication.translate("QSphere", "File parsing XSLT error !<br>Invalid XSLT or XML file !")
                      html = "<div align='center'><img src='file:///%s'><br><br>%s</div>" % (getThemeIcon("parsing_error.gif"), html)
                      dialog.primaryXslt.setStyleSheet("""QComboBox {background: red;}""")
                      dialog.secondaryXslt.setStyleSheet("""QComboBox {background: red;}""")
                   else :
                      html = cleanHtml(html)
                      dialog.primaryXslt.setStyleSheet("""QComboBox {background: red;}""")
                      dialog.secondaryXslt.setStyleSheet("""QComboBox {background: green;}""")
                      
                else:
                    html = cleanHtml(html)
                    dialog.primaryXslt.setStyleSheet("QComboBox {background: green;}")
                    dialog.secondaryXslt.setStyleSheet("QComboBox {background: white;}")

                isXml = True
                webViewer.setHtml(html, baseUrl) 
                dialog.html = html
                dialog.oldurl = correctPath(dirname(baseUrl.toString()))
                
            elif dialog.fileExtension.lower() == ".qss" :
                isXml = True
                qry = QXmlQuery(QXmlQuery.XSLT20)
                test, xsl = fileRessourceExist(dialog, "xml/xsl/generic_parser_table0.xsl")
                html = None
                if xsl != "" :
                   baseUrl = QUrl.fromLocalFile(xsl)
                   qry.setFocus(QUrl.fromLocalFile(url))
                   qry.setQuery(QUrl.fromLocalFile(xsl)) 
                   html = qry.evaluateToString()
                   index = dialog.listXslt.index("generic_parser_table0.xsl")
                   dialog.primaryXslt.setCurrentIndex(index)
                   
                if html is None :
                    html = QApplication.translate("QSphere", "File parsing XSLT error !<br>Invalid XSLT or XML file !")
                    html = "<div align='center'><img src='file:///%s'><br><br>%s</div>" % (getThemeIcon("parsing_error.gif"), html)
                    dialog.primaryXslt.setStyleSheet("""QComboBox {background: red;}""")
                    dialog.secondaryXslt.setStyleSheet("""QComboBox {background: red;}""")                    
                else :
                    webViewer.setHtml(html, baseUrl)
                    dialog.html = html
                    dialog.oldurl = correctPath(dirname(baseUrl.toString()))
                    dialog.primaryXslt.setStyleSheet("""QComboBox {background: green;}""")
                    dialog.secondaryXslt.setStyleSheet("""QComboBox {background: red;}""")
                

        elif url == "#blank" :
            if dialog.eUrl.findText(url) == -1 : dialog.eUrl.addItem(url)
            baseUrl = QUrl.fromLocalFile(dirname(__file__))
            webViewer.setHtml("", baseUrl)
            dialog.eUrl.setStyleSheet("QComboBox {background-color:#AEEE00;}"
                          "QComboBox QAbstractItemView {background-color:#AEEE00; min-width:%spx;}" % (dialog.minWidth)
                          )
               
        else :

            if dialog.eUrl.findText(url)==-1 : dialog.eUrl.addItem(url)

            if QUrl(url).isValid() :
               inputFile = None
               inputFileUrl = unquote(url)
               queryTest = False
               if exists(inputFileUrl) :
                   inputFile = QFileInfo(inputFileUrl)
                   suffixe = inputFile.suffix().lower()
                   test = None
                   test = isRasterFile(inputFileUrl)
                  
                   if (suffixe not in ("html", "htm", "qsp", "qsr", "csv") and not test ) or (suffixe in ("ecw", "jp2", "view") and test):
                       baseUrl = QUrl.fromLocalFile(inputFileUrl)
                       html = fileInfosToHtml(dialog, inputFile)
                       webViewer.setHtml(html, baseUrl)
                       webViewer.toOpenUrl = inputFileUrl 
                       return
                  
               else : 
                   elts, urlQuery, queryTest = url.split("/"), [], False
                   urlQuery = elts[-1].split("=")
                   if urlQuery != [] : 
                      if urlQuery[0] in dialog.listUrlQueries : queryTest = True
                   
               if queryTest :                   
                   webViewer.telech = telecharge(QUrl(url), dialog, None, "*.xml", dialog.duration_info, dialog.duration_warning, False) 
                   webViewer.telech.finTelecharge.connect(webViewer.telecharge_ok) 
               else :
                       dialog.eUrl.setStyleSheet("QComboBox {background-color:#AEEE00;}"
                                  "QComboBox QAbstractItemView {backgrMakeWindowIconound-color:#AEEE00; min-width:%spx;}" % (dialog.minWidth)
                                  )
                       if inputFile != None :
                           if suffixe in ("qsp", "qsr") :
                              baseUrl = QUrl.fromLocalFile(url)

                              with open(inputFileUrl, "r") as myfile : myfileText = myfile.read()
                             
                              html = ""
                              for k in range(len(myfileText)): html+= "%s" % (markerText(dialog, myfileText[k]))
                              webViewer.setHtml(html, baseUrl)
                           else :
                              webViewer.load(QUrl(url))
                       else :
                           try : webViewer.load(QUrl(url))
                           except : pass
                           
            else :
               title = QApplication.translate("QSphere","Information")
               msg = QApplication.translate("QSphere","Invalid URL ! Use open function if necessary for local resources ...")
               sendMsg(dialog, title , msg, Qgis.Warning, dialog.duration_warning)


        if not isXml : dialog.oldurl = ""
        fixeEnabledHistory(dialog, webViewer)    
        page = webViewer.page()
        page.setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        windowIcon(dialog, dialog.iconName)       


class telecharge(QNetworkAccessManager):
    finTelecharge = pyqtSignal(bool)
    def __init__(self, url, dialog, fileName, typeMime, duration_info, duration_warning, silentmode) : 
        QNetworkAccessManager.__init__(self)
        self.initDir = correctPath(dirname(__file__))
        self.dialog = dialog
        self.messageBuffer = QByteArray()
        self.fileName, self.typeMime = fileName, typeMime
        self.duration_info = duration_info
        self.duration_warning = duration_warning
        self.silentmode = silentmode
        self.reply = self.get(QNetworkRequest(url))
        self.reply.readyRead.connect(self.readData)
        self.reply.finished.connect(self.finished)
        self.url = url
        self.dialog.movieCtrl.startMovie()
        self.dialog.labelResultFiles.setVisible(True)

    @pyqtSlot()
    def readData(self):
        self.messageBuffer+= self.reply.readAll()
        if not self.silentmode :
            self.dialog.labelResultFiles.setText("%s" % convertSize(len(self.messageBuffer))) #msg)
            self.dialog.labelResultFiles.repaint()

 
    @pyqtSlot()
    def finished(self):
        self.dialog.barInfo.clearWidgets()
        self.dialog.barInfo.setVisible(False)
        self.dialog.movieCtrl.stopMovie()
        self.dialog.labelResultFiles.setVisible(False)
        
        filterMime = "*.*" if self.typeMime == None else self.typeMime
        initDir = self.initDir if self.fileName == None else "%s%s" % (self.initDir, self.fileName)

        if not filterMime in ("*.htm", "*.xml", "*.html") :
           if filterMime == "*.*" :
               typemime, filterMime = "%s" % (self.reply.header(0)), "*.*"
               if typemime.find(";")!=-1 :
                   typemime = typemime.rsplit(";",1)[0]
                   if typemime.find("/")!=-1 :
                      filterMime = "*.%s" % (typemime.rsplit("/",1)[1].lower()) 
           self.saveFile(filterMime)
           return

        if filterMime == "*.*" :
           typemime, filterMime = "%s" % (self.reply.header(0)), "*.*"
           if typemime.find(";")!=-1 :
               typemime = typemime.rsplit(";",1)[0]
               if typemime.find("/")!=-1 :
                  filterMime = "*.%s" % (typemime.rsplit("/",1)[1].lower())


        if filterMime == "*.xml" :

            test, dataXml = testXmlData(self.dialog, self.messageBuffer, True) 

            if test :        
                   isoParser = xmlIsoParser(None, StringIO(dataXml), 'MEDDE', 'fr') 
                   isoParser.createISOdataStructure(True)

                   if isoParser.getTagDictionnary() :
                               
                               try : objRef = self.dialog.parent
                               except : objRef = None
                               if not objRef : return

                               from qsphere.ui import dlgMetadata
                               from qsphere.qsphere_tools import getThemeIcon
                               
                               title = QApplication.translate("QSphere","Request for confirmation")
                               msg = QApplication.translate("QSphere","Directly Load Data in Main Window ?")
                               msgDlgBox = QMessageBox(self.dialog)

                               msgDlgBox.setWindowTitle(title)
                               windowIcon(msgDlgBox, "navigatorweb.png")
                               msgDlgBox.setIcon(QMessageBox.Question)
                               

                               groupBox1 = QGroupBox(msgDlgBox)
                               groupBox1.setFixedSize(420, 290)
                               groupBox1.setObjectName("groupBox1")

                               cond, child = False, None
                               from qsphere.ui_mainwindow import myMdiWindow
                               if type(objRef)== myMdiWindow :
                                   try : objRef = objRef.parent()
                                   except :
                                        try : objRef = objRef.parent
                                        except : return
                               
                               if objRef.mainWindow :
                                  dialog, child = objRef.mainWindow.metadataWindow()
                                  if type(child)==dlgMetadata : cond = True
                               
                               labelImage = QLabel(groupBox1)
                               myImgPath = getThemeIcon("qsphere_choice.png")
                               icon = QImage(myImgPath)
                               labelImage.setPixmap(QPixmap.fromImage(icon))
                               labelImage.setGeometry(145, 10, 128, 108)

  
                               radioButton1 = QRadioButton(groupBox1)
                               radioButton1.setObjectName("radioButton1")
                               radioButton1.setFixedSize(420, 25)
                               radioButton1.setGeometry(5, 115, 420, 25)
                               radioButton1.setText("%s" % (QApplication.translate("QSphere","Directly Load Data in Main Window ?")))
                               radioButton1.setEnabled(cond)
                               radioButton1.setChecked(False)

                               model = QStandardItemModel()
                               view = QTableView()
                               view.setModel(model)
                               view.horizontalHeader().setVisible(False)
                               view.verticalHeader().setVisible(False)
                               view.setColumnHidden(1, True)
                               view.horizontalHeader().setDefaultSectionSize(500)
                               view.verticalHeader().setDefaultSectionSize(20)
                               view.setStyleSheet("QTableView {min-width: 500px; max-width: 600px; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}")
                               view.setShowGrid(False)

                               comboMdiWindows = QComboBox(groupBox1)
                               comboMdiWindows.setObjectName("comboMdiWindows")
                               comboMdiWindows.setFixedSize(380, 25)
                               comboMdiWindows.setGeometry(5, 150, 380, 25)
                               comboMdiWindows.setEnabled(cond)
                               comboMdiWindows.setModel(model)
                               comboMdiWindows.setView(view)

                               if objRef.mainWindow :  self.updateListWindowsButtons(objRef.mainWindow, comboMdiWindows, model)

                               radioButton2 = QRadioButton(groupBox1)
                               radioButton2.setObjectName("radioButton2")
                               radioButton2.setFixedSize(420, 25)
                               radioButton2.setGeometry(5, 185, 420, 25)
                               radioButton2.setText("%s" % (QApplication.translate("QSphere","Open a new Main Window ?")))
                               radioButton2.setChecked(True)

                               radioButton3 = QRadioButton(groupBox1)
                               radioButton3.setObjectName("radioButton3")
                               radioButton3.setFixedSize(420, 25)
                               radioButton3.setGeometry(5, 215, 420, 25)
                               radioButton3.setText("%s" % (QApplication.translate("QSphere","Save file as ..."))) 

                               radioButton4 = QRadioButton(groupBox1)
                               radioButton4.setObjectName("radioButton4")
                               radioButton4.setFixedSize(420, 25)
                               radioButton4.setGeometry(5, 245, 420, 25)
                               radioButton4.setText("%s" % (QApplication.translate("QSphere", "Open link in my default navigator"))) 
    
                               buttonAccept = QPushButton(msgDlgBox)
                               buttonAccept.setObjectName("savePage")
                               buttonAccept.setText(QApplication.translate("QSphere", "Apply"))
                               buttonAccept.setFixedSize(100, 25)

                               buttonReject = QPushButton(msgDlgBox)
                               buttonReject.setObjectName("CloseButton")
                               buttonReject.setText(QApplication.translate("QSphere", "Close"))
                               buttonReject.setFixedSize(100, 25)

                               buttonAccept.clicked.connect(msgDlgBox.accept)
                               buttonReject.clicked.connect(msgDlgBox.reject)
                               

                               msgDlgBox.layout().addWidget(groupBox1,0,1,1,2)
                               msgDlgBox.layout().addWidget(buttonAccept,4,1,1,1)
                               msgDlgBox.layout().addWidget(buttonReject,4,2,1,1)

                               msgDlgBox.setStandardButtons(msgDlgBox.NoButton)

                               from qsphere.ui import dlgMetadata
                              
                               result = False
                               while True :
                                   ret = msgDlgBox.exec_()
                                   if ret == 1 :

                                      if radioButton1.isChecked() :
                                           objRef = self.dialog.parent
                                                                                   
                                           if objRef :
                                              subWindow = None
                                              child = None
                                              oldTitle = ""
                                              
                                              modelIndex = comboMdiWindows.model().index(comboMdiWindows.currentIndex(),0)
                                              winId = long(comboMdiWindows.model().data(modelIndex, Qt.UserRole))
                                              for subWindow in objRef.mdi.subWindowList(QMdiArea.CreationOrder) :
                                                  if winId == long(subWindow.winId()) : 
                                                        oldTitle = subWindow.windowTitle()
                                                        for thechild in subWindow.children():
                                                            if type(thechild) == dlgMetadata : child = thechild ; break ;
                                                        break
                                                
                                           if child :
                                              result = child.reloadDataFromXml(isoParser, None, 0) 
                                              if subWindow : subWindow.setWindowTitle(oldTitle)
                                              try :
                                                  if objRef.dlgSgbdr != None : objRef.dlgSgbdr.changedMDDFiles = True; objRef.dlgSgbdr.loadListMDDFiles();
                                              except : pass
                                      
                                      elif radioButton2.isChecked() :
                                                
                                           objRef = self.dialog.parent
                                           if not objRef : return
                                          
                                           if type(objRef) != myMdiWindow :
                                               if not objRef.mainWindow :
                                                   from qsphere.ui_mainwindow import myMdiWindow
                                                   objRef.mainWindow = myMdiWindow(objRef.qspherePlugin)
                                                   objRef.mainWindow.setWindowState(Qt.WindowMaximized)
                                                   objRef.mainWindow.show()
                                                   mdiWindow, child =  objRef.mainWindow.newMetadataFile()
                                                   self.dialog.parent = objRef.mainWindow
                                               else :
                                                   objRef.show()
                                                   objRef.activateWindow()
                                                   mdiWindow, child =  objRef.newMetadataFile()
                                                   self.dialog.parent = objRef
                                           else :
                                                   objRef.show()
                                                   objRef.activateWindow()
                                                   mdiWindow, child =  objRef.newMetadataFile()
                                                   self.dialog.parent = objRef
                                           
                                           if mdiWindow != None :
                                              title =  mdiWindow.windowTitle()
                                              result = child.reloadDataFromXml(isoParser, None, 0) 
                                              mdiWindow.setWindowTitle(title)
                                              try :
                                                  if objRef.dlgSgbdr != None : objRef.dlgSgbdr.changedMDDFiles = True; objRef.dlgSgbdr.loadListMDDFiles();
                                              except : pass 

                                              radioButton1.setEnabled(True)
                                              comboMdiWindows.setEnabled(True)

                                              if type(objRef) != myMdiWindow : objRef = objRef.mainWindow
                                              self.updateListWindowsButtons(objRef, comboMdiWindows, model)
                                           
                                      elif radioButton3.isChecked() :
                                           cond, self.fileName = False, "*.xml"
                                           try :
                                               self.fileName = "%s.xml" % (isoParser.UUID[0][0]) if isoParser.UUID[0][0]!="" else "*.xml"
                                           except :
                                               for child in root :
                                                   if "fileIdentifier" in child.tag :
                                                       for elVal in child:
                                                           self.fileName = "%s.xml" % (elVal.text)
                                                           cond = True
                                                   if cond : break
                                           finally : pass
                                           self.saveFile(filterMime)

                                      elif radioButton4.isChecked() :
                                           self.dialog.mainTabWidget.currentWidget().viewHtml.toOpenUrl = self.reply.url().toString()
                                           self.dialog.mainTabWidget.currentWidget().viewHtml.makeOpenUrl()                                           

                                   else : return

    def updateListWindowsButtons(self, obj, comboMdiWindows, model):
          if obj:
              comboMdiWindows.clear()
              for subWindows in obj.mdi.subWindowList(QMdiArea.CreationOrder) :
                  item = QStandardItem(subWindows.windowTitle())
                  item.setData("%s" % long(subWindows.winId()), Qt.UserRole)
                  model.appendRow(item) 
              
        
    def saveFile(self, filterMime):

       if len(self.messageBuffer) > 0 :

          title = QApplication.translate("QSphere","Save file as ...")
          dlgFile = QFileDialog(self.dialog, title)

          if filterMime!= "*.*" : dlgFile.setDefaultSuffix(filterMime.split(".")[1])
          dlgFile.setNameFilters((filterMime,))
          dlgFile.selectNameFilter(filterMime)
          dlgFile.selectFile(self.fileName.replace(":", "-"))
          dlgFile.setViewMode(QFileDialog.Detail)
          dlgFile.setDirectory(self.initDir)
          dlgFile.setAcceptMode(QFileDialog.AcceptSave)

          fixeLabelsFileDialog(self, dlgFile, 1, True)
          
          if dlgFile.exec_():
                fileName = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())          
                self.initDir = dirname(fileName)
                self.fileName = basename(fileName) 
                saveFile = QFile(fileName)

                title = QApplication.translate("QSphere","Information")
                if (saveFile.open(QIODevice.WriteOnly)):
                    saveFile.write(self.messageBuffer)
                    saveFile.close()
                    self.finTelecharge.emit(True)
                else:
                    self.finTelecharge.emit(False)

       elif len(self.messageBuffer) == 0 :       
            
            title = QApplication.translate("QSphere","Warning")
            msg = "%s :\n%s\n%s ?" % (QApplication.translate("QSphere","Some troubleshooting to download the file "), self.url.toString(),
                                      QApplication.translate("QSphere", "Open link in my default navigator"))
            msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, title, msg, "qsphere_stop.png", None, self.dialog.parent)
            windowIcon(msgDlgBox , "globe.png")
            ret = msgDlgBox.exec_()
            if ret == 1 :
               try : webbrowseropen(unicode(self.url.toString()))
               except : pass
            

